@@ -48,6 +48,8 @@ String[] booleanOption={
 "-inroot ",
 "-timeelapsed ",
 "-install ",
+"-wireframe ",
+"-showfps ",
 "-use3d "};
 Checkbox bopt[]=new Checkbox[15];
  public void init()
@@ -93,6 +95,7 @@ lst.addItem("decay");
 lst.addItem("deco");
 lst.addItem("demon");
 lst.addItem("dilemma");
+lst.addItem("dragon");
 lst.addItem("discrete");
 lst.addItem("drift");
 lst.addItem("euler2d");
@@ -105,6 +108,7 @@ lst.addItem("flow");
 lst.addItem("forest");
 lst.addItem("galaxy");
 lst.addItem("gears");
+lst.addItem("glplanet");
 lst.addItem("goop");
 lst.addItem("grav");
 lst.addItem("helix");
@@ -139,9 +143,9 @@ lst.addItem("munch");
 lst.addItem("nose");
 lst.addItem("pacman");
 lst.addItem("penrose");
-lst.addItem("pentominoes");
 lst.addItem("petal");
 lst.addItem("pipes");
+lst.addItem("polyominoes");
 lst.addItem("puzzle");
 lst.addItem("pyro");
 lst.addItem("qix");
@@ -149,8 +153,10 @@ lst.addItem("roll");
 lst.addItem("rotor");
 lst.addItem("rubik");
 lst.addItem("sballs");
+lst.addItem("scooter");
 lst.addItem("shape");
 lst.addItem("sierpinski");
+lst.addItem("sierpinski3d");
 lst.addItem("skewb");
 lst.addItem("slip");
 lst.addItem("solitare");
