@@ -190,7 +190,7 @@ init_decay(ModeInfo * mi)
 	Window window = MI_WINDOW(mi);
 	decaystruct *dp;
 
-	char *s = "random";
+	char *s = (char*) "random";
 
 	if (decay_info == NULL) {
 		if ((decay_info = (decaystruct *) calloc(MI_NUM_SCREENS(mi),
