@@ -705,7 +705,7 @@ init_dilemma(ModeInfo * mi)
 		dp->ncols = nccols / 2;
 		dp->nrows = 2 * (ncrows / 4);
 		dp->xb = (dp->width - dp->xs * nccols) / 2 + dp->xs / 2;
-		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 + dp->ys;
+		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 + dp->ys - 2;
 		for (sides = 0; sides < 6; sides++) {
 			dp->shape.hexagon[sides].x = (dp->xs - 1) * hexagonUnit[sides].x;
 			dp->shape.hexagon[sides].y =
