@@ -2,7 +2,7 @@
 /* cage --- the Impossible Cage, an Escher like scene. */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)cage.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
@@ -44,6 +44,7 @@ static const char sccsid[] = "@(#)cage.c	5.00 2000/11/01 xlockmore";
  * Marcelo F. Vianna (Jun-01-1997)
  *
  * Revision History:
+ * 01-Mar-2001: Added FPS stuff E.Lassauge <lassauge@mail.dotcom.fr>
  * 01-Nov-2000: Allocation checks
  * 01-Jan-1998: Mode separated from escher and renamed
  * 08-Jun-1997: New scene implemented: "Impossible Cage" based in a M.C.
@@ -83,12 +84,13 @@ static const char sccsid[] = "@(#)cage.c	5.00 2000/11/01 xlockmore";
 #define HACK_INIT init_cage
 #define HACK_DRAW draw_cage
 #define cage_opts xlockmore_opts
-#define DEFAULTS "*delay: 1000 \n" \
+#define DEFAULTS "*delay: 25000 \n" \
+ "*showFps  : False \n" \
  "*wireframe: False \n"
 #include "xlockmore.h"		/* from the xscreensaver distribution */
 #else /* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+#include "visgl.h"
 
 #endif /* !STANDALONE */
 
@@ -105,7 +107,7 @@ ModeSpecOpt cage_opts =
 ModStruct   cage_description =
 {"cage", "init_cage", "draw_cage", "release_cage",
  "draw_cage", "change_cage", NULL, &cage_opts,
- 1000, 1, 1, 1, 1.0, 4, "",
+ 25000, 1, 1, 1, 1.0, 4, "",
  "Shows the Impossible Cage, an Escher-like GL scene", 0, NULL};
 
 #endif
@@ -152,7 +154,7 @@ static float MaterialWhite[] =
 {0.7, 0.7, 0.7, 1.0};
 
 static cagestruct *cage = NULL;
-GLuint objects = 0;
+static GLuint objects = 0;
 
 #define PlankWidth      3.0
 #define PlankHeight     0.35
@@ -370,6 +372,29 @@ pinit(ModeInfo *mi)
 	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_specular);
 }
 
+void
+release_cage(ModeInfo * mi)
+{
+	if (cage != NULL) {
+		int screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			cagestruct *cp = &cage[screen];
+
+			if (cp->glx_context) {
+				if (glIsList(objects)) {
+					glDeleteLists(objects, MaxObj);
+					objects = 0;
+				}
+				cp->glx_context = NULL;
+			}
+		}
+		(void) free((void *) cage);
+		cage = NULL;
+	}
+	FreeAllGL(mi);
+}
+
 void
 init_cage(ModeInfo * mi)
 {
@@ -438,7 +463,7 @@ draw_cage(ModeInfo * mi)
 	}
 
 	glPopMatrix();
-
+	if (MI_IS_FPS(mi)) do_fps (mi);
 	glFlush();
 
 	glXSwapBuffers(display, window);
@@ -458,27 +483,4 @@ change_cage(ModeInfo * mi)
 	pinit(mi);
 }
 
-void
-release_cage(ModeInfo * mi)
-{
-	if (cage != NULL) {
-		int screen;
-
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			cagestruct *cp = &cage[screen];
-
-			if (cp->glx_context) {
-				if (glIsList(objects)) {
-					glDeleteLists(objects, MaxObj);
-					objects = 0;
-				}
-				cp->glx_context = NULL;
-			}
-		}
-		(void) free((void *) cage);
-		cage = NULL;
-	}
-	FreeAllGL(mi);
-}
-
 #endif
