@@ -1,11 +1,11 @@
 /* -*- Mode: C; tab-width: 4 -*- */
-/* fire --- 3D fire-like image */
+/* fire --- 3D fire or rain landscape */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)fire.c	5.00 2000/11/28 xlockmore";
+static const char sccsid[] = "@(#)fire.c	5.01 2001/03/09 xlockmore";
 #endif
 
-/* Copyright (c) E. Lassauge, 2000. */
+/* Copyright (c) E. Lassauge, 2001. */
 
 /*
  * Permission to use, copy, modify, and distribute this software and its
@@ -20,15 +20,22 @@ static const char sccsid[] = "@(#)fire.c	5.00 2000/11/28 xlockmore";
  * event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
  *
- * The original code for this mode was written by 
- * David Bucciarelli (tech.hmw@plus.it) and could be found in the demo package 
- * of Mesa (Mesa-3.2/3Dfx/demos/).
+ * The original code for this mode was written by David Bucciarelli 
+ * (tech.hmw@plus.it) and could be found in the demo package 
+ * of Mesa (Mesa-3.2/3Dfx/demos/). This mode is the result of the merge of
+ * two of the David's demos (fire and rain).
  *
  * Eric Lassauge  (October-10-2000) <lassauge@mail.dotcom.fr>
  * 				    http://lassauge.free.fr/linux.html
  *
  * REVISION HISTORY:
  *
+ * E.Lassauge - 09-Mar-2001:
+ *      - get rid of my framerate options to use showfps
+ *
+ * E.Lassauge - 12-Jan-2001:
+ *      - add rain particules, selected if count=0 (no fire means rain !)
+ *
  * E.Lassauge - 28-Nov-2000:
  *      - modified release part to add freeing of GL objects
  *
@@ -73,13 +80,14 @@ static const char sccsid[] = "@(#)fire.c	5.00 2000/11/28 xlockmore";
  "*count: 800 \n" \
  "*size: 0 \n" \
  "*trees: 5 \n" \
+ "*showfps: False \n" \
  "*trackmouse: False \n" \
  "*wireframe:  False \n"	/* still needs some work */
 
 #include "xlockmore.h"		/* from the xscreensaver distribution */
 #else				/* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+#include "visgl.h"
 #endif				/* !STANDALONE */
 #include "iostuff.h"		/* getFont() */
 
@@ -151,7 +159,6 @@ static const char sccsid[] = "@(#)fire.c	5.00 2000/11/28 xlockmore";
 #define MAX_TREES	20
 static Bool do_texture;
 static Bool do_fog;
-static Bool do_framerate;
 static Bool do_shadows;
 static Bool do_trackmouse;
 static int  num_trees;
@@ -162,8 +169,6 @@ static XrmOptionDescRec opts[] = {
     {(char *) "+texture", (char *) ".fire.texture", XrmoptionNoArg, (caddr_t) "off"},
     {(char *) "-fog", (char *) ".fire.fog", XrmoptionNoArg, (caddr_t) "on"},
     {(char *) "+fog", (char *) ".fire.fog", XrmoptionNoArg, (caddr_t) "off"},
-    {(char *) "-framerate", (char *) ".fire.framerate", XrmoptionNoArg, (caddr_t) "on"},
-    {(char *) "+framerate", (char *) ".fire.framerate", XrmoptionNoArg, (caddr_t) "off"},
     {(char *) "-shadows", (char *) ".fire.shadows", XrmoptionNoArg, (caddr_t) "on"},
     {(char *) "+shadows", (char *) ".fire.shadows", XrmoptionNoArg, (caddr_t) "off"},
     {(char *) "-trackmouse", (char *) ".fire.trackmouse", XrmoptionNoArg, (caddr_t) "on"},
@@ -175,7 +180,6 @@ static XrmOptionDescRec opts[] = {
 static argtype vars[] = {
     {(caddr_t *) & do_texture, (char *) "texture", (char *) "Texture", (char *) DEF_TEXTURE, t_Bool},
     {(caddr_t *) & do_fog, (char *) "fog", (char *) "Fog", (char *) DEF_FOG, t_Bool},
-    {(caddr_t *) & do_framerate, (char *) "framerate", (char *) "Framerate", (char *) DEF_FRAMERATE, t_Bool},
     {(caddr_t *) & do_shadows, (char *) "shadows", (char *) "Shadows", (char *) DEF_SHADOWS, t_Bool},
     {(caddr_t *) & do_trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool},
     {(caddr_t *) & num_trees, (char *) "trees", (char *) "Trees", (char *) DEF_TREES, t_Int},
@@ -184,7 +188,6 @@ static argtype vars[] = {
 static OptionStruct desc[] = {
     {(char *) "-/+texture", (char *) "turn on/off texturing"},
     {(char *) "-/+fog", (char *) "turn on/off fog"},
-    {(char *) "-/+framerate", (char *) "turn on/off framerate display"},
     {(char *) "-/+shadows", (char *) "turn on/off shadows"},
     {(char *) "-/+trackmouse", (char *) "turn on/off the tracking of the mouse"},
     {(char *) "-trees num", (char *) "number of trees (0 disables)"},
@@ -213,7 +216,9 @@ ModStruct fire_description =
 
 #define AGRAV 		-9.8	/* gravity */
 
-/* particle struct */
+#define NUMPART		7500	/* rain particles */
+
+/* fire particle struct */
 typedef struct {
     int age;
     float p[3][3];
@@ -221,6 +226,16 @@ typedef struct {
     float c[3][4];
 } part;
 
+/* rain particle struct */
+typedef struct {
+    float age;
+    float acc[3];
+    float vel[3];
+    float pos[3];
+    float partLength;
+    float oldpos[3];
+} rain;
+
 /* colors */
 static float black[3]    = { 0.0, 0.0, 0.0 }; /* shadow color */
 static float partcol1[3] = { 1.0, 0.2, 0.0 }; /* initial color: red-ish */
@@ -259,15 +274,15 @@ typedef struct {
     GLint WIDTH, HEIGHT;	/* display dimensions */
     GLXContext *glx_context;
 
-    int np;			/* number of particles : set it through 'count' resource */
+    int np;			/* number of fire particles : set it through 'count' resource */
     float eject_r;		/* emission radius */
     float dt, maxage, eject_vy, eject_vl;
-    float ridtri;		/* particle size */
+    float ridtri;		/* fire particle size */
     Bool shadows;		/* misc booleans: set them through specific resources */
     Bool fog;
-    Bool framerate;
 
-    part *p;			/* particle array */
+    part *p;			/* fire particles array */
+    rain *r;			/* rain particles array */
 
     XImage *gtexture;		/* ground texture image bits */
     XImage *ttexture;		/* tree texture image bits */
@@ -278,6 +293,9 @@ typedef struct {
     int   num_trees;		/* number of trees: set it through 'trees' resource */
     treestruct *treepos;	/* trees positions: float treepos[num_trees][3] */
 
+    float min[3];		/* raining area */
+    float max[3];
+
     float obs[3];		/* observer coordinates */
     float dir[3];		/* view direction */
     float v;			/* observer velocity */
@@ -300,18 +318,19 @@ static firestruct *fire = NULL;
  *-----------------------------------------------------------------------------
  */
 
-/* utility function for the framerate display */
-static float gettime(firestruct * fs)
+/* utility function for the rain particles */
+static float gettimerain()
 {
-    clock_t tnew, ris;
+  static clock_t told=0;
+  clock_t tnew,ris;
 
-    tnew = clock();
+  tnew=clock();
 
-    ris = tnew - fs->told;
+  ris=tnew-told;
 
-    fs->told = tnew;
+  told=tnew;
 
-    return (ris / (float) CLOCKS_PER_SEC);
+  return (ris/(float)CLOCKS_PER_SEC);
 }
 
 /* my RAND */
@@ -320,7 +339,7 @@ static float vrnd(void)
     return ((float) LRAND() / (float) MAXRAND);
 }
 
-/* initialise new particle */
+/* initialise new fire particle */
 static void setnewpart(firestruct * fs, part * p)
 {
     float a, vi[3], *c;
@@ -351,7 +370,27 @@ static void setnewpart(firestruct * fs, part * p)
 	   c[2] * ((1.0 - RIDCOL) + vrnd() * RIDCOL), 1.0);
 }
 
-/* set particle values */
+/* initialise new rain particle */
+static void setnewrain(firestruct * fs, rain * r)
+{
+    r->age=0.0f;
+
+    vinit(r->acc,0.0f,-0.98f,0.0f);
+    vinit(r->vel,0.0f,0.0f,0.0f);
+    
+    r->partLength=0.2f;
+
+    vinit(r->oldpos,fs->min[0]+(fs->max[0]-fs->min[0])*vrnd(),
+                    fs->max[1]+0.2f*fs->max[1]*vrnd(),
+                    fs->min[2]+(fs->max[2]-fs->min[2])*vrnd());
+    vequ(r->pos,r->oldpos);
+    vadds(r->oldpos,-(r->partLength),r->vel);
+
+    r->pos[1]=(fs->max[1]-fs->min[1])*vrnd()+fs->min[1];
+    r->oldpos[1]=r->pos[1]-r->partLength*r->vel[1];
+}
+
+/* set fire particle values */
 static void setpart(firestruct * fs, part * p)
 {
     float fact;
@@ -389,6 +428,30 @@ static void setpart(firestruct * fs, part * p)
     }
 }
 
+/* set rain particle values */
+static void setpartrain(firestruct * fs, rain * r, float dt)
+{
+    r->age += dt;
+
+    vadds(r->vel,dt,r->acc);
+    vadds(r->pos,dt,r->vel);
+
+    if(r->pos[0]<fs->min[0])
+	r->pos[0]=fs->max[0]-(fs->min[0]-r->pos[0]);
+    if(r->pos[2]<fs->min[2])
+	r->pos[2]=fs->max[2]-(fs->min[2]-r->pos[2]);
+
+    if(r->pos[0]>fs->max[0])
+	r->pos[0]=fs->min[0]+(r->pos[0]-fs->max[0]);
+    if(r->pos[2]>fs->max[2])
+	r->pos[2]=fs->min[2]+(r->pos[2]-fs->max[2]);
+
+    vequ(r->oldpos,r->pos);
+    vadds(r->oldpos,-(r->partLength),r->vel);
+    if(r->pos[1]<fs->min[1])
+    	setnewrain(fs, r);
+}
+
 /* draw a tree */
 static void drawtree(float x, float y, float z)
 {
@@ -433,16 +496,12 @@ static void calcposobs(firestruct * fs)
     fs->obs[0] += fs->v * fs->dir[0];
     fs->obs[1] += fs->v * fs->dir[1];
     fs->obs[2] += fs->v * fs->dir[2];
-}
 
-/* utility function for string printing for framerate */
-static void printstring(GLuint fontbase, char *string)
-{
-    int len;
-
-    len = (int) strlen(string);
-    glListBase(fontbase);
-    glCallLists(len, GL_UNSIGNED_BYTE, (GLubyte *) string);
+    if (!fs->np)
+    {
+    	vinit(fs->min,fs->obs[0]-7.0f,-0.2f,fs->obs[2]-7.0f);
+    	vinit(fs->max,fs->obs[0]+7.0f,8.0f,fs->obs[2]+7.0f);
+    }
 }
 
 /* track the mouse in a joystick manner : not perfect but it works */
@@ -612,7 +671,6 @@ static void Reshape(ModeInfo * mi)
 
 static void DrawFire(ModeInfo * mi)
 {
-    float fr;
     int j;
     firestruct *fs = &fire[MI_SCREEN(mi)];
     Bool wire = MI_IS_WIREFRAME(mi);
@@ -706,11 +764,22 @@ static void DrawFire(ModeInfo * mi)
     }
     glEnd();
 
-    /* manage framerate value */
-    if (fs->framerate && fs->frcount >= FRAME) {
-        fs->frcount = 0;
-	fr = gettime(fs);
-	(void) sprintf(fs->frbuf, "Frame rate: %f", FRAME / fr);
+    /* draw rain particles if no fire particles */
+    if (!fs->np)
+    {
+        float timeused = gettimerain();
+        glDisable(GL_TEXTURE_2D);
+	glShadeModel(GL_SMOOTH);
+	glBegin(GL_LINES);
+	for (j = 0; j < NUMPART; j++) {
+	    glColor4f(0.7f,0.95f,1.0f,0.0f);
+	    glVertex3fv(fs->r[j].oldpos);
+	    glColor4f(0.3f,0.7f,1.0f,1.0f);
+	    glVertex3fv(fs->r[j].pos);
+	    setpartrain(fs, &fs->r[j],timeused);
+	}
+	glEnd();
+	glShadeModel(GL_FLAT);
     }
 
     glDisable(GL_TEXTURE_2D);
@@ -719,19 +788,7 @@ static void DrawFire(ModeInfo * mi)
     glDisable(GL_FOG);
 
     /* manage framerate display */
-    if (fs->framerate) {
-    	glMatrixMode(GL_PROJECTION);
-    	glLoadIdentity();
-    	glOrtho(-0.5, 639.5, -0.5, 479.5, -1.0, 1.0);
-    	glMatrixMode(GL_MODELVIEW);
-    	glLoadIdentity();
-
-	/* draw framerate (red text) */
-	glColor3f(1.0, 0.0, 0.0);
-	glRasterPos2i(10, 10);
-	printstring(fs->fontbase, fs->frbuf);
-	fs->frcount++;
-    }
+    if (MI_IS_FPS(mi)) do_fps (mi);
     glPopMatrix();
 }
 
@@ -741,8 +798,7 @@ static Bool Init(ModeInfo * mi)
     int i;
     firestruct *fs = &fire[MI_SCREEN(mi)];
 
-    /* Default settings */
-
+    /* default settings */
     fs->eject_r = 0.1 + NRAND(10) * 0.03;
     fs->dt = 0.015;
     fs->eject_vy = 4;
@@ -755,6 +811,7 @@ static Bool Init(ModeInfo * mi)
     fs->alpha = DEF_ALPHA;
     fs->beta = DEF_BETA;
 
+    /* initialise texture stuff */
     if (do_texture)
     	inittextures(mi);
     else
@@ -763,41 +820,13 @@ static Bool Init(ModeInfo * mi)
 	fs->gtexture = (XImage*) NULL;
     }
 
-    /* initialise font stuff */
-    if (fs->framerate)
-    {
-	Font id;
-	unsigned int first, last;
-
-	if (mode_font == None)
-	    mode_font = getFont(MI_DISPLAY(mi)); 
-	if (mode_font == None) {
-	    fs->framerate = False;
-	} else {
-	    id    = mode_font->fid;
-	    first = mode_font->min_char_or_byte2;
-	    last  = mode_font->max_char_or_byte2;
-
-	    fs->fontbase = glGenLists((GLuint) last + 1);
-	    if (!fs->fontbase) {
-		(void) fprintf(stderr, "%s: unable to allocate font lists.\n",MI_NAME(mi));
-		fs->framerate = False;
-	    }
-	    else
-	    {
-		glXUseXFont(id, first, last - first + 1, fs->fontbase + first);
-	    }
-	}
-    }
-
     if (MI_IS_DEBUG(mi)) {
 	(void) fprintf(stderr,
-		       "%s:\n\tnum_part=%d\n\ttrees=%d\n\tfog=%s\n\tframerate=%s\n\tshadows=%s\n\teject_r=%.3f\n\tridtri=%.3f\n",
+		       "%s:\n\tnum_part=%d\n\ttrees=%d\n\tfog=%s\n\tshadows=%s\n\teject_r=%.3f\n\tridtri=%.3f\n",
 		       MI_NAME(mi),
 		       fs->np,
 		       fs->num_trees,
 		       fs->fog ? "on" : "off",
-		       fs->framerate ? "on" : "off",
 		       fs->shadows ? "on" : "off",
 		       fs->eject_r, fs->ridtri);
     }
@@ -806,7 +835,7 @@ static Bool Init(ModeInfo * mi)
     glEnable(GL_DEPTH_TEST);
 
     /* makes particles blend with background */
-    if (!MI_IS_WIREFRAME(mi))
+    if (!MI_IS_WIREFRAME(mi)||(!fs->np))
     {
     	glEnable(GL_BLEND);
     	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -828,6 +857,17 @@ static Bool Init(ModeInfo * mi)
 	if (!inittree(mi)) {
 		return False;
 	}
+
+    /* if no fire particles then initialise rain particles */
+    if (!fs->np)
+    {
+	vinit(fs->min,-7.0f,-0.2f,-7.0f);
+  	vinit(fs->max,7.0f,8.0f,7.0f);
+    	for (i = 0; i < NUMPART; i++) {
+	    setnewrain(fs, &(fs->r[i]));
+    	}
+    }
+    
     return True;
 }
 
@@ -852,6 +892,10 @@ free_fire(firestruct *fs)
 		(void) free((void *) fs->p);
 		fs->p = NULL;
 	}
+	if (fs->r != NULL) {
+		(void) free((void *) fs->r);
+		fs->r = NULL;
+	}
 	if (fs->treepos != NULL) {
 		(void) free((void *) fs->treepos);
 		fs->treepos = NULL;
@@ -879,27 +923,40 @@ init_fire(ModeInfo * mi)
 {
     firestruct *fs;
 
+    /* allocate the main fire table if needed */
     if (fire == NULL) {
 	if ((fire = (firestruct *) calloc(MI_NUM_SCREENS(mi),
 					  sizeof(firestruct))) == NULL)
 	    return;
     }
+
+    /* initialise the per screen fire structure */
     fs = &fire[MI_SCREEN(mi)];
     fs->np = MI_COUNT(mi);
     fs->fog = do_fog;
     fs->shadows = do_shadows;
-    fs->framerate = do_framerate;
-    if (fs->p == NULL) {
+    /* initialise fire particles if any */
+    if ((fs->np)&&(fs->p == NULL)) {
 	if ((fs->p = (part *) calloc(fs->np, sizeof(part))) == NULL) {
 	    free_fire(fs);
 	    return;
 	}
     }
+    else if (fs->r == NULL) {
+        /* initialise rain particles if no fire particles */
+	if ((fs->r = (rain *) calloc(NUMPART, sizeof(part))) == NULL) {
+	    free_fire(fs);
+	    return;
+	}
+    }
+
+    /* check tree number */
     if (do_texture)
     	fs->num_trees = (num_trees<MAX_TREES)?num_trees:MAX_TREES;
     else
     	fs->num_trees = 0;
 
+    /* xlock GL stuff */
     if ((fs->glx_context = init_GL(mi)) != NULL) {
 
 	Reshape(mi);
@@ -988,12 +1045,11 @@ void change_fire(ModeInfo * mi)
 
     if (MI_IS_DEBUG(mi)) {
 	(void) fprintf(stderr,
-		       "%s:\n\tnum_part=%d\n\ttrees=%d\n\tfog=%s\n\tframerate=%s\n\tshadows=%s\n\teject_r=%.3f\n\tridtri=%.3f\n",
+		       "%s:\n\tnum_part=%d\n\ttrees=%d\n\tfog=%s\n\tshadows=%s\n\teject_r=%.3f\n\tridtri=%.3f\n",
 		       MI_NAME(mi),
 		       fs->np,
 		       fs->num_trees,
 		       fs->fog ? "on" : "off",
-		       fs->framerate ? "on" : "off",
 		       fs->shadows ? "on" : "off",
 		       fs->eject_r, fs->ridtri);
     }
