@@ -2,7 +2,7 @@
 /* invert --- shere inversion */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)invert.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)invert.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
@@ -29,6 +29,8 @@ static const char sccsid[] = "@(#)invert.c	5.00 2000/11/01 xlockmore";
  * Demonstration of turning a sphere inside out without creating
  * any kinks (two surfaces can occupy the same space at the same time).
  *
+ * Revision History:
+ * 01-Mar-2001: Added FPS stuff E.Lassauge <lassauge@mail.dotcom.fr>
  */
 
 /*-
@@ -37,7 +39,7 @@ static const char sccsid[] = "@(#)invert.c	5.00 2000/11/01 xlockmore";
  */
 #include <X11/Intrinsic.h>
 #include "xlock.h"
-#include "vis.h"
+#include "visgl.h"
 
 #ifdef MODE_invert
 
@@ -47,6 +49,18 @@ static const char sccsid[] = "@(#)invert.c	5.00 2000/11/01 xlockmore";
 ModeSpecOpt invert_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct invert_description =
+{(char *) "invert", (char *) "init_invert",
+ (char *) "draw_invert", (char *) "release_invert",
+ (char *) "draw_invert", (char *) "init_invert",
+ NULL, &invert_opts,
+ 100, 1, 1, 1, 64, 1.0, (char *) "",
+ (char *) "Shows a sphere inverted without wrinkles", 0, NULL};
+
+#endif
+
+
 static spherestruct *spheres = NULL;
 
 /* new window size or exposure */
@@ -198,6 +212,7 @@ draw_invert(ModeInfo * mi)
   gp->view_rotz = (int) (gp->view_rotz + rot_incr) % 360;
 
   glFinish();
+  if (MI_IS_FPS(mi)) do_fps (mi);
   glXSwapBuffers(display, window);
 }
 
