@@ -2,7 +2,7 @@
 /* lament --- Shows Lemarchand's Box */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)lament.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)lament.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
@@ -17,6 +17,7 @@ static const char sccsid[] = "@(#)lament.c	5.00 2000/11/01 xlockmore";
  * implied warranty.
  *
  * Revisions:
+ * 01-Mar-2001: Added FPS stuff - Eric Lassauge <lassauge@mail.dotcom.fr>
  * 01-Nov-2000: Allocation checks
  * 25-Jul-1998: Written by jwz
  */
@@ -105,12 +106,13 @@ static const char sccsid[] = "@(#)lament.c	5.00 2000/11/01 xlockmore";
 #define HACK_DRAW	draw_lament
 #define lament_opts	xlockmore_opts
 #define DEFAULTS	"*delay:	10000   \n"	\
+			"*showFps:      False   \n"     \
 			"*wireframe:	False	\n"	\
 			"*texture:	True	\n"
 #include "xlockmore.h"
 #else /* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+#include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_lament
@@ -2407,6 +2409,7 @@ draw_lament(ModeInfo * mi)
 
 	glXMakeCurrent(display, window, *(lc->glx_context));
 	draw(mi);
+        if (MI_IS_FPS(mi)) do_fps (mi);
 	glFinish();
 	glXSwapBuffers(display, window);
 
