@@ -2,7 +2,7 @@
 /* moebius --- Moebius Strip II, an Escher-like GL scene with ants. */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)moebius.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)moebius.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
@@ -44,6 +44,9 @@ static const char sccsid[] = "@(#)moebius.c	5.00 2000/11/01 xlockmore";
  * Marcelo F. Vianna (Jun-01-1997)
  *
  * Revision History:
+ * 01-Mar-2001: backported from xscreensaver by lassauge@mail.dotcom.fr
+ *    Feb-2001: Made motion and rotation be smoother Jamie Zawinski
+ *              <jwz@jwz.org>
  * 01-Nov-2000: Allocation checks
  * 01-Jan-1998: Mode separated from escher and renamed
  * 08-Jun-1997: New scene implemented: "Impossible Cage" based in a M.C.
@@ -79,20 +82,23 @@ static const char sccsid[] = "@(#)moebius.c	5.00 2000/11/01 xlockmore";
 #include <X11/Intrinsic.h>
 
 #ifdef STANDALONE
-#define PROGCLASS "Moebius"
-#define HACK_INIT init_moebius
-#define HACK_DRAW draw_moebius
-#define moebius_opts xlockmore_opts
-#define DEFAULTS "*delay: 1000 \n"
-#include "xlockmore.h"		/* from the xscreensaver distribution */
+# define PROGCLASS			"Moebius"
+# define HACK_INIT			init_moebius
+# define HACK_DRAW			draw_moebius
+# define HACK_RESHAPE			reshape_moebius
+# define moebius_opts			xlockmore_opts
+# define DEFAULTS			"*cycles:	1       \n"	\
+					"*delay:	20000   \n"	\
+					"*showFps:      False   \n"	\
+					"*wireframe:	False	\n"
+# include "xlockmore.h"		/* from the xscreensaver distribution */
 #else /* !STANDALONE */
-#include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+# include "xlock.h"		/* from the xlockmore distribution */
+# include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_moebius
 
-
 #include <GL/glu.h>
 #include "e_textures.h"
 
@@ -105,14 +111,15 @@ static int  noants;
 static XrmOptionDescRec opts[] =
 {
   {(char *) "-solidmoebius", (char *) ".moebius.solidmoebius", XrmoptionNoArg, (caddr_t) "on"},
- {(char *) "+solidmoebius", (char *) ".moebius.solidmoebius", XrmoptionNoArg, (caddr_t) "off"},
-	{(char *) "-noants", (char *) ".moebius.noants", XrmoptionNoArg, (caddr_t) "on"},
-	{(char *) "+noants", (char *) ".moebius.noants", XrmoptionNoArg, (caddr_t) "off"}
+  {(char *) "+solidmoebius", (char *) ".moebius.solidmoebius", XrmoptionNoArg, (caddr_t) "off"},
+  {(char *) "-noants", (char *) ".moebius.noants", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+noants", (char *) ".moebius.noants", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & solidmoebius, (char *) "solidmoebius", (char *) "Solidmoebius", (char *) DEF_SOLIDMOEBIUS, t_Bool},
-	{(caddr_t *) & noants, (char *) "noants", (char *) "Noants", (char *) DEF_NOANTS, t_Bool}
+  {(caddr_t *) & solidmoebius, (char *) "solidmoebius", (char *) "Solidmoebius", (char *) DEF_SOLIDMOEBIUS, t_Bool},
+  {(caddr_t *) & noants, (char *) "noants", (char *) "Noants", (char *) DEF_NOANTS, t_Bool}
+
 };
 static OptionStruct desc[] =
 {
@@ -153,6 +160,12 @@ typedef struct {
 	GLfloat     ant_position;
 	Bool        AreObjectsDefined[MaxObj];
 	GLXContext *glx_context;
+
+  GLfloat rotx, roty, rotz;	   /* current object rotation */
+  GLfloat dx, dy, dz;		   /* current rotational velocity */
+  GLfloat ddx, ddy, ddz;	   /* current rotational acceleration */
+  GLfloat d_max;			   /* max velocity */
+
 } moebiusstruct;
 
 static float front_shininess[] =
@@ -198,6 +211,8 @@ static float MaterialGray8[] =
 static moebiusstruct *moebius = NULL;
 static GLuint objects = 0;
 
+#define NUM_SCENES      2
+
 static Bool
 mySphere(float radius)
 {
@@ -441,6 +456,7 @@ draw_moebius_strip(ModeInfo * mi)
 		if (glGetError() != GL_NO_ERROR) {
 			return False;
 		}
+
 		if (solidmoebius) {
 			glBegin(GL_QUAD_STRIP);
 			Phi = 0;
@@ -568,19 +584,25 @@ draw_moebius_strip(ModeInfo * mi)
 #undef MoebiusTransversals
 
 static void
-reshape(ModeInfo * mi, int width, int height)
+reshape_moebius(ModeInfo * mi, int width, int height)
 {
 	moebiusstruct *mp = &moebius[MI_SCREEN(mi)];
-	int i;
 
 	glViewport(0, 0, mp->WindW = (GLint) width, mp->WindH = (GLint) height);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 	glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 15.0);
 	glMatrixMode(GL_MODELVIEW);
-	i = width / 512 + 1;
-	glLineWidth(i);
-	glPointSize(i);
+	if (width >= 1024) {
+		glLineWidth(3);
+		glPointSize(3);
+	} else if (width >= 512) {
+		glLineWidth(2);
+		glPointSize(2);
+	} else {
+		glLineWidth(1);
+		glPointSize(1);
+	}
 	mp->AreObjectsDefined[ObjMoebiusStrip] = False;
 	mp->AreObjectsDefined[ObjAntBody] = False;
 }
@@ -625,6 +647,79 @@ pinit(void)
 	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_specular);
 }
 
+
+
+/* lifted from lament.c */
+#define RANDSIGN() ((LRAND() & 1) ? 1 : -1)
+#define FLOATRAND(a) (((double)LRAND() / (double)MAXRAND) * a)
+
+static void
+rotate(GLfloat *pos, GLfloat *v, GLfloat *dv, GLfloat max_v)
+{
+  double ppos = *pos;
+
+  /* tick position */
+  if (ppos < 0)
+    ppos = -(ppos + *v);
+  else
+    ppos += *v;
+
+  if (ppos > 1.0)
+    ppos -= 1.0;
+  else if (ppos < 0)
+    ppos += 1.0;
+
+  if (ppos < 0) abort();
+  if (ppos > 1.0) abort();
+  *pos = (*pos > 0 ? ppos : -ppos);
+
+  /* accelerate */
+  *v += *dv;
+
+  /* clamp velocity */
+  if (*v > max_v || *v < -max_v)
+    {
+      *dv = -*dv;
+    }
+  /* If it stops, start it going in the other direction. */
+  else if (*v < 0)
+    {
+      if (random() % 4)
+	{
+	  *v = 0;
+
+	  /* keep going in the same direction */
+	  if (random() % 2)
+	    *dv = 0;
+	  else if (*dv < 0)
+	    *dv = -*dv;
+	}
+      else
+	{
+	  /* reverse gears */
+	  *v = -*v;
+	  *dv = -*dv;
+	  *pos = -*pos;
+	}
+    }
+
+  /* Alter direction of rotational acceleration randomly. */
+  if (! (random() % 120))
+    *dv = -*dv;
+
+  /* Change acceleration very occasionally. */
+  if (! (random() % 200))
+    {
+      if (*dv == 0)
+	*dv = 0.00001;
+      else if (random() & 1)
+	*dv *= 1.2;
+      else
+	*dv *= 0.8;
+    }
+}
+
+
 void
 init_moebius(ModeInfo * mi)
 {
@@ -639,13 +734,33 @@ init_moebius(ModeInfo * mi)
 	mp->step = NRAND(90);
 	mp->ant_position = NRAND(90);
 
+    mp->rotx = FLOATRAND(1.0) * RANDSIGN();
+    mp->roty = FLOATRAND(1.0) * RANDSIGN();
+    mp->rotz = FLOATRAND(1.0) * RANDSIGN();
+
+    /* bell curve from 0-1.5 degrees, avg 0.75 */
+    mp->dx = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
+    mp->dy = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
+    mp->dz = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
+
+    mp->d_max = mp->dx * 2;
+
+    mp->ddx = 0.00006 + FLOATRAND(0.00003);
+    mp->ddy = 0.00006 + FLOATRAND(0.00003);
+    mp->ddz = 0.00006 + FLOATRAND(0.00003);
+
+    mp->ddx = 0.00001;
+    mp->ddy = 0.00001;
+    mp->ddz = 0.00001;
+
 	if ((mp->glx_context = init_GL(mi)) != NULL) {
-		reshape(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
+
+		reshape_moebius(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
 		glDrawBuffer(GL_BACK);
 		if (!glIsList(objects))
 			if ((objects = glGenLists(MaxObj)) == 0) {
 				MI_CLEARWINDOW(mi);
-				release_cage(mi);
+				release_moebius(mi);
 				return;
 			}
 		pinit();
@@ -657,12 +772,13 @@ init_moebius(ModeInfo * mi)
 void
 draw_moebius(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
-	Window      window = MI_WINDOW(mi);
 	moebiusstruct *mp;
 
-	if (moebius == NULL)
-		return;
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+
+        if (moebius == NULL)
+	    return;
 	mp = &moebius[MI_SCREEN(mi)];
 
 	MI_IS_DRAWN(mi) = True;
@@ -684,10 +800,19 @@ draw_moebius(ModeInfo * mi)
 		glScalef(Scale4Iconic * mp->WindH / mp->WindW, Scale4Iconic, Scale4Iconic);
 	}
 
+    {
+      GLfloat x = mp->rotx;
+      GLfloat y = mp->roty;
+      GLfloat z = mp->rotz;
+      if (x < 0) x = 1 - (x + 1);
+      if (y < 0) y = 1 - (y + 1);
+      if (z < 0) z = 1 - (z + 1);
+      glRotatef(x * 360, 1.0, 0.0, 0.0);
+      glRotatef(y * 360, 0.0, 1.0, 0.0);
+      glRotatef(z * 360, 0.0, 0.0, 1.0);
+    }
+
 	/* moebius */
-	glRotatef(mp->step * 100, 1, 0, 0);
-	glRotatef(mp->step * 95, 0, 1, 0);
-	glRotatef(mp->step * 90, 0, 0, 1);
 	if (!draw_moebius_strip(mi)) {
 		release_moebius(mi);
 		return;
@@ -695,6 +820,11 @@ draw_moebius(ModeInfo * mi)
 
 	glPopMatrix();
 
+    rotate(&mp->rotx, &mp->dx, &mp->ddx, mp->d_max);
+    rotate(&mp->roty, &mp->dy, &mp->ddy, mp->d_max);
+    rotate(&mp->rotz, &mp->dz, &mp->ddz, mp->d_max);
+
+    if (MI_IS_FPS(mi)) do_fps (mi);
 	glFlush();
 
 	glXSwapBuffers(display, window);
@@ -718,21 +848,13 @@ void
 release_moebius(ModeInfo * mi)
 {
 	if (moebius != NULL) {
-		int screen;
-
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			moebiusstruct *mp = &moebius[screen];
-
-			if (mp->glx_context) {
-				if (glIsList(objects)) {
-					glDeleteLists(objects, MaxObj);
-					objects = 0;
-				}
-			}
-		}
 		(void) free((void *) moebius);
 		moebius = NULL;
 	}
+	if (glIsList(objects)) {
+		glDeleteLists(objects, MaxObj);
+		objects = 0;
+	}
 	FreeAllGL(mi);
 }
 
