@@ -2,7 +2,7 @@
 /* morph3d --- Shows 3D morphing objects */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)morph3d.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)morph3d.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
@@ -45,8 +45,9 @@ static const char sccsid[] = "@(#)morph3d.c	5.00 2000/11/01 xlockmore";
  * Marcelo F. Vianna (Feb-13-1997)
  *
  * Revision History:
- * 27-Jul-97: Speed ups by Marcelo F. Vianna.
- * 08-May-97: Speed ups by Marcelo F. Vianna.
+ * 01-Mar-2001: Added FPS stuff E.Lassauge <lassauge@mail.dotcom.fr>
+ * 27-Jul-1997: Speed ups by Marcelo F. Vianna.
+ * 08-May-1997: Speed ups by Marcelo F. Vianna.
  *
  */
 
@@ -887,7 +888,7 @@ draw_morph3d(ModeInfo * mi)
 	glPopMatrix();
 
 	glFlush();
-
+	if (MI_IS_FPS(mi)) do_fps (mi);
 	glXSwapBuffers(display, window);
 
 	mp->step += 0.05;
