@@ -2,7 +2,7 @@
 /* rubik --- Shows an auto-solving Rubik's cube */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)rubik.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)rubik.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
@@ -37,6 +37,7 @@ static const char sccsid[] = "@(#)rubik.c	5.00 2000/11/01 xlockmore";
  * Marcelo F. Vianna (Jul-31-1997)
  *
  * Revision History:
+ * 01-Mar-2001: Added FPS stuff - Eric Lassauge <lassauge@mail.dotcom.fr>
  * 01-Nov-2000: Allocation checks
  * 27-Apr-1999: LxMxN stuff added.
  * 26-Sep-1998: Added some more movement (the cube does not stay in the screen
@@ -123,12 +124,13 @@ static const char sccsid[] = "@(#)rubik.c	5.00 2000/11/01 xlockmore";
 #define rubik_opts xlockmore_opts
 #define DEFAULTS "*delay: 40000 \n" \
  "*count: -30 \n" \
+ "*showFps: False \n" \
  "*cycles: 5 \n" \
  "*size: -6 \n"
 #include "xlockmore.h"		/* from the xscreensaver distribution */
 #else /* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+#include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_rubik
@@ -1887,6 +1889,29 @@ free_rubik(rubikstruct *rp)
 	}
 }
 
+void
+release_rubik(ModeInfo * mi)
+{
+	if (rubik != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			rubikstruct *rp = &rubik[screen];
+
+			free_rubik(rp);
+			if (rp->glx_context) {
+				if (glIsList(objects)) {
+					glDeleteLists(objects, MaxObj);
+					objects = 0;
+				}
+			}
+		}
+		(void) free((void *) rubik);
+		rubik = NULL;
+	}
+	FreeAllGL(mi);
+}
+
 void
 init_rubik(ModeInfo * mi)
 {
@@ -2001,6 +2026,7 @@ draw_rubik(ModeInfo * mi)
 		release_rubik(mi);
 		return;
 	}
+        if (MI_IS_FPS(mi)) do_fps (mi);
 	glXSwapBuffers(display, window);
 
 	if (rp->action == ACTION_SHUFFLE) {
@@ -2116,27 +2142,4 @@ change_rubik(ModeInfo * mi)
 	}
 }
 
-void
-release_rubik(ModeInfo * mi)
-{
-	if (rubik != NULL) {
-		int         screen;
-
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			rubikstruct *rp = &rubik[screen];
-
-			free_rubik(rp);
-			if (rp->glx_context) {
-				if (glIsList(objects)) {
-					glDeleteLists(objects, MaxObj);
-					objects = 0;
-				}
-			}
-		}
-		(void) free((void *) rubik);
-		rubik = NULL;
-	}
-	FreeAllGL(mi);
-}
-
 #endif
