@@ -1,10 +1,10 @@
 /* sballs --- 8 balls spinning like crazy in GL */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)sballs.c	5.00 2000/11/14 xlockmore";
+static const char sccsid[] = "@(#)sballs.c	5.01 2001/03/09 xlockmore";
 #endif
 
-/* Copyright (c) E. Lassauge, 2000. */
+/* Copyright (c) E. Lassauge, 2001. */
 
 /*
  * Permission to use, copy, modify, and distribute this software and its
@@ -28,6 +28,8 @@ static const char sccsid[] = "@(#)sballs.c	5.00 2000/11/14 xlockmore";
  *
  * REVISION HISTORY:
  *
+ * E.Lassauge - 09-Mar-2001:
+ *      - get rid of my framerate options to use showfps
  * E.Lassauge - 28-Nov-2000:
  *      - add handling of polyhedrons (like in ico)
  *      - modified release part to add freeing of GL objects
@@ -45,12 +47,13 @@ static const char sccsid[] = "@(#)sballs.c	5.00 2000/11/14 xlockmore";
  "*size: 0 \n" \
  "*object: 0\n" \	/* random object */
  "*trackmouse: False \n" \
+ "*showfps: False \n" \
  "*wireframe:  False \n"
 
 #include "xlockmore.h"		/* from the xscreensaver distribution */
 #else				/* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+#include "visgl.h"
 #endif				/* !STANDALONE */
 #include "iostuff.h"            /* getFont() */
 
@@ -76,12 +79,10 @@ static const char sccsid[] = "@(#)sballs.c	5.00 2000/11/14 xlockmore";
 
 /* Manage option vars */
 #define DEF_TEXTURE	"True"
-#define DEF_FRAMERATE	"False"
 #define DEF_TRACKMOUSE  "False"
 #define DEF_OBJECT	"2"
 #define DEF_OBJECT_INDX	2
 static Bool do_texture;
-static Bool do_framerate;
 static Bool do_trackmouse;
 static int  object;
 static int  spheres;
@@ -90,8 +91,6 @@ static XFontStruct *mode_font = None;
 static XrmOptionDescRec opts[] = {
     {(char *) "-texture", (char *) ".sballs.texture", XrmoptionNoArg, (caddr_t) "on"},
     {(char *) "+texture", (char *) ".sballs.texture", XrmoptionNoArg, (caddr_t) "off"},
-    {(char *) "-framerate", (char *) ".sballs.framerate", XrmoptionNoArg, (caddr_t) "on"},
-    {(char *) "+framerate", (char *) ".sballs.framerate", XrmoptionNoArg, (caddr_t) "off"},
     {(char *) "-trackmouse", (char *) ".sballs.trackmouse", XrmoptionNoArg, (caddr_t) "on"},
     {(char *) "+trackmouse", (char *) ".sballs.trackmouse", XrmoptionNoArg, (caddr_t) "off"},
     {(char *) "-object", (char *) ".sballs.object", XrmoptionSepArg, (caddr_t) NULL},
@@ -100,7 +99,6 @@ static XrmOptionDescRec opts[] = {
 
 static argtype vars[] = {
     {(caddr_t *) & do_texture, (char *) "texture", (char *) "Texture", (char *) DEF_TEXTURE, t_Bool},
-    {(caddr_t *) & do_framerate, (char *) "framerate", (char *) "Framerate", (char *) DEF_FRAMERATE, t_Bool},
     {(caddr_t *) & do_trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool},
     {(caddr_t *) & object, (char *) "object", (char *) "Object", (char *) DEF_OBJECT, t_Int},
 
@@ -110,7 +108,6 @@ static OptionStruct desc[] = {
     /*{(char *) "-count spheres", (char *) "set number of spheres"},*/
     /*{(char *) "-cycles speed", (char *) "set ball speed value"},*/
     {(char *) "-/+texture", (char *) "turn on/off texturing"},
-    {(char *) "-/+framerate", (char *) "turn on/off framerate display"},
     {(char *) "-/+trackmouse", (char *) "turn on/off the tracking of the mouse"},
     {(char *) "-object num", (char *) "number of the 3D object (0 means random)"},
 };
@@ -146,8 +143,6 @@ typedef struct {
     GLint WIDTH, HEIGHT;	/* display dimensions */
     GLXContext *glx_context;
 
-    Bool framerate;
-
 
     XImage *btexture;		/* back texture image bits */
     XImage *ftexture;		/* face texture image bits */
@@ -372,30 +367,6 @@ static void clamp(vec3_t v)
             v[i] = 0;
 }
 
-/* utility function for the framerate display */
-static float gettime(sballsstruct * sb)
-{
-    clock_t tnew, ris;
-
-    tnew = clock();
-
-    ris = tnew - sb->told;
-
-    sb->told = tnew;
-
-    return (ris / (float) CLOCKS_PER_SEC);
-}
-
-/* utility function for string printing for framerate */
-static void printstring(GLuint fontbase, char *string)
-{
-    int len;
-
-    len = (int) strlen(string);
-    glListBase(fontbase);
-    glCallLists(len, GL_UNSIGNED_BYTE, (GLubyte *) string);
-}
-
 /* track the mouse in a joystick manner : not perfect but it works */
 static void trackmouse(ModeInfo * mi)
 {
@@ -593,7 +564,6 @@ static void Reshape(ModeInfo * mi)
 
 static void Draw(ModeInfo * mi)
 {
-    float fr;
     sballsstruct *sb = &sballs[MI_SCREEN(mi)];
     int sphere;
 
@@ -655,31 +625,12 @@ static void Draw(ModeInfo * mi)
         drawSphere(mi,sphere);
     }
 
-    /* manage framerate value */
-    if (sb->framerate && sb->frcount >= FRAME) {
-        sb->frcount = 0;
-	fr = gettime(sb);
-	(void) sprintf(sb->frbuf, "Frame rate: %f", FRAME / fr);
-    }
-
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_DEPTH_TEST);
     glDisable(GL_LIGHTING);
 
     /* manage framerate display */
-    if (sb->framerate) {
-    	glMatrixMode(GL_PROJECTION);
-    	glLoadIdentity();
-    	glOrtho(-0.5, sb->WIDTH - 0.5 , -0.5, sb->HEIGHT-0.5, -1.0, 1.0);
-    	glMatrixMode(GL_MODELVIEW);
-    	glLoadIdentity();
-
-	/* draw framerate (red text) */
-	glColor3f(1.0, 0.0, 0.0);
-	glRasterPos2i(10, 10);
-	printstring(sb->fontbase, sb->frbuf);
-	sb->frcount++;
-    }
+    if (MI_IS_FPS(mi)) do_fps (mi);
     glPopMatrix();
 }
 
@@ -732,44 +683,13 @@ static void Init(ModeInfo * mi)
 #endif
     }
 
-    /* initialise font stuff */
-    if (sb->framerate)
-    {
-	Font id;
-	unsigned int first, last;
-
-	if (mode_font == None)
-	    mode_font = getFont(MI_DISPLAY(mi)); 
-	if (mode_font == None)
-        {
-	    sb->framerate = False;
-	}
-	else
-	{
-	    id    = mode_font->fid;
-	    first = mode_font->min_char_or_byte2;
-	    last  = mode_font->max_char_or_byte2;
-
-	    sb->fontbase = glGenLists((GLuint) last + 1);
-	    if (!sb->fontbase) {
-		(void) fprintf(stderr, "%s: unable to allocate font lists.\n",MI_NAME(mi));
-		sb->framerate = False;
-	    }
-	    else
-	    {
-		glXUseXFont(id, first, last - first + 1, sb->fontbase + first);
-	    }
-	}
-    }
-
     if (MI_IS_DEBUG(mi)) {
 	(void) fprintf(stderr,
-		       "%s:\n\tobject=%s\n\tspheres=%d\n\tspeed=%d\n\tframerate=%s\n\ttexture=%s\n",
+		       "%s:\n\tobject=%s\n\tspheres=%d\n\tspeed=%d\n\ttexture=%s\n",
 		       MI_NAME(mi),
 		       polygons[object].shortname,
 		       spheres,
 		       MI_CYCLES(mi),
-		       sb->framerate ? "on" : "off",
 		       do_texture ? "on" : "off"
 			);
     }
@@ -806,7 +726,6 @@ void init_sballs(ModeInfo * mi)
 	    return;
     }
     sb = &sballs[MI_SCREEN(mi)];
-    sb->framerate = do_framerate;
 
     if ((sb->glx_context = init_GL(mi)) != NULL) {
 
@@ -919,12 +838,11 @@ void change_sballs(ModeInfo * mi)
 
     if (MI_IS_DEBUG(mi)) {
 	(void) fprintf(stderr,
-		       "%s:\n\tobject=%s\n\tspheres=%d\n\tspeed=%d\n\tframerate=%s\n\ttexture=%s\n",
+		       "%s:\n\tobject=%s\n\tspheres=%d\n\tspeed=%d\n\ttexture=%s\n",
 		       MI_NAME(mi),
 		       polygons[object].shortname,
 		       spheres,
 		       MI_CYCLES(mi),
-		       sb->framerate ? "on" : "off",
 		       do_texture ? "on" : "off"
 			);
     }
