@@ -2,7 +2,7 @@
 /* superquadrics --- 3D mathematical shapes */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)superquadrics.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)superquadrics.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
@@ -31,20 +31,21 @@ static const char sccsid[] = "@(#)superquadrics.c	5.00 2000/11/01 xlockmore";
  * quadric surface in a similar fashion.
  *
  * Revision History:
+ * 01-Mar-2001: Added FPS stuff - Eric Lassauge <lassauge@mail.dotcom.fr>
  * 01-Nov-2000: Allocation checks
- * 30-Mar-1997: Turned into a module for xlockmore 4.02 alpha.  The code
+ * 30-Mar-97: Turned into a module for xlockmore 4.02 alpha.  The code
  *    is almost unrecognizable now from the first revision, except for
  *    a few remaining two-letter variable names.  I still don't have
  *    the normal vectors working right (I wrote the buggy normal vector
  *    code myself, can you tell?)
- * 07-Jan-1997: A legend reborn; Superquadrics make an appearance as a
+ * 07-Jan-97: A legend reborn; Superquadrics make an appearance as a
  *    real OpenGL program written in C.  I can even render them with
  *    proper lighting and specular highlights.  Gee, they look almost
  *    as good now as the original color plates of them that my uncle
  *    showed me as a child in 1981.  I don't know what computer hardware
  *    he was using at the time, but it's taken a couple decades for the
  *    PC clone hardware to catch up to it.
- * 05-Jan-1997: After almost a decade, Superquadrics had almost faded away
+ * 05-Jan-97: After almost a decade, Superquadrics had almost faded away
  *    into the myths and folklore of all the things my brother and I played
  *    with on computers when we were kids with too much time on our hands.
  *    I had since gotten involved in Unix, OpenGL, and other things.
@@ -80,18 +81,19 @@ static const char sccsid[] = "@(#)superquadrics.c	5.00 2000/11/01 xlockmore";
 #include <X11/Intrinsic.h>
 
 #ifdef STANDALONE
-#define PROGCLASS "Superquadrics"
-#define HACK_INIT init_superquadrics
-#define HACK_DRAW draw_superquadrics
-#define superquadrics_opts xlockmore_opts
-#define DEFAULTS "*delay: 100 \n" \
- "*count: 25 \n" \
- "*cycles: 40 \n" \
- "*wireframe: False \n"
-#include "xlockmore.h"		/* from the xscreensaver distribution */
-#else /* !STANDALONE */
-#include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+# define PROGCLASS				"Superquadrics"
+# define HACK_INIT				init_superquadrics
+# define HACK_DRAW				draw_superquadrics
+# define superquadrics_opts			xlockmore_opts
+# define DEFAULTS	"*delay:	40000   \n"	\
+			"*count:	25      \n"	\
+			"*cycles:	40      \n"	\
+			"*showfps:      False   \n"	\
+			"*wireframe:	False	\n"
+# include "xlockmore.h"			/* from the xscreensaver distribution */
+#else  /* !STANDALONE */
+# include "xlock.h"			/* from the xlockmore distribution */
+# include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_superquadrics
@@ -109,11 +111,11 @@ static float spinspeed;
 
 static XrmOptionDescRec opts[] =
 {
-  {(char *) "-spinspeed", (char *) ".superquadrics.spinspeed", XrmoptionSepArg, (caddr_t) NULL}
+  {(char *) "-spinspeed",(char *) ".superquadrics.spinspeed", XrmoptionSepArg, (caddr_t) NULL}
 };
 static argtype vars[] =
 {
-  {(caddr_t *) & spinspeed, (char *) "spinspeed", (char *) "Spinspeed", (char *) DEF_SPINSPEED, t_Float}
+  {(caddr_t *) & spinspeed, (char *) "spinspeed", (char *) "Spinspeed", (char *)DEF_SPINSPEED, t_Float}
 };
 static OptionStruct desc[] =
 {
@@ -714,6 +716,8 @@ init_superquadrics(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
+	int         screen = MI_SCREEN(mi);
+
 	superquadricsstruct *sp;
 
 	if (superquadrics == NULL) {
@@ -721,8 +725,7 @@ init_superquadrics(ModeInfo * mi)
 				      sizeof (superquadricsstruct))) == NULL)
 			return;
 	}
-	sp = &superquadrics[MI_SCREEN(mi)];
-
+	sp = &superquadrics[screen];
 	sp->mono = (MI_IS_MONO(mi) ? 1 : 0);
 
 	if ((sp->glx_context = init_GL(mi)) != NULL) {
@@ -758,6 +761,7 @@ draw_superquadrics(ModeInfo * mi)
 
 	NextSuperquadricDisplay(sp);
 
+    	if (MI_IS_FPS(mi)) do_fps (mi);
 	glFinish();
 	glXSwapBuffers(display, window);
 }
