@@ -2,7 +2,7 @@
 /* text3d --- Shows moving 3D texts */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)text3d.cc	5.0 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)text3d.cc	5.01 2001/03/09 xlockmore";
 
 #endif
 
@@ -24,15 +24,16 @@ static const char sccsid[] = "@(#)text3d.cc	5.0 2000/11/01 xlockmore";
  * This module is based on a demo of the gltt graphics library
  * Copyright (C) 1998 Stephane Rehel.
  *
- * See the gltt Official Site at http://home.worldnet.fr/~rehel/gltt/gltt.html
- * May have better luck at http://perso.libertysurf.fr/lassauge/en_xlock.htm
+ * See the gltt Official Site at http://gltt.sourceforge.net/
+ * May have better luck at http://lassauge.free.fr/xlock/
  *
  * My e-mail address changed to lassauge@mail.dotcom.fr
- * Web site at http://perso.libertysurf.fr/lassauge/
+ * Web site at http://lassauge.free.fr/
  *
  * Eric Lassauge  (October-28-1999)
  *
  * Revision History:
+ * 09-Mar-2001: Removed an erroneous PushMatrix !!!
  * 01-Nov-2000: Allocation checks
  * 28-Oct-1999: fixes from Jouk "I play with every mode" Jansen.
  *               Option ttanimate added.
@@ -51,6 +52,9 @@ static const char sccsid[] = "@(#)text3d.cc	5.0 2000/11/01 xlockmore";
  *       Light problem with some letters (don't know why they "reflect" more):
  *       is the problem in gltt or Mesa ???
  *       SPEED !!!!
+ *       It may sigfault when compiled with -fschedule-insns2
+ *         (i686-pc-linux-gnu)
+ *          
  */
 
 #ifdef STANDALONE
@@ -77,6 +81,7 @@ extern "C"
 #undef error
 #endif
 #endif				/* !STANDALONE */
+#include "iostuff.h"
 
 #ifdef MODE_text3d
 
@@ -110,10 +115,28 @@ extern "C" { void change_text3d(ModeInfo * mi); }
 extern "C" { void release_text3d(ModeInfo * mi); }
 extern "C" { void refresh_text3d(ModeInfo * mi); }
 
-/* Manage Option vars */
-/* I found this font on NT in c:\WINDOWS\SYSTEM\ARIAL.TTF can also be found
-   on Windows95 in c:\windows\fonts\arial.ttf */
-#define DEF_FONT "arial.ttf"
+/* arial.ttf is not supplied for legal reasons. */
+/* NT and Windows 3.1 in c:\WINDOWS\SYSTEM\ARIAL.TTF */
+/* Windows95 in c:\windows\fonts\arial.ttf */
+
+#ifndef DEF_TTFONT
+#if defined (SOLARIS2) || defined (SUNOS4)
+#if 0
+#define DEF_TTFONT "/usr/share/fonts/xlockmore/arial.ttf"
+#else
+/* Directory of only *.ttf */
+#define DEF_TTFONT "/usr/share/fonts/xlockmore/"
+#endif
+#else
+#if 0
+#define DEF_TTFONT "/usr/X11R6/lib/X11/fonts/TTF/arial.ttf"
+#else
+/* Directory of only *.ttf */
+#define DEF_TTFONT "/usr/X11R6/lib/X11/fonts/TTF/"
+#endif
+#endif
+#endif
+
 #define DEF_EXTRUSION  "25.0"
 #define DEF_ROTAMPL  "1.0"
 #define DEF_ROTFREQ  "0.001"
@@ -127,6 +150,8 @@ static char *mode_font;
 static int nosplit;
 static char *animate;
 
+/* Manage Option vars */
+
 static XrmOptionDescRec opts[] =
 {
     {(char *) "-ttfont", (char *) ".text3d.ttfont", XrmoptionSepArg, (caddr_t) NULL},
@@ -140,7 +165,7 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-    {(caddr_t *) & mode_font, (char *) "ttfont", (char *) "TTFont", (char *) DEF_FONT, t_String},
+    {(caddr_t *) & mode_font, (char *) "ttfont", (char *) "TTFont", (char *) DEF_TTFONT, t_String},
     {(caddr_t *) & extrusion, (char *) "extrusion", (char *) "Extrusion", (char *) DEF_EXTRUSION, t_Float},
     {(caddr_t *) & rampl, (char *) "rot_amplitude", (char *) "RotationAmplitude", (char *) DEF_ROTAMPL, t_Float},
     {(caddr_t *) & rfreq, (char *) "rot_frequency", (char *) "RotationFrequency", (char *) DEF_ROTFREQ, t_Float},
@@ -171,22 +196,6 @@ ModStruct text3d_description =
 
 static text3dstruct *text3d = (text3dstruct *) NULL;
 
-/* Hacks to use $TOP/xlock/util.c and iostuff.c functions */
-extern "C" { int index_dir(char *str1, char *substr); }
-extern "C" { char *getWords(int screen, int screens); }
-extern "C" { FILE *my_fopen(char *filename, char *type); }
-
-#if HAVE_DIRENT_H
-#define sel_font	sel_image	/* "rename" the sel_image function */
-extern "C" { void get_dir(char *fullpath, char *dir, char *filename); }
-extern "C" { int sel_font(struct dirent *name); }
-extern "C" { int scan_dir(const char *directoryname,
-		 struct dirent ***namelist,
-		 int (*select) (struct dirent *),
-		 int (*compare) (const void *, const void *));
-}
-#endif				/* HAVE_DIRENT_H */
-
 const double angle_speed = 2.5 / 180.0 * M_PI;
 
 extern "C" {
@@ -208,6 +217,7 @@ static void t3d_anim_rotatexy(text3dstruct * tp);
 static void t3d_anim_rotateyz(text3dstruct * tp);
 static void t3d_anim_frequency(text3dstruct * tp);
 static void t3d_anim_amplitude(text3dstruct * tp);
+static char *fontfile = NULL;
 
 #ifdef __cplusplus
 }
@@ -261,73 +271,6 @@ static int anims=sizeof anim_array / sizeof anim_array[0] ;
  *-----------------------------------------------------------------------------
  */
 
-static char *
- text3d_newstr(char *s1)
-{
-    char *s;
-
-    s = (char *) malloc(strlen(s1));
-    strcpy(s, s1);
-    return s;
-}
-
-static int
- readable(char *filename)
-{
-    FILE *fp;
-
-    if ((fp = my_fopen(filename, (char *) "r")) == NULL)
-	return False;
-    (void) fclose(fp);
-    return True;
-}
-
-/*-------------------------------------------------------------*/
-#if HAVE_DIRENT_H
-
-static void
- randomFileFromList(char *directory,
-		    struct dirent **filelist, int numfiles, char *file_local)
-{
-    int num;
-
-    num = NRAND(numfiles);
-    if (strlen(directory) + strlen(filelist[num]->d_name) + 1 < 256)
-    {
-	(void) sprintf(file_local, "%s%s",
-		       directory, filelist[num]->d_name);
-    }
-}
-
-/*-------------------------------------------------------------*/
-static void
- getRandomFontFile(char *randomfile, char *randomfile_local)
-{
-    /* mimic getRandomImageFile from $TOP/xlock/iostuff.c */
-    char directory_r[DIRBUF];
-    struct dirent ***fonts_list;
-    struct dirent **font_list = (dirent **) NULL;
-    int num_list;
-
-    /* extern char 'stolen' from $TOP/xlock/resource.c */
-    extern char filename_r[MAXNAMLEN];
-
-    get_dir(randomfile, directory_r, filename_r);
-
-    fonts_list = (struct dirent ***) malloc(sizeof(struct dirent **));
-
-    num_list = scan_dir(directory_r, fonts_list, sel_font, (int (*)(const
-						 void *, const void *)) NULL);
-    font_list = *fonts_list;
-    (void) free((void *) fonts_list);
-    if (num_list > 0)
-    {
-	randomFileFromList(directory_r, font_list, num_list, randomfile_local);
-    }
-}
-#endif				/* HAVE_DIRENT_H */
-
-/*-------------------------------------------------------------*/
 
 #ifdef DIFFUSE_COLOR
 static void
@@ -703,7 +646,7 @@ static void
 	text_length = index_dir(tp->words, (char *) " ");
 	if (text_length == 0)
 	    text_length = strlen(tp->words);
-	c_text = (char *) malloc(text_length);
+	if ((c_text = (char *) malloc(text_length)) != NULL)
 	strncpy(c_text, tp->words, text_length);
     }
     else
@@ -750,6 +693,7 @@ static void
 	    continue;
 	FTGlyphVectorizer & v = vec[i];
 	v.setPrecision(10.0);
+	/* PURIFY reports an Array Bounds Write on the next line */
 	if (!v.init(g))
 	    continue;
 
@@ -789,7 +733,7 @@ static void
     }
 
     if (!nosplit)
-	free(c_text);
+	(void) free((void *) c_text);
     if (size_x == 0.0)
     {
 	(void) fprintf(stderr, "Please give something to draw !\n");
@@ -1095,7 +1039,6 @@ static void
 void
  init_text3d(ModeInfo * mi)
 {
-    char *fontfile = (char *) NULL;
     int i;
     text3dstruct *tp;
 
@@ -1112,54 +1055,6 @@ void
     tp->rfreq = rfreq;
     tp->camera_dist = 0.0;
 
-    /* Get fontfile from mode_font (it can be a dir name) */
-    if (mode_font && strlen(mode_font))
-    {
-	fontfile = (char *) malloc(256);
-	(void) strncpy(fontfile, mode_font, 256);
-#if HAVE_DIRENT_H
-	getRandomFontFile(mode_font, fontfile);
-#endif
-    }
-
-    /* Check if fontfile exists */
-    if (!fontfile || !strlen(fontfile) || (fontfile && !readable(fontfile)))
-    {
-	(void) fprintf(stderr,
-		       "%s: could not read file \"%s\" !\n",
-		       MI_NAME(mi), fontfile);
-	(void) free((void *) text3d);
-	text3d = (text3dstruct *) NULL;
-#ifdef USE_BLANK
-	(void) fprintf(stderr,
-		       "%s: jumping to 'blank' mode.\n", MI_NAME(mi));
-	init_blank(mi);
-#endif
-	return;
-    }
-
-    tp->face = new FTFace;
-    if (!tp->face->open(fontfile))
-    {
-	(void) fprintf(stderr, "%s: unable to open True Type font %s !\n", MI_NAME(mi), fontfile);
-	delete tp->face;
-
-	(void) free((void *) text3d);
-	text3d = (text3dstruct *) NULL;
-#ifdef USE_BLANK
-	(void) fprintf(stderr,
-		       "%s: jumping to 'blank' mode.\n", MI_NAME(mi));
-	init_blank(mi);
-#endif
-	return;
-    }
-    if (MI_IS_DEBUG(mi))
-    {
-	(void) fprintf(stderr,
-		       "%s:\n\tfontfile=%s .\n", MI_NAME(mi), fontfile);
-    }
-    free(fontfile);
-
     /* Get animation function */
     tp->animation = 0;	/* Not found equals "Random" */
     tp->direction = (LRAND() & 1) ? 1 : -1; /* random direction */
@@ -1190,9 +1085,7 @@ void
 		   MI_NAME(mi), tp->animation, anim_names[tp->animation+1]);
     }
 
-    /* Initialize displayed string */
-    tp->words_start = tp->words =
-	text3d_newstr(getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi)));
+
     tp->counter = 0;
 
     if ((tp->glx_context = init_GL(mi)) != NULL)
@@ -1216,6 +1109,46 @@ void
     {
 	MI_CLEARWINDOW(mi);
     }
+	fontfile = getModeFont(mode_font);
+	if (!fontfile) {
+		release_text3d(mi);
+#ifdef USE_BLANK
+		(void) fprintf(stderr,
+			"%s: jumping to 'blank' mode.\n", MI_NAME(mi));
+		init_blank(mi);
+#endif
+		return;
+    }
+    tp->face = new FTFace;
+    if (!tp->face || !tp->face->open(fontfile)) {
+		(void) fprintf(stderr, "%s: unable to open True Type font %s !\n", MI_NAME(mi), fontfile);
+    	if (tp->face)
+			delete tp->face;
+		release_text3d(mi);
+#ifdef USE_BLANK
+		(void) fprintf(stderr,
+				"%s: jumping to 'blank' mode.\n", MI_NAME(mi));
+		init_blank(mi);
+#endif
+		return;
+    }
+    if (MI_IS_DEBUG(mi)) {
+		(void) fprintf(stderr,
+			"%s:\n\tfontfile=%s .\n", MI_NAME(mi), fontfile);
+    }
+
+    /* Do not free fontfile getModeFont handles potential leak */
+
+    /* Initialize displayed string */
+    tp->words_start = tp->words =
+	getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi));
+    if (MI_IS_DEBUG(mi))
+    {
+	(void) fprintf(stderr,
+		   "%s words:\n%s\n",
+		   MI_NAME(mi), tp->words);
+    }
+
 }
 
 /*
@@ -1254,10 +1187,8 @@ void
 	text_length = strlen(tp->words);
 	if (text_length == 0)
 	{
-	    (void) free((void *) tp->words_start);
 	    tp->words_start = tp->words =
-		text3d_newstr(getWords(MI_SCREEN(mi),
-				       MI_NUM_SCREENS(mi)));
+		getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi));
 	}
     }
     glDrawBuffer(GL_BACK);
@@ -1265,8 +1196,6 @@ void
 
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
-    glPushMatrix();
-
     Draw(tp, display, window);
     Animate(tp);
     if (MI_IS_DEBUG(mi))
@@ -1303,7 +1232,7 @@ void
 		delete tp->face;
 	}
 	(void) free((void *) text3d);
-	text3d = (text3dstruct *) NULL;
+	text3d = NULL;
     }
 #ifdef USE_BLANK
     else
