@@ -2,7 +2,7 @@
 /* hop --- real plane fractals */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)hop.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)hop.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -23,30 +23,31 @@ static const char sccsid[] = "@(#)hop.c	4.07 97/11/24 xlockmore";
  *
  * Revision History:
  * Changes in xlockmore distribution
- * 24-Jun-97: EJK and RR functions stolen from xmartin2.2
- *            Ed Kubaitis <ejk@ux2.cso.uiuc.edu> ejk functions and xmartin
- *            Renaldo Recuerdo rr function, generalized exponent version
+ * 01-Nov-2000: Allocation checks
+ * 24-Jun-1997: EJK and RR functions stolen from xmartin2.2
+ *              Ed Kubaitis <ejk@ux2.cso.uiuc.edu> ejk functions and xmartin
+ *              Renaldo Recuerdo rr function, generalized exponent version
  *              of the Barry Martin's square root function
- * 10-May-97: Compatible with xscreensaver
- * 27-Jul-95: added Peter de Jong's hop from Scientific American
- *            July 87 p. 111.  Sometimes they are amazing but there are a
- *            few duds (I did not see a pattern in the parameters).
- * 29-Mar-95: changed name from hopalong to hop
- * 09-Dec-94: added Barry Martin's sine hop
+ * 10-May-1997: Compatible with xscreensaver
+ * 27-Jul-1995: added Peter de Jong's hop from Scientific American
+ *              July 87 p. 111.  Sometimes they are amazing but there are a
+ *              few duds (I did not see a pattern in the parameters).
+ * 29-Mar-1995: changed name from hopalong to hop
+ * 09-Dec-1994: added Barry Martin's sine hop
  * Changes in original xlock
- * 29-Oct-90: fix bad (int) cast.
- * 29-Jul-90: support for multiple screens.
- * 08-Jul-90: new timing and colors and new algorithm for fractals.
- * 15-Dec-89: Fix for proper skipping of {White,Black}Pixel() in colors.
- * 08-Oct-89: Fixed long standing typo bug in RandomInitHop();
- *	      Fixed bug in memory allocation in init_hop();
- *	      Moved seconds() to an extern.
- *	      Got rid of the % mod since .mod is slow on a sparc.
- * 20-Sep-89: Lint.
- * 31-Aug-88: Forked from xlock.c for modularity.
- * 23-Mar-88: Coded HOPALONG routines from Scientific American Sept. 86 p. 14.
- *            Hopalong was attributed to Barry Martin of Aston University
- *            (Birmingham, England)
+ * 29-Oct-1990: fix bad (int) cast.
+ * 29-Jul-1990: support for multiple screens.
+ * 08-Jul-1990: new timing and colors and new algorithm for fractals.
+ * 15-Dec-1989: Fix for proper skipping of {White,Black}Pixel() in colors.
+ * 08-Oct-1989: Fixed long standing typo bug in RandomInitHop();
+ *	            Fixed bug in memory allocation in init_hop();
+ *	            Moved seconds() to an extern.
+ *	            Got rid of the % mod since .mod is slow on a sparc.
+ * 20-Sep-1989: Lint.
+ * 31-Aug-1988: Forked from xlock.c for modularity.
+ * 23-Mar-1988: Coded HOPALONG routines from Scientific American Sept. 86 p. 14.
+ *              Hopalong was attributed to Barry Martin of Aston University
+ *              (Birmingham, England)
  */
 
 
@@ -196,8 +197,8 @@ init_hop(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	GC          gc = MI_GC(mi);
-	hopstruct  *hp;
 	double      range;
+	hopstruct  *hp;
 
 	if (hops == NULL) {
 		if ((hops = (hopstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -380,8 +381,11 @@ init_hop(ModeInfo * mi)
 		hp->pix = NRAND(MI_NPIXELS(mi));
 	hp->bufsize = MI_COUNT(mi);
 
-	if (!hp->pointBuffer)
-		hp->pointBuffer = (XPoint *) malloc(hp->bufsize * sizeof (XPoint));
+	if (hp->pointBuffer == NULL) {
+		if ((hp->pointBuffer = (XPoint *) malloc(hp->bufsize *
+				sizeof (XPoint))) == NULL)
+			return;
+	}
 
 	MI_CLEARWINDOW(mi);
 
@@ -393,13 +397,20 @@ init_hop(ModeInfo * mi)
 void
 draw_hop(ModeInfo * mi)
 {
-	hopstruct  *hp = &hops[MI_SCREEN(mi)];
 	double      oldj, oldi;
-	XPoint     *xp = hp->pointBuffer;
-	int         k = hp->bufsize;
+	XPoint     *xp;
+	int         k;
+	hopstruct  *hp;
+
+	if (hops == NULL)
+		return;
+	hp = &hops[MI_SCREEN(mi)];
+	if (hp->pointBuffer == NULL)
+		return;
+	xp = hp->pointBuffer;
+	k = hp->bufsize;
 
 	MI_IS_DRAWN(mi) = True;
-
 	hp->inc++;
 	if (MI_NPIXELS(mi) > 2) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_PIXEL(mi, hp->pix));
@@ -534,10 +545,9 @@ release_hop(ModeInfo * mi)
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			hopstruct  *hp = &hops[screen];
 
-			if (hp->pointBuffer)
+			if (hp->pointBuffer != NULL)
 				(void) free((void *) hp->pointBuffer);
 		}
-
 		(void) free((void *) hops);
 		hops = NULL;
 	}
