@@ -212,6 +212,7 @@ ModStruct   juggle_description = {
 #define FIGURE1 7
 #define FIGURE2 3
 #define TRACE_LENGTH 50
+#define SPIN_DEGREES 750  /* Average spinning between a throw and the next catch */
 
 /* macros */
 
@@ -255,7 +256,8 @@ typedef struct trajectory {
 
   /* LinkedAction */
   int color;
-  int spin, degree_offset, divisions;
+  int spin, divisions;
+  double degree_offset;
   TrajectoryPtr balllink;
   TrajectoryPtr handlink;
 
@@ -359,13 +361,21 @@ static patternstruct portfolio[] = {
 
 /* t receives trajectory to be created.  ot must point to an existing
    trajectory or be identical to t to start a new list. */
-#define INSERT_AFTER(t, ot, ret)					\
+#define INSERT_AFTER_TOP(t, ot)					\
   if ((t = (Trajectory *)calloc(1, sizeof(Trajectory))) == NULL) \
-    {free_juggle(sp); return ret;}				\
+    {free_juggle(sp); return;}					\
   (t)->next = (ot)->next;					\
   (t)->prev = (ot);						\
   (ot)->next = (t);						\
   (t)->next->prev = (t)
+#define INSERT_AFTER(t, ot)					\
+  if ((t = (Trajectory *)calloc(1, sizeof(Trajectory))) == NULL) \
+    {free_juggle(sp); return False;}				\
+  (t)->next = (ot)->next;					\
+  (t)->prev = (ot);						\
+  (ot)->next = (t);						\
+  (t)->next->prev = (t)
+
 
 /* t must point to an existing trajectory.  t must not be an
    expression ending ->next or ->prev */
@@ -402,7 +412,7 @@ add_throw(jugglestruct *sp, char type, int h, Notation n)
 {
   Trajectory *t;
 
-  INSERT_AFTER(t, sp->head->prev, False);
+  INSERT_AFTER(t, sp->head->prev);
   t->posn = type;
   if (n == ADAM) {
 	t->adam = h;
@@ -567,7 +577,7 @@ part(jugglestruct *sp)
 		p = p->prev; /* early throw */
 	  }
 	  t->action = CATCH;
-	  INSERT_AFTER(nt, p, False);
+	  INSERT_AFTER(nt, p);
 	  nt->status = ACTION;
 	  nt->action = THROW;
 	  nt->height = t->height;
@@ -794,7 +804,7 @@ projectile(jugglestruct *sp)
 			t->dx = (t->balllink->x - t->x) / tc;
 
 			/* ball follows parabola down */
-			INSERT_AFTER(n, t->prev, False);
+			INSERT_AFTER(n, t->prev);
 			n->start = t->start;
 			n->finish = (int) (t->start + tb);
 			n->type = Ball;
@@ -819,7 +829,7 @@ projectile(jugglestruct *sp)
 
 
 			/* ball follows parabola up */
-			INSERT_AFTER(n, t->prev, False);
+			INSERT_AFTER(n, t->prev);
 			n->start = (int) (t0 + tb);
 			n->finish = (int) (t0 + tc);
 			n->type = Ball;
@@ -850,7 +860,7 @@ projectile(jugglestruct *sp)
 		  double t0, dt;
 
 		  /* ball follows parabola */
-		  INSERT_AFTER(n, t->prev, False);
+		  INSERT_AFTER(n, t->prev);
 		  n->start = t->start;
 		  n->finish = t->balllink->start;
 		  n->type = Ball;
@@ -1091,6 +1101,23 @@ static int compare_num_balls(const void *p1, const void *p2)
 
 /* Public functions */
 
+void
+release_juggle(ModeInfo * mi)
+{
+	if (juggles != NULL) {
+		int screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_juggle(&juggles[screen]);
+		(void) free((void *) juggles);
+		juggles = NULL;
+	}
+	if (patternindex != NULL) {
+		(void) free((void *) patternindex);
+		patternindex = NULL;
+	}
+}
+
 void
 init_juggle(ModeInfo * mi)
 {
@@ -1168,7 +1195,9 @@ init_juggle(ModeInfo * mi)
 
 	sp->width = MI_WIDTH(mi);
 	sp->height = MI_HEIGHT(mi);
-	sp->count = MI_COUNT(mi);
+	sp->count = ABS(MI_COUNT(mi));
+	if (sp->count == 0)
+		sp->count = 150;
 	sp->scale = sp->height / 480.0;
 	/* vary x a little so the juggler does not burn the screen */
 	sp->cx = sp->width / 2 + RFX + NRAND(LFX - RFX + 1);
@@ -1226,7 +1255,7 @@ init_juggle(ModeInfo * mi)
 	free_pattern(sp);
 
 	/* create circular list */
-	INSERT_AFTER(sp->head, sp->head, /**/);
+	INSERT_AFTER_TOP(sp->head, sp->head);
 
 	/* generate pattern */
 	if (pattern == NULL) {
@@ -1246,7 +1275,12 @@ init_juggle(ModeInfo * mi)
 
 		{ /* vary number of balls */
 		  int new_balls = num_balls;
-		  int change = NRAND(2 + CHANGE_BIAS);
+		  int change;
+
+		  if (new_balls == 2) /* Do not juggle 2 that often */
+		    change = NRAND(2 + CHANGE_BIAS / 4);
+		  else
+		    change = NRAND(2 + CHANGE_BIAS);
 		  switch (change) {
 		  case 0:
 			new_balls++;
@@ -1364,7 +1398,7 @@ init_juggle(ModeInfo * mi)
 #endif
 
 static void
-draw_juggle_ball(ModeInfo *mi, unsigned long color, int x, int y, int degree_offset, int divisions)
+draw_juggle_ball(ModeInfo *mi, unsigned long color, int x, int y, double degree_offset, int divisions)
 {
 	Display    *dpy = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
@@ -1381,7 +1415,7 @@ draw_juggle_ball(ModeInfo *mi, unsigned long color, int x, int y, int degree_off
 			0, 23040);
 		return;
 	}
-	offset = degree_offset << 6; /* multiply by 64 */
+	offset = (int) (degree_offset * 64);
 	if (divisions == 4) { /* 90 degree divisions */
 		XFillArc(dpy, window, gc,
 			x - BALLRADIUS, y - BALLRADIUS,
@@ -1476,7 +1510,12 @@ draw_juggle(ModeInfo * mi)
 			ERASE_BALL(traj->x, traj->y);
 		  }
 		  draw_juggle_ball(mi, color, x, y, traj->degree_offset, traj->divisions);
-		  traj->degree_offset = ((traj->degree_offset + 360) + 5 * traj->spin) % 360;
+		  traj->degree_offset = traj->degree_offset +
+		    SPIN_DEGREES * traj->spin / sp->count;
+		  if (traj->degree_offset < 0.0)
+			traj->degree_offset += 360.0;
+		  else if (traj->degree_offset >= 360.0)
+			traj->degree_offset -= 360.0;
 		}
 		traj->x = x;
 		traj->y = y;
@@ -1502,21 +1541,4 @@ draw_juggle(ModeInfo * mi)
 
 }
 
-void
-release_juggle(ModeInfo * mi)
-{
-	if (juggles != NULL) {
-		int screen;
-
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
-			free_juggle(&juggles[screen]);
-		(void) free((void *) juggles);
-		juggles = NULL;
-	}
-	if (patternindex != NULL) {
-		(void) free((void *) patternindex);
-		patternindex = NULL;
-	}
-}
-
 #endif /* MODE_juggle */
