@@ -394,8 +394,7 @@ initialize_maze(ModeInfo * mi)
 	if (MI_NPIXELS(mi) <= 2) {
 		mp->color = MI_WHITE_PIXEL(mi);
 	} else {
-		i = NRAND(MI_NPIXELS(mi));
-		mp->color = MI_PIXEL(mi, i);
+		mp->color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	}
 	XSetForeground(display, mp->backGC, mp->color);
 	XSetForeground(display, mp->grayGC, mp->color);
