@@ -5,7 +5,7 @@
  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)nose.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)nose.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -23,15 +23,16 @@ static const char sccsid[] = "@(#)nose.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 06-Jun-97: Compatible with xscreensaver and now colorized (idea from
- *            xscreensaver but colors are random).
- * 27-Feb-96: Added new ModeInfo arg to init and callback hooks.  Removed
- *		references to onepause, now uses MI_PAUSE(mi) interface.
- *		Ron Hitchens <ron@idiom.com>
- * 10-Oct-95: A better way of handling fortunes from a file, thanks to
- *            Jouk Jansen <joukj@hrem.stm.tudelft.nl>.
- * 21-Sep-95: font option added, debugged for multiscreens
- * 12-Aug-95: xlock version
+ * 01-Nov-2000: Allocation checks
+ * 06-Jun-1997: Compatible with xscreensaver and now colorized (idea from
+ *              xscreensaver but colors are random).
+ * 27-Feb-1996: Added new ModeInfo arg to init and callback hooks.  Removed
+ *              references to onepause, now uses MI_PAUSE(mi) interface.
+ *              Ron Hitchens <ron@idiom.com>
+ * 10-Oct-1995: A better way of handling fortunes from a file, thanks to
+ *              Jouk Jansen <joukj@hrem.stm.tudelft.nl>.
+ * 21-Sep-1995: font option added, debugged for multiscreens
+ * 12-Aug-1995: xlock version
  * 1992: xscreensaver version, noseguy (Jamie Zawinski <jwz@jwz.org>)
  * 1990: X11 version, xnlock (Dan Heller <argv@sun.com>)
  */
@@ -138,15 +139,13 @@ typedef struct {
 
 static nosestruct *noses = NULL;
 
-#ifndef USE_MB
-static XFontStruct *mode_font = None;
-#else
+#ifdef USE_MB
 static XFontSet mode_font = None;
+#else
+static XFontStruct *mode_font = None;
 #endif
 
-#ifndef USE_MB
-#define font_height(f) ((f == None) ? 8 : f->ascent + f->descent)
-#else
+#ifdef USE_MB
 static int font_height(XFontSet f) {
   XRectangle ink, log;
   if (f == None) {
@@ -156,6 +155,8 @@ static int font_height(XFontSet f) {
 	return log.height;
   }
 }
+#else
+#define font_height(f) ((f == None) ? 8 : f->ascent + f->descent)
 #endif
 
 #define LEFT 001
@@ -172,6 +173,31 @@ XSetStipple(d,g,p); XSetTSOrigin(d,g,x,y);\
 XFillRectangle(d,np,g,x,y,w,h)
 
 static void
+free_nose(Display *display, nosestruct *np)
+{
+	int         pix;
+
+	if (np->text_fg_gc != None) {
+		XFreeGC(display, np->text_fg_gc);
+	    np->text_fg_gc = None;
+	}
+	if (np->text_bg_gc != None) {
+		XFreeGC(display, np->text_bg_gc);
+	    np->text_bg_gc = None;
+	}
+	for (pix = 0; pix < PIXMAPS; pix++) {
+		if (np->position[pix] != None) {
+			XFreePixmap(display, np->position[pix]);
+	   		np->position[pix] = None;
+		}
+		if (np->noseGC[pix] != None) {
+			XFreeGC(display, np->noseGC[pix]);
+	   		np->noseGC[pix] = None;
+		}
+	}
+}
+
+static Bool
 pickClothes(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
@@ -192,35 +218,48 @@ pickClothes(ModeInfo * mi)
 	gcv.foreground = MI_BLACK_PIXEL(mi);
 	gcv.background = MI_BLACK_PIXEL(mi);
 	for (i = 0; i < PIXMAPS; i++) {
-		np->position[i] = XCreatePixmap(display, window, PIXMAP_SIZE, PIXMAP_SIZE,
-						MI_DEPTH(mi));
-		np->noseGC[i] = XCreateGC(display, np->position[i],
-		    GCForeground | GCBackground | GCGraphicsExposures, &gcv);
+		if ((np->position[i] = XCreatePixmap(display, window,
+				PIXMAP_SIZE, PIXMAP_SIZE, MI_DEPTH(mi))) == None)
+			return False;
+		if ((np->noseGC[i] = XCreateGC(display, np->position[i],
+				GCForeground | GCBackground | GCGraphicsExposures,
+				&gcv)) == None)
+			return False;
 		XFillRectangle(display, np->position[i], np->noseGC[i],
 			       0, 0, PIXMAP_SIZE, PIXMAP_SIZE);
 	}
 	XSetBackground(display, gc, MI_BLACK_PIXEL(mi));
 	XSetFillStyle(display, gc, FillStippled);
 	/* DOWN NOSE GUY */
-	hat_pix = XCreateBitmapFromData(display, window,
-					(char *) nose_hat_down_bits,
-				  nose_hat_down_width, nose_hat_down_height);
-	face_pix = XCreateBitmapFromData(display, window,
-					 (char *) nose_face_down_bits,
-				nose_face_down_width, nose_face_down_height);
-	shoe_pix = XCreateBitmapFromData(display, window,
-					 (char *) nose_shoe_front_bits,
-			      nose_shoe_front_width, nose_shoe_front_height);
+	if ((shoe_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_shoe_front_bits,
+			nose_shoe_front_width, nose_shoe_front_height)) == None){
+		return False;
+	}
 	COPY(display, gc, shoe_color, shoe_pix, np->position[D],
 	     (PIXMAP_SIZE - nose_shoe_front_width) / 2,
 	     nose_hat_height + nose_face_front_height + 3,
 	     nose_shoe_front_width, nose_shoe_front_height);
+	if ((face_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_face_down_bits,
+			nose_face_down_width, nose_face_down_height)) == None) {
+		XFreePixmap(display, shoe_pix);
+		return False;
+	}
 	COPY(display, gc, face_color, face_pix, np->position[D],
 	  (PIXMAP_SIZE - nose_face_down_width) / 2, nose_hat_down_height + 7,
 	     nose_face_down_width, nose_face_down_height);
+	XFreePixmap(display, face_pix);
+	if ((hat_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_hat_down_bits,
+			nose_hat_down_width, nose_hat_down_height)) == None) {
+		XFreePixmap(display, shoe_pix);
+		return False;
+	}
 	COPY(display, gc, hat_color, hat_pix, np->position[D],
 	     (PIXMAP_SIZE - nose_hat_down_width) / 2, 7,
 	     nose_hat_down_width, nose_hat_down_height);
+	XFreePixmap(display, hat_pix);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
 		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
@@ -228,19 +267,17 @@ pickClothes(ModeInfo * mi)
 			       0, nose_hat_down_height + 6, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
 	}
-	XFreePixmap(display, face_pix);
-	XFreePixmap(display, hat_pix);
 	/* FRONT NOSE GUY */
-	hat_pix = XCreateBitmapFromData(display, window,
-					(char *) nose_hat_bits,
-					nose_hat_width, nose_hat_height);
-	face_pix = XCreateBitmapFromData(display, window,
-					 (char *) nose_face_front_bits,
-			      nose_face_front_width, nose_face_front_height);
 	COPY(display, gc, shoe_color, shoe_pix, np->position[F],
 	     (PIXMAP_SIZE - nose_shoe_front_width) / 2,
 	     nose_hat_height + nose_face_front_height + 3,
 	     nose_shoe_front_width, nose_shoe_front_height);
+	XFreePixmap(display, shoe_pix);
+	if ((hat_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_hat_bits,
+			nose_hat_width, nose_hat_height)) == None) {
+		return False;
+	}
 	COPY(display, gc, hat_color, hat_pix, np->position[F],
 	     (PIXMAP_SIZE - nose_hat_width) / 2, 4,
 	     nose_hat_width, nose_hat_height);
@@ -251,14 +288,23 @@ pickClothes(ModeInfo * mi)
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
 	}
+	if ((face_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_face_front_bits,
+			nose_face_front_width, nose_face_front_height)) == None) {
+		XFreePixmap(display, hat_pix);
+		return False;
+	}
 	COPY(display, gc, face_color, face_pix, np->position[F],
 	     (PIXMAP_SIZE - nose_face_front_width) / 2, nose_hat_height + 1,
 	     nose_face_front_width, nose_face_front_height);
-	XFreePixmap(display, shoe_pix);
 	/* FRONT LEFT NOSE GUY */
-	shoel_pix = XCreateBitmapFromData(display, window,
-					  (char *) nose_shoe_left_bits,
-				nose_shoe_left_width, nose_shoe_left_height);
+	if ((shoel_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_shoe_left_bits,
+			nose_shoe_left_width, nose_shoe_left_height)) == None) {
+		XFreePixmap(display, hat_pix);
+		XFreePixmap(display, face_pix);
+		return False;
+	}
 	COPY(display, gc, shoe_color, shoel_pix, np->position[LF],
 	     (PIXMAP_SIZE - nose_shoe_left_width) / 2 - 4,
 	     nose_hat_height + nose_face_front_height + 3,
@@ -277,9 +323,14 @@ pickClothes(ModeInfo * mi)
 	     (PIXMAP_SIZE - nose_face_front_width) / 2, nose_hat_height + 1,
 	     nose_face_front_width, nose_face_front_height);
 	/* FRONT RIGHT NOSE GUY */
-	shoer_pix = XCreateBitmapFromData(display, window,
-					  (char *) nose_shoe_right_bits,
-			      nose_shoe_right_width, nose_shoe_right_height);
+	if ((shoer_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_shoe_right_bits,
+			nose_shoe_right_width, nose_shoe_right_height)) == None) {
+		XFreePixmap(display, hat_pix);
+		XFreePixmap(display, shoel_pix);
+		XFreePixmap(display, face_pix);
+		return False;
+	}
 	COPY(display, gc, shoe_color, shoer_pix, np->position[RF],
 	     (PIXMAP_SIZE - nose_shoe_right_width) / 2 + 4,
 	     nose_hat_height + nose_face_front_height + 3,
@@ -299,13 +350,11 @@ pickClothes(ModeInfo * mi)
 	     nose_face_front_width, nose_face_front_height);
 	XFreePixmap(display, face_pix);
 	/* LEFT NOSE GUY */
-	face_pix = XCreateBitmapFromData(display, window,
-					 (char *) nose_face_left_bits,
-				nose_face_left_width, nose_face_left_height);
 	COPY(display, gc, shoe_color, shoel_pix, np->position[L],
 	     (PIXMAP_SIZE - nose_shoe_left_width) / 2 - 4,
 	     nose_hat_height + nose_face_front_height + 3,
 	     nose_shoe_left_width, nose_shoe_left_height);
+	XFreePixmap(display, shoel_pix);
 	COPY(display, gc, hat_color, hat_pix, np->position[L],
 	     (PIXMAP_SIZE - nose_hat_width) / 2, 4,
 	     nose_hat_width, nose_hat_height);
@@ -316,18 +365,30 @@ pickClothes(ModeInfo * mi)
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
 	}
+	if ((face_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_face_left_bits,
+			nose_face_left_width, nose_face_left_height)) == None) {
+		XFreePixmap(display, hat_pix);
+		XFreePixmap(display, shoer_pix);
+		return False;
+	}
 	COPY(display, gc, face_color, face_pix, np->position[L],
 	   (PIXMAP_SIZE - nose_face_left_width) / 2 - 4, nose_hat_height + 4,
 	     nose_face_left_width, nose_face_left_height);
-	XFreePixmap(display, shoel_pix);
 	/* LEFT NOSE GUY STEPPING */
-	shoel_pix = XCreateBitmapFromData(display, window,
-					  (char *) nose_step_left_bits,
-				nose_step_left_width, nose_step_left_height);
+	if ((shoel_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_step_left_bits,
+			nose_step_left_width, nose_step_left_height)) == None) {
+		XFreePixmap(display, hat_pix);
+		XFreePixmap(display, face_pix);
+		XFreePixmap(display, shoer_pix);
+		return False;
+	}
 	COPY(display, gc, shoe_color, shoel_pix, np->position[LSTEP],
 	     (PIXMAP_SIZE - nose_step_left_width) / 2,
 	     nose_hat_height + nose_face_front_height - 1,
 	     nose_step_left_width, nose_step_left_height);
+	XFreePixmap(display, shoel_pix);
 	COPY(display, gc, hat_color, hat_pix, np->position[LSTEP],
 	     (PIXMAP_SIZE - nose_hat_width) / 2, 4,
 	     nose_hat_width, nose_hat_height);
@@ -342,15 +403,19 @@ pickClothes(ModeInfo * mi)
 	   (PIXMAP_SIZE - nose_face_left_width) / 2 - 4, nose_hat_height + 4,
 	     nose_face_left_width, nose_face_left_height);
 	XFreePixmap(display, face_pix);
-	XFreePixmap(display, shoel_pix);
 	/* RIGHT NOSE GUY */
-	face_pix = XCreateBitmapFromData(display, window,
-					 (char *) nose_face_right_bits,
-			      nose_face_right_width, nose_face_right_height);
+	if ((face_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_face_right_bits,
+			nose_face_right_width, nose_face_right_height)) == None) {
+		XFreePixmap(display, hat_pix);
+		XFreePixmap(display, shoer_pix);
+		return False;
+	}
 	COPY(display, gc, shoe_color, shoer_pix, np->position[R],
 	     (PIXMAP_SIZE - nose_shoe_right_width) / 2 + 4,
 	     nose_hat_height + nose_face_front_height + 3,
 	     nose_shoe_right_width, nose_shoe_right_height);
+	XFreePixmap(display, shoer_pix);
 	COPY(display, gc, hat_color, hat_pix, np->position[R],
 	     (PIXMAP_SIZE - nose_hat_width) / 2, 4,
 	     nose_hat_width, nose_hat_height);
@@ -364,18 +429,23 @@ pickClothes(ModeInfo * mi)
 	COPY(display, gc, face_color, face_pix, np->position[R],
 	  (PIXMAP_SIZE - nose_face_right_width) / 2 + 4, nose_hat_height + 4,
 	     nose_face_right_width, nose_face_right_height);
-	XFreePixmap(display, shoer_pix);
 	/* RIGHT NOSE GUY STEPPING */
-	shoer_pix = XCreateBitmapFromData(display, window,
-					  (char *) nose_step_right_bits,
-			      nose_step_right_width, nose_step_right_height);
+	if ((shoer_pix = XCreateBitmapFromData(display, window,
+			(char *) nose_step_right_bits,
+			nose_step_right_width, nose_step_right_height)) == None) {
+		XFreePixmap(display, face_pix);
+		XFreePixmap(display, hat_pix);
+		return False;
+	}
 	COPY(display, gc, shoe_color, shoer_pix, np->position[RSTEP],
 	     (PIXMAP_SIZE - nose_step_right_width) / 2,
 	     nose_hat_height + nose_face_front_height - 1,
 	     nose_step_right_width, nose_step_right_height);
+	XFreePixmap(display, shoer_pix);
 	COPY(display, gc, hat_color, hat_pix, np->position[RSTEP],
 	     (PIXMAP_SIZE - nose_hat_width) / 2, 4,
 	     nose_hat_width, nose_hat_height);
+	XFreePixmap(display, hat_pix);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
 		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
@@ -387,9 +457,8 @@ pickClothes(ModeInfo * mi)
 	  (PIXMAP_SIZE - nose_face_right_width) / 2 + 4, nose_hat_height + 4,
 	     nose_face_right_width, nose_face_right_height);
 	XFreePixmap(display, face_pix);
-	XFreePixmap(display, shoer_pix);
-	XFreePixmap(display, hat_pix);
 	XSetFillStyle(display, gc, FillSolid);
+	return True;
 }
 
 static void
@@ -515,11 +584,11 @@ talk(ModeInfo * mi, Bool force_erase)
 		if (mode_font == None)
 			width = 8;
 		else {
-#ifndef USE_MB
-			width = XTextWidth(mode_font, np->words, total);
-#else
+#ifdef USE_MB
 			XmbTextExtents(mode_font, np->words, total, &ink, &log);
 			width = ink.width;
+#else
+			width = XTextWidth(mode_font, np->words, total);
 #endif
 		}
 		height = 0;
@@ -529,13 +598,14 @@ talk(ModeInfo * mi, Bool force_erase)
 			int         w;
 
 			*p2 = 0;
-#ifndef USE_MB
-			if (mode_font != None && (w = XTextWidth(mode_font, p, p2 - p)) > width)
-				width = w;
-#else
+#ifdef USE_MB
 			XmbTextExtents(mode_font, p, p2 - p, &ink, &log);
 			if (mode_font != None && (w = log.width) > width)
 			  width = w;
+#else
+			if (mode_font != None &&
+					(w = XTextWidth(mode_font, p, p2 - p)) > width)
+				width = w;
 #endif
 			total += p2 - p;	/* total chars; count to determine reading time */
 			(void) strncpy(args[height], p, MAXWIDTH - 1);
@@ -577,12 +647,12 @@ talk(ModeInfo * mi, Bool force_erase)
 
 	/* now print each string in reverse order (start at bottom of box) */
 	for (Z = 0; Z < height; Z++) {
-#ifndef USE_MB
+#ifdef USE_MB
+		(void) XmbDrawString(display, window, mode_font, np->text_fg_gc,
+			 np->s.x + 15, np->s.y + Y, args[Z], strlen(args[Z]));
+#else
 		(void) XDrawString(display, window, np->text_fg_gc,
 			np->s.x + 15, np->s.y + Y, args[Z], strlen(args[Z]));
-#else
-		(void) XmbDrawString(display, window, mode_font, np->text_fg_gc,
-							 np->s.x + 15, np->s.y + Y, args[Z], strlen(args[Z]));
 #endif
 		Y += font_height(mode_font);
 	}
@@ -701,9 +771,8 @@ init_nose(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
-	nosestruct *np;
 	XGCValues   gcv;
-	int         i = 0;
+	nosestruct *np;
 
 	if (noses == NULL) {
 		if ((noses = (nosestruct *) calloc(MI_NUM_SCREENS(mi),
@@ -722,18 +791,21 @@ init_nose(ModeInfo * mi)
 
 	XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
-	/* don't want any exposure events from XCopyPlane */
+	/* do not want any exposure events from XCopyPlane */
 	XSetGraphicsExposures(display, gc, False);
 
 	if (mode_font == None) {
-#ifndef USE_MB
-		mode_font = getFont(display);
-#else
+#ifdef USE_MB
 		mode_font = getFontSet(display);
+#else
+		mode_font = getFont(display);
 #endif
 	}
-	if (np->noseGC[0] == NULL)
-		pickClothes(mi);
+	if (np->noseGC[0] == None)
+		if (!pickClothes(mi)) {
+			free_nose(display, np);
+			return;
+		}
 	np->words = getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi));
 	if (np->text_fg_gc == NULL && mode_font != None) {
 #ifndef USE_MB
@@ -743,29 +815,36 @@ init_nose(ModeInfo * mi)
 		gcv.graphics_exposures = False;
 		gcv.foreground = MI_WHITE_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
-#ifndef USE_MB
-		np->text_fg_gc = XCreateGC(display, window,
-					   GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
+		if ((np->text_fg_gc = XCreateGC(display, window,
+#ifdef USE_MB
+				 GCForeground | GCBackground | GCGraphicsExposures,
 #else
-		np->text_fg_gc = XCreateGC(display, window,
-					   GCForeground | GCBackground | GCGraphicsExposures, &gcv);
+				 GCForeground | GCBackground | GCGraphicsExposures | GCFont,
 #endif
+				 &gcv)) == None) {
+			free_nose(display, np);
+			return;
+		}
 		gcv.foreground = MI_BLACK_PIXEL(mi);
 		gcv.background = MI_WHITE_PIXEL(mi);
-#ifndef USE_MB
-		np->text_bg_gc = XCreateGC(display, window,
-					   GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
+		if ((np->text_bg_gc = XCreateGC(display, window,
+#ifdef USE_MB
+				 GCForeground | GCBackground | GCGraphicsExposures,
 #else
-		np->text_bg_gc = XCreateGC(display, window,
-					   GCForeground | GCBackground | GCGraphicsExposures, &gcv);
+				 GCForeground | GCBackground | GCGraphicsExposures | GCFont,
 #endif
+				 &gcv)) == None) {
+			free_nose(display, np);
+			return;
+		}
 	}
 	np->up = 1;
 	if (np->tinymode) {
+		int pos = NRAND(PIXMAPS);
+
 		np->x = 0;
 		np->y = 0;
-		i = (NRAND(PIXMAPS));
-		XCopyArea(display, np->position[i], window, np->noseGC[i],
+		XCopyArea(display, np->position[pos], window, np->noseGC[pos],
 			  0, 0, PIXMAP_SIZE, PIXMAP_SIZE,
 			  (np->width - PIXMAP_SIZE) / 2,
 			  (np->height - PIXMAP_SIZE) / 2);
@@ -781,10 +860,15 @@ init_nose(ModeInfo * mi)
 void
 draw_nose(ModeInfo * mi)
 {
-	nosestruct *np = &noses[MI_SCREEN(mi)];
+	nosestruct *np;
+
+	if (noses == NULL)
+		return;
+	np = &noses[MI_SCREEN(mi)];
+	if (np->noseGC[0] == None)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	if (np->busyLoop > 0) {
 		np->busyLoop--;
 		return;
@@ -805,30 +889,16 @@ release_nose(ModeInfo * mi)
 	if (noses != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			Display    *display = MI_DISPLAY(mi);
-			nosestruct *np = &noses[screen];
-			int         i;
-
-			if (np->text_fg_gc != NULL)
-				XFreeGC(display, np->text_fg_gc);
-			if (np->text_bg_gc != NULL)
-				XFreeGC(display, np->text_bg_gc);
-			for (i = 0; i < PIXMAPS; i++) {
-				if (np->position[i])
-					XFreePixmap(display, np->position[i]);
-				if (np->noseGC[i] != NULL)
-					XFreeGC(display, np->noseGC[i]);
-			}
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_nose(MI_DISPLAY(mi), &noses[screen]);
 		(void) free((void *) noses);
 		noses = NULL;
 	}
 	if (mode_font != None) {
-#ifndef USE_MB
-		XFreeFont(MI_DISPLAY(mi), mode_font);
-#else
+#ifdef USE_MB
 		XFreeFontSet(MI_DISPLAY(mi), mode_font);
+#else
+		XFreeFont(MI_DISPLAY(mi), mode_font);
 #endif
 		mode_font = None;
 	}
