@@ -2,7 +2,7 @@
 /* pacman --- Mr. Pacman and his ghost friends */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)pacman.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)pacman.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,7 +22,8 @@ static const char sccsid[] = "@(#)pacman.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 04-Jun-97: Compatible with xscreensaver
+ * 01-Nov-2000: Allocation checks
+ * 04-Jun-1997: Compatible with xscreensaver
  *
  */
 
@@ -119,6 +120,37 @@ typedef struct {
 
 static pacmangamestruct *pacmangames = NULL;
 
+static void
+free_pacman(Display *display, pacmangamestruct *pp)
+{
+	int         dir, mouth;
+
+	if (pp->ghosts != NULL) {
+		(void) free((void *) pp->ghosts);
+		pp->ghosts = NULL;
+	}
+#ifdef DEFUNCT
+	if (pp->eaten != NULL) {
+		(void) free((void *) pp->eaten);
+		pp->eaten = NULL;
+	}
+#endif
+	if (pp->stippledGC != None) {
+		XFreeGC(display, pp->stippledGC);
+		pp->stippledGC = None;
+	}
+	if (pp->ghostPixmap != None) {
+		XFreePixmap(display, pp->ghostPixmap);
+		pp->ghostPixmap = None;
+	}
+	for (dir = 0; dir < 4; dir++)
+		for (mouth = 0; mouth < MAXMOUTH; mouth++)
+			if (pp->pacmanPixmap[dir][mouth] != None) {
+				XFreePixmap(display, pp->pacmanPixmap[dir][mouth]);
+				pp->pacmanPixmap[dir][mouth] = None;
+			}
+}
+
 #if DEFUNCT
 static void
 clearcorners(ModeInfo * mi)
@@ -162,6 +194,7 @@ clearcorners(ModeInfo * mi)
 	}
 }
 #endif
+
 static void
 repopulate(ModeInfo * mi)
 {
@@ -171,9 +204,9 @@ repopulate(ModeInfo * mi)
 #if DEFUNCT
 	if (pp->eaten)
 		(void) free((void *) pp->eaten);
-	pp->eaten = (unsigned int *) calloc((pp->nrows * pp->ncols),
-					    sizeof (unsigned int));
-
+	if ((pp->eaten = (unsigned int *) calloc((pp->nrows * pp->ncols),
+			sizeof (unsigned int))) == NULL)
+		return;
 #endif
 
 	MI_CLEARWINDOW(mi);
@@ -210,7 +243,6 @@ movepac(ModeInfo * mi)
 
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
-	GC          gc = MI_GC(mi);
 	pacmangamestruct *pp = &pacmangames[MI_SCREEN(mi)];
 	int         ghost, alldead, dir;
 	XPoint      delta;
@@ -265,7 +297,8 @@ movepac(ModeInfo * mi)
 
 	p.oldcf = pp->pacman.col * pp->xs + pp->xb;
 	p.oldrf = pp->pacman.row * pp->ys + pp->yb;
-	g = (being *) malloc(pp->nghosts * sizeof (being));
+	if ((g = (being *) malloc(pp->nghosts * sizeof (being))) == NULL)
+		return;
 
 	for (ghost = 0; ghost < pp->nghosts; ghost++) {
 		if (pp->ghosts[ghost].dead == 0) {
@@ -451,6 +484,7 @@ init_pacman(ModeInfo * mi)
 			return;
 	}
 	pp = &pacmangames[MI_SCREEN(mi)];
+
 	pp->width = MI_WIDTH(mi);
 	pp->height = MI_HEIGHT(mi);
 	if (pp->ghostPixmap != None) {
@@ -503,16 +537,26 @@ init_pacman(ModeInfo * mi)
 	pp->yb = pp->height - pp->nrows * pp->ys;
 
 	if (pp->pixelmode) {
-        pp->ghostPixmap = XCreatePixmap(display, window,
-            pp->xs, pp->ys, 1);
+        if ((pp->ghostPixmap = XCreatePixmap(display, window,
+           		pp->xs, pp->ys, 1)) == None) {
+			free_pacman(display, pp);
+			return;
+		}
         gcv.foreground = 0;
         gcv.background = 1;
-        bg_gc = XCreateGC(display, pp->ghostPixmap,
-			GCForeground | GCBackground, &gcv);
+        if ((bg_gc = XCreateGC(display, pp->ghostPixmap,
+			GCForeground | GCBackground, &gcv)) == None) {
+			free_pacman(display, pp);
+			return;
+		}
         gcv.foreground = 1;
         gcv.background = 0;
-        fg_gc = XCreateGC(display, pp->ghostPixmap,
-			GCForeground | GCBackground, &gcv);
+        if ((fg_gc = XCreateGC(display, pp->ghostPixmap,
+			GCForeground | GCBackground, &gcv)) == None) {
+        	XFreeGC(display, bg_gc);
+			free_pacman(display, pp);
+			return;
+		}
         XFillRectangle(display, pp->ghostPixmap, bg_gc,
             0, 0, pp->xs, pp->ys);
 		XFillArc(display, pp->ghostPixmap, fg_gc,
@@ -526,8 +570,10 @@ init_pacman(ModeInfo * mi)
 		gcv.foreground = MI_BLACK_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
 		if ((pp->stippledGC = XCreateGC(display, window,
-				GCForeground | GCBackground, &gcv)) == None)
+				GCForeground | GCBackground, &gcv)) == None) {
+			free_pacman(display, pp);
 			return;
+		}
 	}
 	if (pp->pacmanPixmap[0][0] != None)
 		for (dir = 0; dir < 4; dir++)
@@ -536,16 +582,26 @@ init_pacman(ModeInfo * mi)
 
 	for (dir = 0; dir < 4; dir++)
 		for (mouth = 0; mouth < MAXMOUTH; mouth++) {
-			pp->pacmanPixmap[dir][mouth] = XCreatePixmap(display, MI_WINDOW(mi),
-							  pp->xs, pp->ys, 1);
+			if ((pp->pacmanPixmap[dir][mouth] = XCreatePixmap(display,
+					 MI_WINDOW(mi), pp->xs, pp->ys, 1)) == None) {
+				free_pacman(display, pp);
+				return;
+			}
 			gcv.foreground = 1;
         	gcv.background = 0;
-			fg_gc = XCreateGC(display, pp->pacmanPixmap[dir][mouth],
-				GCForeground | GCBackground, &gcv);
+			if ((fg_gc = XCreateGC(display, pp->pacmanPixmap[dir][mouth],
+					GCForeground | GCBackground, &gcv)) == None) {
+				free_pacman(display, pp);
+				return;
+			}
 			gcv.foreground = 0;
         	gcv.background = 0;
-			bg_gc = XCreateGC(display, pp->pacmanPixmap[dir][mouth],
-				GCForeground | GCBackground, &gcv);
+			if ((bg_gc = XCreateGC(display, pp->pacmanPixmap[dir][mouth],
+					GCForeground | GCBackground, &gcv)) == None) {
+        		XFreeGC(display, fg_gc);
+				free_pacman(display, pp);
+				return;
+			}
 			XFillRectangle(display, pp->pacmanPixmap[dir][mouth], bg_gc,
 				0, 0, pp->xs, pp->ys);
 			if (pp->xs == 1 && pp->ys == 1)
@@ -574,7 +630,11 @@ init_pacman(ModeInfo * mi)
 		pp->nghosts = MINGHOSTS;
 
 	if (!pp->ghosts)
-		pp->ghosts = (beingstruct *) calloc(pp->nghosts, sizeof (beingstruct));
+		if ((pp->ghosts = (beingstruct *) calloc(pp->nghosts,
+				sizeof (beingstruct))) == NULL) {
+			free_pacman(display, pp);
+			return;
+		}
 	for (ghost = 0; ghost < pp->nghosts; ghost++)
 		pp->ghosts[ghost].nextbox = NOWHERE;
 
@@ -598,8 +658,11 @@ init_pacman(ModeInfo * mi)
 #if DEFUNCT
 	if (pp->eaten)
 		(void) free((void *) pp->eaten);
-	pp->eaten = (unsigned int *) malloc((pp->nrows * pp->ncols) *
-					    sizeof (unsigned int));
+	if ((pp->eaten = (unsigned int *) malloc((pp->nrows * pp->ncols) *
+					    sizeof (unsigned int))) == NULL) {
+		free_pacman(display, pp);
+		return;
+	}
 
 	if (pp->eaten)
 		for (ghost = 0; ghost < (pp->nrows * pp->ncols); ghost++)
@@ -610,11 +673,16 @@ init_pacman(ModeInfo * mi)
 void
 draw_pacman(ModeInfo * mi)
 {
-	pacmangamestruct *pp = &pacmangames[MI_SCREEN(mi)];
 	int         g;
+	pacmangamestruct *pp;
+
+	if (pacmangames == NULL)
+		return;
+	pp = &pacmangames[MI_SCREEN(mi)];
+	if (pp->ghosts == NULL)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	do {
 		if (NRAND(3) == 2)
 			pp->pacman.nextbox = NRAND(5);
@@ -731,28 +799,10 @@ void
 release_pacman(ModeInfo * mi)
 {
 	if (pacmangames != NULL) {
-		int         screen, dir, mouth;
+		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			pacmangamestruct *pp = &pacmangames[screen];
-			Display    *display = MI_DISPLAY(mi);
-
-			if (pp->ghosts != NULL)
-				(void) free((void *) pp->ghosts);
-#ifdef DEFUNCT
-			if (pp->eaten != NULL)
-				(void) free((void *) pp->eaten);
-#endif
-			if (pp->stippledGC != NULL) {
-				XFreeGC(display, pp->stippledGC);
-			}
-			if (pp->ghostPixmap != None)
-				XFreePixmap(display, pp->ghostPixmap);
-			if (pp->pacmanPixmap[0][0] != None)
-				for (dir = 0; dir < 4; dir++)
-					for (mouth = 0; mouth < MAXMOUTH; mouth++)
-						XFreePixmap(display, pp->pacmanPixmap[dir][mouth]);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_pacman(MI_DISPLAY(mi), &pacmangames[screen]);
 		(void) free((void *) pacmangames);
 		pacmangames = NULL;
 	}
