@@ -153,6 +153,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_gears
 	"gears",
 #endif
+#ifdef MODE_glplanet
+	"glplanet",
+#endif
 #ifdef MODE_invert
 	"invert",
 #endif
@@ -162,6 +165,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_moebius
 	"moebius",
 #endif
+#ifdef MODE_molecule
+	"molecule",
+#endif
 #ifdef MODE_morph3d
 	"morph3d",
 #endif
@@ -174,6 +180,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_sballs
 	"sballs",
 #endif
+#ifdef MODE_sierpinski3d
+	"sierpinski3d",
+#endif
 #ifdef MODE_skewb
 	"skewb",
 #endif
@@ -283,6 +292,9 @@ static char nice_modes[][MAXMODECHARS] =
 #ifdef MODE_deco
 	"deco",
 #endif
+#ifdef MODE_dragon
+	"dragon",
+#endif
 #ifdef MODE_eyes
 	"eyes",
 #endif
@@ -510,6 +522,9 @@ static char fractal_modes[][MAXMODECHARS] =
 #ifdef MODE_discrete
 	"discrete",
 #endif
+#ifdef MODE_dragon
+	"dragon",
+#endif
 #ifdef MODE_drift
 	"drift",
 #endif
@@ -598,12 +613,12 @@ static char geometry_modes[][MAXMODECHARS] =
 #ifdef MODE_penrose
 	"penrose",
 #endif
-#ifdef MODE_pentominoes
-	"pentominoes",
-#endif
 #ifdef MODE_petal
 	"petal",
 #endif
+#ifdef MODE_polyominoes
+	"polyominoes",
+#endif
 #ifdef MODE_qix
 	"qix",
 #endif
