@@ -1262,10 +1262,12 @@ ModeSpecOpt solitare_opts =
 
 #ifdef USE_MODULES
 ModStruct solitare_description =
-{"solitare", "init_solitare", "draw_solitare", "release_solitare",
- "refresh_solitare", "init_solitare", NULL, &solitare_opts,
- 2000000, 1, 1, 1, 64, 1.0, "",
- "Shows Klondike's game of solitare", 0, NULL};
+{(char *) "solitare", (char *) "init_solitare",
+ (char *) "draw_solitare", (char *) "release_solitare",
+ (char *) "refresh_solitare", (char *) "init_solitare",
+ NULL, &solitare_opts,
+ 2000000, 1, 1, 1, 64, 1.0, (char *) "",
+ (char *) "Shows Klondike's game of solitare", 0, NULL};
 #endif
 
 /*
