@@ -2,7 +2,7 @@
 /* tetris --- autoplaying tetris game */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)tetris.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)tetris.c	5.01 2000/12/19 xlockmore";
 
 #endif
 
@@ -51,10 +51,10 @@ static const char sccsid[] = "@(#)tetris.c	5.00 2000/11/01 xlockmore";
  *     possible)
  *  -Get welltris option to work better.
  *     draw more sophisticated squares (i.e. something like the bitmaps
- *       on tetris
+ *       on tetris)
  *
  * Revision History:
- * 01-Dec-2000: Lifted code from pentominoes variable size pieces
+ * 01-Dec-2000: Lifted code from polyominoes for variable size pieces
  *              Copyright (c) 2000 by Stephen Montgomery-Smith
  * 01-Nov-2000: Allocation checks
  * 02-Aug-2000: alwelltris trackmouse implemented
@@ -87,11 +87,13 @@ static const char sccsid[] = "@(#)tetris.c	5.00 2000/11/01 xlockmore";
 
 #define DEF_BONUS "False"
 #define DEF_CYCLE "True"
+#define DEF_PLAIN "False"
 #define DEF_TRACKMOUSE  "False"
 #define DEF_WELL "False"
 
 static Bool bonus;
 static Bool cycle_p;
+static Bool plain;
 static Bool trackmouse;
 static Bool well;
 
@@ -103,6 +105,8 @@ static XrmOptionDescRec opts[] =
   {(char *) "+bonus", (char *) ".tetris.bonus", XrmoptionNoArg, (caddr_t) "off"},
   {(char *) "-cycle", (char *) ".tetris.cycle", XrmoptionNoArg, (caddr_t) "on"},
   {(char *) "+cycle", (char *) ".tetris.cycle", XrmoptionNoArg, (caddr_t) "off"},
+  {(char *) "-plain", (char *) ".tetris.plain", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+plain", (char *) ".tetris.plain", XrmoptionNoArg, (caddr_t) "off"},
 #ifndef DISABLE_INTERACTIVE
   {(char *) "-trackmouse", (char *) ".tetris.trackmouse", XrmoptionNoArg, (caddr_t) "on"},
   {(char *) "+trackmouse", (char *) ".tetris.trackmouse", XrmoptionNoArg, (caddr_t) "off"},
@@ -115,6 +119,7 @@ static argtype vars[] =
 {
   {(caddr_t *) & bonus, (char *) "bonus", (char *) "Bonus", (char *) DEF_BONUS, t_Bool},
   {(caddr_t *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool},
+  {(caddr_t *) & plain, (char *) "plain", (char *) "Plain", (char *) DEF_PLAIN, t_Bool},
 #ifndef DISABLE_INTERACTIVE
   {(caddr_t *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool},
 #endif
@@ -124,6 +129,7 @@ static OptionStruct desc[] =
 {
   {(char *) "-/+bonus", (char *) "turn on/off larger bonus pieces"},
   {(char *) "-/+cycle", (char *) "turn on/off colour cycling"},
+  {(char *) "-/+plain", (char *) "turn on/off plain pieces"},
 #ifndef DISABLE_INTERACTIVE
   {(char *) "-/+trackmouse", (char *) "turn on/off the tracking of the mouse"},
 #endif
@@ -143,12 +149,12 @@ ModStruct   tetris_description =
 #endif
 
 #define MINGRIDSIZE 10 
-#define MAXPIXELSIZE 13 
+#define MAXPIXELSIZE 12
 #define MINSIZE 6 
 #define MAX_SIDES 4
 
 #define NUM_FLASHES 16
-#define BITMAPS 16
+#define BITMAPS 256
 
 #define DELTA 1
 #define WELL_WIDTH 8
@@ -157,13 +163,10 @@ ModStruct   tetris_description =
 #define BASE_WIDTH (WELL_WIDTH*tp->xs+DELTA)
 #define ENDPT (3*BASE_WIDTH)
 
-
 #define THRESHOLD(x) ((x+1)*10)
-#define CHECKUP(x) ((x)%8)
-#define CHECKDOWN(x) (((x)/4)*4+(x)%2)
 #define EMPTY (-1)
 
-typedef enum {FALL, DROP, LEFT, RIGHT, ROTATE, REFLECT} move_t;
+typedef enum {FALL, DROP, MOVE_LEFT, MOVE_RIGHT, ROTATE, REFLECT} move_t;
 
 typedef struct {
   int  squares, polyomino_number;
@@ -203,7 +206,7 @@ typedef struct {
   int   object, orient;
   GC   gc;
   Colormap cmap;
-  Bool   cycle_p, mono_p, no_colors;
+  Bool   cycle_p, mono_p, no_colors, use3D;
   unsigned long blackpixel, whitepixel, fg, bg;
   int   direction;
   unsigned long colour;
@@ -234,19 +237,105 @@ typedef struct {
   ModeInfo *mi;
 } trisstruct;
 
+#define ARR(i,j) (((i)<0||(i)>=tp->curPolyomino.size||\
+(j)<0||(j)>=tp->curPolyomino.size)?-2:\
+(polytris[(int) tp->bonusNow].polyomino[tp->curPolyomino.polyomino_number].shape[(j) * tp->curPolyomino.size + (i)]>0))
+
 #define ROUND8(n) ((((n)+7)/8)*8)
+
+/* Defines to index the bitmaps.  A set bit indicates that an edge or
+   corner is required. */
+#define LEFT 1 /* (1<<0) */
+#define DOWN 2 /* (1<<1) */
+#define RIGHT 4 /* (1<<2) */
+#define UP 8 /* (1<<3) */
+#define LEFT_DOWN 16 /* (1<<4) */
+#define RIGHT_DOWN 32 /* (1<<5) */
+#define RIGHT_UP 64 /* (1<<6) */
+#define LEFT_UP 128 /* (1<<7) */
+#define IS_LEFT(n) ((n)&LEFT)
+#define IS_DOWN(n) ((n)&DOWN)
+#define IS_RIGHT(n) ((n)&RIGHT)
+#define IS_UP(n) ((n)&UP)
+#define IS_LEFT_DOWN(n) ((n)&LEFT_DOWN)
+#define IS_RIGHT_DOWN(n) ((n)&RIGHT_DOWN)
+#define IS_RIGHT_UP(n) ((n)&RIGHT_UP)
+#define IS_LEFT_UP(n) ((n)&LEFT_UP)
+
+#define CHECKDOWN(x) ((x)|DOWN)
+#define CHECKUP(x) ((x)|UP)
+
+/* Defines to access the bitmaps. */
 #define BITNO(x,y) ((x)+(y)*ROUND8(tp->ys))
 #define SETBIT(x,y) {data[BITNO(x,y)/8] |= 1<<(BITNO(x,y)%8);}
 #define RESBIT(x,y) {data[BITNO(x,y)/8] &= ~(1<<(BITNO(x,y)%8));}
-#define HALFBIT(x,y) {if ((x%2)^(y%2)) RESBIT(x,y) else SETBIT(x,y)}
-#define YESSET(b,x,y) {if (b) SETBIT(x,y) else HALFBIT(x,y)}
-#define LEFT(n) ((n)&1)
-#define RIGHT(n) ((n)&2)
-#define UP(n) ((n)&4)
-#define DOWN(n) ((n)&8)
-#define ARR(x,y) (((x)<0||(x)>=tp->width||(y)<0||(y)>=tp->height)?-1:tp->array[x][y])
- /* Map from Q. Alex Zhao notation to Stephen Montgomery-Smith notation */
-#define Z2MS(z) ((!((z)&1))+((!((z)&2))<<3)+((!((z)&4))<<1)+((!((z)&8))<<2))
+#define TWOTHIRDSBIT(x,y) {if ((x+y-1)%3) SETBIT(x,y) else RESBIT(x,y)}
+#define HALFBIT(x,y) {if ((x-y)%2) SETBIT(x,y) else RESBIT(x,y)}
+#define THIRDBIT(x,y) {if (!((x-y-1)%3)) SETBIT(x,y) else RESBIT(x,y)}
+#define THREEQUARTERSBIT(x,y) \
+  {if ((y%2)||((x+2+y/2+1)%2)) SETBIT(x,y) else RESBIT(x,y)}
+#define NOTHALFBIT(x,y) {if ((x-y)%2) RESBIT(x,y) else SETBIT(x,y)}
+
+/* Parameters for bitmaps. */
+#define G ((tp->ys/45)+1)         /* 1/2 of gap between pentominoes. */
+#define T ((tp->ys<=12)?1:(G*2))  /* Thickness of walls of pentominoes. */
+#define R ((tp->ys<=12)?1:(G*6))  /* Amount of rounding. */
+#define RT ((tp->ys<=12)?1:(G*3)) /* Thickness of wall of rounded parts. 
+                                  Here 3 is an approximation to 2 sqrt(2).  */
+#define RR 0   /* Roof ridge thickness */
+
+/* A list of those bitmaps we need to create to display any polyomino. */
+static int images_needed[] =
+{
+/* This needed for mononimo*/
+ LEFT|RIGHT|UP|DOWN|LEFT_UP|LEFT_DOWN|RIGHT_UP|RIGHT_DOWN,
+
+ LEFT_UP|LEFT_DOWN|RIGHT_UP|RIGHT_DOWN,
+
+ LEFT|RIGHT_UP|RIGHT_DOWN,
+ RIGHT|LEFT_UP|LEFT_DOWN,
+ UP|LEFT_DOWN|RIGHT_DOWN,
+ DOWN|LEFT_UP|RIGHT_UP,
+ LEFT|RIGHT_UP,
+ RIGHT|LEFT_UP,
+ UP|LEFT_DOWN,
+ DOWN|LEFT_UP,
+ LEFT|RIGHT_DOWN,
+ RIGHT|LEFT_DOWN,
+ UP|RIGHT_DOWN,
+ DOWN|RIGHT_UP,
+
+#if 0
+/* These needed for hexonimoes*/
+ LEFT,
+ RIGHT,
+ UP,
+ DOWN,
+ LEFT_DOWN|RIGHT_UP|RIGHT_DOWN,
+ LEFT_UP|RIGHT_UP|RIGHT_DOWN,
+ LEFT_UP|LEFT_DOWN|RIGHT_DOWN,
+ LEFT_UP|LEFT_DOWN|RIGHT_UP,
+#endif
+
+ LEFT|UP|RIGHT_DOWN,
+ LEFT|DOWN|RIGHT_UP,
+ RIGHT|UP|LEFT_DOWN,
+ RIGHT|DOWN|LEFT_UP,
+ LEFT|UP,
+ LEFT|DOWN,
+ RIGHT|UP,
+ RIGHT|DOWN,
+
+ LEFT|RIGHT,
+ UP|DOWN,
+
+ RIGHT|UP|DOWN,
+ LEFT|UP|DOWN,
+ LEFT|RIGHT|DOWN,
+ LEFT|RIGHT|UP,
+
+ -1
+};
 
 static trisstruct *triss = NULL;
 
@@ -368,6 +457,16 @@ ominos[] =
   1, 0, 1,  0,   0, 0, 2, 0, 0,  0, 0, 10, 0, 0,  0, 0, 10, 0, 0,  0, 0, 10, 0, 0,  0, 0, 8, 0, 0,
 };
 
+static int
+image_needed(int n) {
+  int i;
+
+  for (i=0;images_needed[i]!=-1;i++)
+    if (n == images_needed[i])
+      return 1;
+  return 0;
+}
+
 static void 
 leadingEmptyWidth(Polyominoes * polyo)
 {
@@ -540,10 +639,12 @@ drawSquare(ModeInfo * mi, int pmid, int cid, int col, int row)
     XFillRectangle(display, window, tp->gc,
       tp->xb + col * tp->xs, tp->yb + row * tp->ys, tp->xs, tp->ys);
   else {
+    if (pmid < 0 || pmid >= 256)
+      (void) printf("pmid = %d\n", pmid);
 /*-
  * PURIFY on SunOS4 and on Solaris 2 reports a 120 byte memory leak on
  * the next line */
-    (void) XPutImage(display, window, tp->gc, tp->images[Z2MS(pmid)], 0, 0,
+    (void) XPutImage(display, window, tp->gc, tp->images[pmid], 0, 0,
       tp->xb + tp->xs * col, tp->yb + tp->ys * row,
       tp->xs, tp->ys);
   }
@@ -557,6 +658,8 @@ xorSquare(ModeInfo * mi, int i, int j)
   trisstruct *tp = &triss[MI_SCREEN(mi)];
 
   XSetFunction(display, tp->gc, GXxor);
+  XSetBackground(display, tp->gc, 0);
+  XSetForeground(display, tp->gc, 0xFFFFFF);
   if (tp->well)
     XFillRectangle(display, window, tp->gc,
       tp->xb + i * tp->xs + 1, tp->yb + j * tp->ys + 1,
@@ -566,6 +669,8 @@ xorSquare(ModeInfo * mi, int i, int j)
       tp->xb, tp->yb + j * tp->ys,
       tp->xs * tp->ncols, tp->ys);
   XSetFunction(display, tp->gc, GXcopy);
+  XSetBackground(display, tp->gc, tp->blackpixel);
+  XSetForeground(display, tp->gc, tp->whitepixel);
 }
 
 static void
@@ -656,28 +761,19 @@ drawTrapazoid(ModeInfo * mi, /* int pmid, */ int cid, int i, int j)
   drawWallSquare(mi, tp->gc, i, j); /* pmid not used yet */
 }
 
-#if 0
-static void
-xorTrapazoid(ModeInfo * mi, /* int pmid, */ int cid, int i, int j)
-{
-  trisstruct *tp = &triss[MI_SCREEN(mi)];
-
-  setColour(mi, cid /* , 1 */ );
-  XSetFunction(MI_DISPLAY(mi), tp->gc, GXxor);
-  drawWallSquare(mi, tp->gc, i, j); /* pmid not used yet */
-  XSetFunction(MI_DISPLAY(mi), tp->gc, GXcopy);
-}
-#endif
-
 static void
 freezeTrapazoid(ModeInfo * mi, int i, int j)
 {
+  Display    *display = MI_DISPLAY(mi);
   trisstruct *tp = &triss[MI_SCREEN(mi)];
 
-  setColour(mi, -1 /* , 1 */);
-  XSetFunction(MI_DISPLAY(mi), tp->gc, GXxor);
+  XSetFunction(display, tp->gc, GXxor);
+  XSetBackground(display, tp->gc, 0);
+  XSetForeground(display, tp->gc, 0xFFFFFF);
   drawWallSquare(mi, tp->gc, i, j); /* pmid not used yet */
-  XSetFunction(MI_DISPLAY(mi), tp->gc, GXcopy);
+  XSetFunction(display, tp->gc, GXcopy);
+  XSetBackground(display, tp->gc, tp->blackpixel);
+  XSetForeground(display, tp->gc, tp->whitepixel);
 }
 
 /* Wall number ... imagine a 4 hour clock...
@@ -820,8 +916,32 @@ checkFreeze(ModeInfo *mi)
   tp->curPolyomino.xpos += w * WELL_WIDTH;
 }
 
+static int
+curPixmap(trisstruct *tp, int i, int j)
+{
+  int pmid = 0;
+
+  if (ARR(i,j) != ARR(i-1,j))
+    pmid |= LEFT;
+  if (ARR(i,j) != ARR(i,j+1))
+    pmid |= DOWN;
+  if (ARR(i,j) != ARR(i+1,j))
+    pmid |= RIGHT;
+  if (ARR(i,j) != ARR(i,j-1))
+    pmid |= UP;
+  if (ARR(i,j) != ARR(i-1,j+1))
+    pmid |= LEFT_DOWN;
+  if (ARR(i,j) != ARR(i+1,j+1))
+    pmid |= RIGHT_DOWN;
+  if (ARR(i,j) != ARR(i+1,j-1))
+    pmid |= RIGHT_UP;
+  if (ARR(i,j) != ARR(i-1,j-1))
+    pmid |= LEFT_UP;
+  return pmid;
+}
+
 static void
-drawBox(ModeInfo * mi, int pmid, int cid, int i, int j)
+drawBox(ModeInfo * mi, int cid, int i, int j)
 {
   if (j < WELL_DEPTH) {
     drawTrapazoid(mi, /* pmid, */ cid, i, j);
@@ -829,7 +949,7 @@ drawBox(ModeInfo * mi, int pmid, int cid, int i, int j)
     int base_i, base_j;
 
     wall_to_base(&base_i, &base_j, i, j);
-    drawSquare(mi, pmid, cid, base_i, base_j);
+    drawSquare(mi, 15, cid, base_i, base_j);
   }
 }
 
@@ -877,12 +997,9 @@ drawPolyomino(ModeInfo * mi)
 
           xi = (tp->curPolyomino.xpos + i) % WELL_PERIMETER;
           yj = tp->curPolyomino.ypos + j;
-          drawBox(mi, polytris[(int) tp->bonusNow].polyomino
-            [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i],
-            tp->curPolyomino.color_number, xi, yj);
+          drawBox(mi, tp->curPolyomino.color_number, xi, yj);
         } else {
-          drawSquare(mi, polytris[(int) tp->bonusNow].polyomino
-            [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i],
+          drawSquare(mi, curPixmap(tp, i, j),
             tp->curPolyomino.color_number,
             tp->curPolyomino.xpos + i, tp->curPolyomino.ypos + j);
         }
@@ -897,26 +1014,7 @@ drawPolyominoDiff(ModeInfo *mi, thing_t * old)
   trisstruct *tp = &triss[MI_SCREEN(mi)];
   int   i, j, ox, oy;
 
-  for (i = 0; i < tp->curPolyomino.size; i++)
-    for (j = 0; j < tp->curPolyomino.size; j++)
-      if ((tp->curPolyomino.ypos + j >= 0) &&
-        polytris[(int) tp->bonusNow].polyomino
-          [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i]) {
-        if (tp->well) {
-          int xi, yj;
-
-          xi = (tp->curPolyomino.xpos + i) % WELL_PERIMETER;
-          yj = tp->curPolyomino.ypos + j;
-          drawBox(mi, polytris[(int) tp->bonusNow].polyomino
-            [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i],
-            tp->curPolyomino.color_number, xi, yj);
-        } else {
-          drawSquare(mi, polytris[(int) tp->bonusNow].polyomino
-            [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i],
-            tp->curPolyomino.color_number, tp->curPolyomino.xpos + i, tp->curPolyomino.ypos + j);
-        }
-      }
-
+  /* Erase first or will have invisible squares when folded down a corner */
   for (i = 0; i < tp->curPolyomino.size; i++)
     for (j = 0; j < tp->curPolyomino.size; j++) {
       ox = old->xpos + i - tp->curPolyomino.xpos;
@@ -937,6 +1035,24 @@ drawPolyominoDiff(ModeInfo *mi, thing_t * old)
         }
       }
     }
+
+  for (i = 0; i < tp->curPolyomino.size; i++)
+    for (j = 0; j < tp->curPolyomino.size; j++)
+      if ((tp->curPolyomino.ypos + j >= 0) &&
+        polytris[(int) tp->bonusNow].polyomino
+          [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i]) {
+        if (tp->well) {
+          int xi, yj;
+
+          xi = (tp->curPolyomino.xpos + i) % WELL_PERIMETER;
+          yj = tp->curPolyomino.ypos + j;
+          drawBox(mi, tp->curPolyomino.color_number, xi, yj);
+        } else {
+          drawSquare(mi, curPixmap(tp, i, j),
+            tp->curPolyomino.color_number,
+            tp->curPolyomino.xpos + i, tp->curPolyomino.ypos + j);
+        }
+      }
 }
 
 static void
@@ -1010,8 +1126,7 @@ putBox(ModeInfo * mi)
           }
         } else {
           pos = (y + j) * tp->ncols + x + i;
-          tp->field[pos].pmid = polytris[(int) tp->bonusNow].polyomino
-            [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i] % BITMAPS;
+          tp->field[pos].pmid = curPixmap(tp, i, j);
           tp->field[pos].cid = tp->curPolyomino.color_number;
         }
       }
@@ -1203,7 +1318,7 @@ tryMove(ModeInfo * mi, move_t move)
       tp->curPolyomino.ypos--;
       break;
 
-    case LEFT:
+    case MOVE_LEFT:
       if (tp->well) {
         tp->curPolyomino.xpos = (tp->curPolyomino.xpos + WELL_PERIMETER - 1) % WELL_PERIMETER;
         if (atBaseFully(mi) || (atBasePartially(mi) && wallChange(mi, old, tp->curPolyomino)))
@@ -1212,7 +1327,7 @@ tryMove(ModeInfo * mi, move_t move)
         tp->curPolyomino.xpos--;
       break;
 
-    case RIGHT:
+    case MOVE_RIGHT:
       if (tp->well) {
         tp->curPolyomino.xpos = (tp->curPolyomino.xpos + WELL_PERIMETER + 1) % WELL_PERIMETER;
         if (atBaseFully(mi) || (atBasePartially(mi) && wallChange(mi, old, tp->curPolyomino)))
@@ -1264,7 +1379,7 @@ fillLines(ModeInfo *mi)
   if (tp->well) {
     for (j = 0; j < WELL_DEPTH + WELL_WIDTH; j++)
       for (i = 0; i < WELL_PERIMETER; i++) {
-         drawBox(mi, 0,
+         drawBox(mi, /* 0, */
            LRAND() % polytris[(int) tp->bonusNow].mode.number,
                  i, j);
          XFlush(display);
@@ -1272,10 +1387,10 @@ fillLines(ModeInfo *mi)
   } else {
     for (j = 0; j <= tp->nrows / 2; j++) {
       for (i = 0; i < tp->ncols; i++) {
-         drawSquare(mi, 0,
+         drawSquare(mi, BITMAPS - 1,
            LRAND() % polytris[(int) tp->bonusNow].mode.number,
                  i, j);
-         drawSquare(mi, 0,
+         drawSquare(mi, BITMAPS - 1,
            LRAND() % polytris[(int) tp->bonusNow].mode.number,
                  i, tp->nrows - j - 1);
       }
@@ -1287,12 +1402,27 @@ fillLines(ModeInfo *mi)
 static void
 free_images(trisstruct *tp)
 {
-  int i;
+  int n;
 
-  for (i = 0; i < BITMAPS; i++) {
-    if (tp->images[i] != None) {
-      (void) XDestroyImage(tp->images[i]);
-      tp->images[i] = None;
+  if (tp->images[BITMAPS - 1] != None) {
+    (void) XDestroyImage(tp->images[BITMAPS - 1]);
+    tp->images[BITMAPS - 1] = None;
+  }
+  for (n = 0; n < BITMAPS - 1; n++) {
+    /* Don't bother to free duplicates */
+    if (IS_UP(n) && IS_DOWN(n) && IS_LEFT(n) && IS_RIGHT(n))
+      tp->images[n] = None;
+    else if (IS_LEFT_UP(n) && (IS_LEFT(n) || IS_UP(n)))
+      tp->images[n] = None;
+    else if (IS_LEFT_DOWN(n) && (IS_LEFT(n) || IS_DOWN(n)))
+      tp->images[n] = None;
+    else if (IS_RIGHT_UP(n) && (IS_RIGHT(n) || IS_UP(n)))
+      tp->images[n] = None;
+    else if (IS_RIGHT_DOWN(n) && (IS_RIGHT(n) || IS_DOWN(n)))
+      tp->images[n] = None;
+    else if (tp->images[n] != None) {
+      (void) XDestroyImage(tp->images[n]);
+      tp->images[n] = None;
     }
   }
 }
@@ -1363,12 +1493,12 @@ checkLines(ModeInfo *mi)
 
     if (nset[0] || nset[1]) {
         fieldstruct temp_base[WELL_WIDTH][WELL_WIDTH];
-        for (k = 0; k < ((NUM_FLASHES / (nset[0] + nset[1])) % 2) * 2; k++) {
+        for (k = 0; k < ((NUM_FLASHES / (nset[0] + nset[1])) / 2) * 2; k++) {
             for (j = 0; j < WELL_WIDTH; j++) {
                 if (lSet[0][j] == WELL_WIDTH)
                   for (i = 0; i < WELL_WIDTH; i++)
                     xorSquare(mi, i, j);
-                if (lSet[1][i] == WELL_WIDTH)
+                if (lSet[1][j] == WELL_WIDTH)
                   for (i = 0; i < WELL_WIDTH; i++)
                     xorSquare(mi, j, i);
             }
@@ -1378,52 +1508,52 @@ checkLines(ModeInfo *mi)
         for (j = 0; j < WELL_WIDTH; j++)
           for (i = 0; i < WELL_WIDTH; i++)
             temp_base[j][i] = tp->base[j][i];
-        if (nset[0] > 0)
-          for (j = 0; j < WELL_WIDTH; j++)
-            if (lSet[0][j] == WELL_WIDTH) {
-              if (j < WELL_WIDTH / 2)
-                for (i = 0; i < WELL_WIDTH; i++) {
-                  for (k = j; k > 0; k--)
-                    temp_base[k][i] = temp_base[k - 1][i];
-                  temp_base[0][i].pmid = EMPTY;
-                }
-              else
-                for (i = 0; i < WELL_WIDTH; i++) {
-                  for (k = j; k < WELL_WIDTH - 1; k++)
-                    temp_base[k][i] = temp_base[k + 1][i];
-                  temp_base[WELL_WIDTH - 1][i].pmid = EMPTY;
-                }
-            }
-        if (nset[1] > 0)
+        if (nset[0] > 0) {
+          for (j = 0; j < WELL_WIDTH / 2; j++)
+            if (lSet[0][j] == WELL_WIDTH)
+              for (i = 0; i < WELL_WIDTH; i++) {
+                for (k = j; k > 0; k--)
+                  temp_base[k][i] = temp_base[k - 1][i];
+                temp_base[0][i].pmid = EMPTY;
+              }
+          for (j = WELL_WIDTH - 1; j >= WELL_WIDTH / 2; j--)
+            if (lSet[0][j] == WELL_WIDTH)
+              for (i = 0; i < WELL_WIDTH; i++) {
+                for (k = j; k < WELL_WIDTH - 1; k++)
+                  temp_base[k][i] = temp_base[k + 1][i];
+                temp_base[WELL_WIDTH - 1][i].pmid = EMPTY;
+              }
+        }
+        if (nset[1] > 0) {
+          for (i = 0; i < WELL_WIDTH / 2; i++)
+            if (lSet[1][i] == WELL_WIDTH)
+              for (j = 0; j < WELL_WIDTH; j++) {
+                for (k = i; k > 0; k--)
+                  temp_base[j][k] = temp_base[j][k - 1];
+                temp_base[j][0].pmid = EMPTY;
+              }
+          for (i = WELL_WIDTH - 1; i >= WELL_WIDTH / 2; i--)
+            if (lSet[1][i] == WELL_WIDTH)
+              for (j = 0; j < WELL_WIDTH; j++) {
+                for (k = i; k < WELL_WIDTH - 1; k++)
+                  temp_base[j][k] = temp_base[j][k + 1];
+                temp_base[j][WELL_WIDTH - 1].pmid = EMPTY;
+              }
+        }
+        for (j = 0; j < WELL_WIDTH; j++)
           for (i = 0; i < WELL_WIDTH; i++)
-            if (lSet[1][i] == WELL_WIDTH) {
-              if (i < WELL_WIDTH / 2)
-                for (j = 0; j < WELL_WIDTH; j++) {
-                  for (k = i; k > 0; k--)
-                    temp_base[j][k] = temp_base[j][k - 1];
-                  temp_base[j][0].pmid = EMPTY;
-                }
+            if ((temp_base[j][i].cid != tp->base[j][i].cid) ||
+                (temp_base[j][i].pmid != tp->base[j][i].pmid))
+            {
+              tp->base[j][i] = temp_base[j][i];
+              if (tp->base[j][i].pmid >= 0)
+                drawSquare(mi, tp->base[j][i].pmid, tp->base[j][i].cid, i, j);
               else
-                for (j = 0; j < WELL_WIDTH; j++) {
-                  for (k = i; k < WELL_WIDTH - 1; k++)
-                    temp_base[j][k] = temp_base[j][k + 1];
-                  temp_base[j][WELL_WIDTH - 1].pmid = EMPTY;
-                }
+                clearSquare(mi, i, j);
+              XFlush(display);
             }
-       for (j = 0; j < WELL_WIDTH; j++)
-         for (i = 0; i < WELL_WIDTH; i++)
-           if ((temp_base[j][i].cid != tp->base[j][i].cid) ||
-               (temp_base[j][i].pmid != tp->base[j][i].pmid))
-           {
-             tp->base[j][i] = temp_base[j][i];
-             if (tp->base[j][i].pmid >= 0)
-               drawSquare(mi, tp->base[j][i].pmid, tp->base[j][i].cid, i, j);
-             else
-               clearSquare(mi, i, j);
-             XFlush(display);
-           }
 
-      XFlush(display);
+       XFlush(display);
     }
 
     return (nset[0] + nset[1]);
@@ -1517,98 +1647,198 @@ gameOver(ModeInfo *mi)
 }
 
 static Bool
-create_images(ModeInfo *mi, trisstruct *tp)
+create_an_image(ModeInfo *mi, trisstruct *tp, int n)
 {
-  int x, y, n;
+  int x, y;
   unsigned char * data;
 
-  for (n = 0; n < BITMAPS; n++) {
-    if ((data = (unsigned char *) malloc(sizeof(unsigned char)*
-           (tp->ys*ROUND8(tp->xs)/8))) == NULL) {
-        return False;  /* no biggie, will not use images */
-    }
+  if ((data = (unsigned char *) malloc(sizeof(char)*(tp->ys*ROUND8(tp->ys)/8))) == NULL) {
+    return False;  /* no biggie, will not use images */
+  }
 
-    for (y=0;y<tp->ys;y++) for (x=0;x<tp->xs;x++) {
-      if ((x>=y && x<=tp->ys-y-1 && x<=tp->ys/2)
-          || (x<=y && x<=tp->ys-y-1 && y<=tp->ys/2)
-          || (UP(n)     && x>=y && x<=tp->ys-y-1 && x> tp->ys/2)
-          || (!DOWN(n)  && x<=y && x>=tp->ys-y-1 && x<=tp->ys/2)
-          || (LEFT(n)   && x<=y && x<=tp->ys-y-1 && y> tp->ys/2)
-          || (!RIGHT(n) && x>=y && x>=tp->ys-y-1 && y<=tp->ys/2))
+  for (y=0;y<tp->ys;y++) for (x=0;x<tp->ys;x++) {
+     if (!tp->use3D) {
+#ifdef SMALL_BELLYBUTTON 
+        if (x >= tp->ys / 2 && x <= tp->ys / 2 + 1 &&
+            y >= tp->ys / 2 && y <= tp->ys / 2 + 1)
+          NOTHALFBIT(x,y)
+        else
+#endif
+          HALFBIT(x,y)
+      } else if ((x>=y && x<=tp->ys-y-1 && IS_UP(n))
+          || (x<=y && x<=tp->ys-y-1 && y<tp->ys/2 && !IS_LEFT(n))
+          || (x>=y && x>=tp->ys-y-1 && y<tp->ys/2 && !IS_RIGHT(n)))
         SETBIT(x,y)
-      else
+      else if ((x<=y && x<=tp->ys-y-1 && IS_LEFT(n))
+          || (x>=y && x<=tp->ys-y-1 && x<tp->ys/2 && !IS_UP(n))
+          || (x<=y && x>=tp->ys-y-1 && x<tp->ys/2 && !IS_DOWN(n)))
+        TWOTHIRDSBIT(x,y)
+      else if ((x>=y && x>=tp->ys-y-1 && IS_RIGHT(n))
+          || (x>=y && x<=tp->ys-y-1 && x>=tp->ys/2 && !IS_UP(n))
+          || (x<=y && x>=tp->ys-y-1 && x>=tp->ys/2 && !IS_DOWN(n)))
         HALFBIT(x,y)
-    }
+      else if ((x<=y && x>=tp->ys-y-1 && IS_DOWN(n))
+          || (x<=y && x<=tp->ys-y-1 && y>=tp->ys/2 && !IS_LEFT(n))
+          || (x>=y && x>=tp->ys-y-1 && y>=tp->ys/2 && !IS_RIGHT(n)))
+        THIRDBIT(x,y)
+  }
 
-#define T 2
-#define G 1
-    if (LEFT(n))
-      for (y=0;y<tp->ys;y++) for (x=G;x<G+T;x++)
-        SETBIT(x,y)
-    if (RIGHT(n))
-      for (y=0;y<tp->ys;y++) for (x=G;x<G+T;x++)
-        SETBIT(tp->ys-1-x,y)
-    if (UP(n))
-      for (x=0;x<tp->ys;x++) for (y=G;y<G+T;y++)
-        SETBIT(x,y)
-    if (DOWN(n))
-      for (x=0;x<tp->ys;x++) for (y=G;y<G+T;y++)
-        SETBIT(x,tp->ys-1-y)
-    if (LEFT(n))
-      for (y=0;y<tp->ys;y++) for (x=0;x<G;x++)
-        RESBIT(x,y)
-    if (RIGHT(n))
-      for (y=0;y<tp->ys;y++) for (x=0;x<G;x++)
-        RESBIT(tp->ys-1-x,y)
-    if (UP(n))
-      for (x=0;x<tp->ys;x++) for (y=0;y<G;y++)
-        RESBIT(x,y)
-    if (DOWN(n))
-      for (x=0;x<tp->ys;x++) for (y=0;y<G;y++)
-        RESBIT(x,tp->ys-1-y)
+  if (IS_LEFT(n))
+    for (y=0;y<tp->ys;y++) for (x=G;x<G+T;x++)
+      SETBIT(x,y)
+  if (IS_RIGHT(n))
+    for (y=0;y<tp->ys;y++) for (x=G;x<G+T;x++)
+      SETBIT(tp->ys-1-x,y)
+  if (IS_UP(n))
+    for (x=0;x<tp->ys;x++) for (y=G;y<G+T;y++)
+      SETBIT(x,y)
+  if (IS_DOWN(n))
+    for (x=0;x<tp->ys;x++) for (y=G;y<G+T;y++)
+      SETBIT(x,tp->ys-1-y)
+  if (IS_LEFT(n))
+    for (y=0;y<tp->ys;y++) for (x=0;x<G;x++)
+      RESBIT(x,y)
+  if (IS_RIGHT(n))
+    for (y=0;y<tp->ys;y++) for (x=0;x<G;x++)
+      RESBIT(tp->ys-1-x,y)
+  if (IS_UP(n))
+    for (x=0;x<tp->ys;x++) for (y=0;y<G;y++)
+      RESBIT(x,y)
+  if (IS_DOWN(n))
+    for (x=0;x<tp->ys;x++) for (y=0;y<G;y++)
+      RESBIT(x,tp->ys-1-y)
 
-#define R 6
-#define RT 3
-    if (LEFT(n) && UP(n))
-      for (x=G;x<=G+R;x++)
-        for (y=G;y<=R+2*G-x;y++) {
-          if (x+y>R+2*G-RT)
-            SETBIT(x,y)
-          else
-            RESBIT(x,y)
-		}
-    if (LEFT(n) && DOWN(n))
-      for (x=G;x<=G+R;x++)
-        for (y=G;y<=R+2*G-x;y++) {
-          if (x+y>R+2*G-RT)
-            SETBIT(x,tp->ys-1-y)
-          else
-            RESBIT(x,tp->ys-1-y)
-        }
-    if (RIGHT(n) && UP(n))
-      for (x=G;x<=G+R;x++)
-        for (y=G;y<=R+2*G-x;y++) {
-          if (x+y>R+2*G-RT)
-            SETBIT(tp->ys-1-x,y)
-          else
-            RESBIT(tp->ys-1-x,y)
-        }
-    if (RIGHT(n) && DOWN(n))
-      for (x=G;x<=G+R;x++)
-        for (y=G;y<=R+2*G-x;y++) {
-          if (x+y>R+2*G-RT)
-            SETBIT(tp->ys-1-x,tp->ys-1-y)
-          else
-            RESBIT(tp->ys-1-x,tp->ys-1-y)
-        }
-    if ((tp->images[n] = XCreateImage(MI_DISPLAY(mi), MI_VISUAL(mi),
-             1, XYBitmap, 0, (char *) data, tp->xs, tp->ys, 8, 0)) == None) {
-       (void) free((void *) data);
-       return False;
+  if (IS_LEFT(n) && IS_UP(n))
+    for (x=G;x<=G+R;x++)
+      for (y=G;y<=R+2*G-x;y++) {
+        if (x+y>R+2*G-RT)
+          SETBIT(x,y)
+        else
+          RESBIT(x,y)
+      }
+  if (IS_LEFT(n) && IS_DOWN(n))
+    for (x=G;x<=G+R;x++)
+      for (y=G;y<=R+2*G-x;y++) {
+        if (x+y>R+2*G-RT)
+          SETBIT(x,tp->ys-1-y)
+        else
+          RESBIT(x,tp->ys-1-y)
+      }
+  if (IS_RIGHT(n) && IS_UP(n))
+    for (x=G;x<=G+R;x++)
+      for (y=G;y<=R+2*G-x;y++) {
+        if (x+y>R+2*G-RT)
+          SETBIT(tp->ys-1-x,y)
+        else
+          RESBIT(tp->ys-1-x,y)
+      }
+  if (IS_RIGHT(n) && IS_DOWN(n))
+    for (x=G;x<=G+R;x++)
+      for (y=G;y<=R+2*G-x;y++) {
+        if (x+y>R+2*G-RT)
+          SETBIT(tp->ys-1-x,tp->ys-1-y)
+        else
+          RESBIT(tp->ys-1-x,tp->ys-1-y)
+      }
+
+  if (!IS_LEFT(n) && !IS_UP(n) && IS_LEFT_UP(n)) {
+    for (x=0;x<G;x++) for(y=0;y<G;y++)
+      RESBIT(x,y)
+    for (x=G;x<G+T;x++) for(y=0;y<G;y++)
+      SETBIT(x,y)
+    for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
+      SETBIT(x,y)
+  }
+  if (!IS_LEFT(n) && !IS_DOWN(n) && IS_LEFT_DOWN(n)) {
+    for (x=0;x<G;x++) for(y=0;y<G;y++)
+      RESBIT(x,tp->ys-1-y)
+    for (x=G;x<G+T;x++) for(y=0;y<G;y++)
+      SETBIT(x,tp->ys-1-y)
+    for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
+      SETBIT(x,tp->ys-1-y)
+  }
+  if (!IS_RIGHT(n) && !IS_UP(n) && IS_RIGHT_UP(n)) {
+    for (x=0;x<G;x++) for(y=0;y<G;y++)
+      RESBIT(tp->ys-1-x,y)
+    for (x=G;x<G+T;x++) for(y=0;y<G;y++)
+      SETBIT(tp->ys-1-x,y)
+    for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
+      SETBIT(tp->ys-1-x,y)
+  }
+  if (!IS_RIGHT(n) && !IS_DOWN(n) && IS_RIGHT_DOWN(n)) {
+    for (x=0;x<G;x++) for(y=0;y<G;y++)
+      RESBIT(tp->ys-1-x,tp->ys-1-y)
+    for (x=G;x<G+T;x++) for(y=0;y<G;y++)
+      SETBIT(tp->ys-1-x,tp->ys-1-y)
+    for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
+      SETBIT(tp->ys-1-x,tp->ys-1-y)
+  }
+#ifdef LARGE_BELLYBUTTON 
+  if (!tp->use3D) {
+    if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
+      for (x=0;x<G+T;x++) for(y=0;y<G+T;y++)
+        SETBIT(x,y)
+    if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
+      for (x=0;x<G+T;x++) for(y=tp->ys-G-T;y<tp->ys;y++)
+        SETBIT(x,y)
+    if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
+      for (x=tp->ys-G-T;x<tp->ys;x++) for(y=0;y<G+T;y++)
+        SETBIT(x,y)
+    if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
+      for (x=tp->ys-G-T;x<tp->ys;x++) for(y=tp->ys-G-T;y<tp->ys;y++)
+        SETBIT(x,y)
+  } else
+#else
+  if (tp->use3D)
+#endif
+  {
+    if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
+      for (x=0;x<tp->ys/2-RR;x++) for(y=0;y<tp->ys/2-RR;y++)
+        THREEQUARTERSBIT(x,y)
+    if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
+      for (x=0;x<tp->ys/2-RR;x++) for(y=tp->ys/2+RR;y<tp->ys;y++)
+        THREEQUARTERSBIT(x,y)
+    if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
+      for (x=tp->ys/2+RR;x<tp->ys;x++) for(y=0;y<tp->ys/2-RR;y++)
+        THREEQUARTERSBIT(x,y)
+    if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
+      for (x=tp->ys/2+RR;x<tp->ys;x++) for(y=tp->ys/2+RR;y<tp->ys;y++)
+        THREEQUARTERSBIT(x,y)
+  }
+
+  if ((tp->images[n] = XCreateImage(MI_DISPLAY(mi), MI_VISUAL(mi),
+           1, XYBitmap, 0, (char *) data, tp->xs, tp->ys, 8, 0)) == None) {
+     (void) free((void *) data);
+     return False;
+  }
+  tp->images[n]->byte_order = MSBFirst;
+  tp->images[n]->bitmap_unit = 8;
+  tp->images[n]->bitmap_bit_order = LSBFirst;
+  return True;
+}
+
+static Bool
+create_images(ModeInfo *mi, trisstruct *tp)
+{
+  int n;
+
+  (void) create_an_image(mi, tp, BITMAPS - 1);
+  for (n = 0; n < BITMAPS - 1; n++) {
+    /* Avoid duplication of identical images. */
+    if (IS_UP(n) && IS_DOWN(n) && IS_LEFT(n) && IS_RIGHT(n))
+      tp->images[n] = tp->images[BITMAPS - 1];
+    else if (IS_LEFT_UP(n) && (IS_LEFT(n) || IS_UP(n)))
+      tp->images[n] = tp->images[n & ~LEFT_UP];
+    else if (IS_LEFT_DOWN(n) && (IS_LEFT(n) || IS_DOWN(n)))
+      tp->images[n] = tp->images[n & ~LEFT_DOWN];
+    else if (IS_RIGHT_UP(n) && (IS_RIGHT(n) || IS_UP(n)))
+      tp->images[n] = tp->images[n & ~RIGHT_UP];
+    else if (IS_RIGHT_DOWN(n) && (IS_RIGHT(n) || IS_DOWN(n)))
+      tp->images[n] = tp->images[n & ~RIGHT_DOWN];
+    else if (image_needed(n)) {
+      if (!create_an_image(mi, tp, n))
+        return False;
     }
-    tp->images[n]->byte_order = MSBFirst;
-    tp->images[n]->bitmap_unit = 8;
-    tp->images[n]->bitmap_bit_order = LSBFirst;
   }
   return True;
 }
@@ -1643,7 +1873,7 @@ refresh_tetris(ModeInfo * mi)
     for (i = 0; i < WELL_WIDTH; i++)
       for (j = 0; j < WELL_WIDTH; j++)
         if (tp->base[j][i].pmid >= 0)
-          drawSquare(mi,  tp->base[j][i].pmid, tp->base[j][i].cid, i, j);
+          drawSquare(mi, tp->base[j][i].pmid, tp->base[j][i].cid, i, j);
   } else {
     int col, row, pos;
 
@@ -1736,10 +1966,12 @@ init_tetris(ModeInfo * mi) {
 
   if (MI_IS_FULLRANDOM(mi)) {
     tp->bonus = (Bool) (LRAND() & 1);
-    tp->well = (Bool) (LRAND() & 1);
+    tp->well = !(NRAND(3));
+    tp->use3D = (Bool) (NRAND(4));
   } else {
     tp->bonus = bonus;
     tp->well = well;
+    tp->use3D = !plain;
   }
 
   if (tp->gc == None) {
@@ -1803,7 +2035,7 @@ init_tetris(ModeInfo * mi) {
     tp->width = 2 + 2 * tp->well;
   if (tp->height < 2 + 2 * tp->well)
     tp->height = 2 + 2 * tp->well;
-  if (size > MAXPIXELSIZE  &&
+  if (size >= MAXPIXELSIZE  &&
         (MINGRIDSIZE * size < tp->width && MINGRIDSIZE * size < tp->height)) {
       if (!tp->well) {
         tp->xs = size;
@@ -1823,11 +2055,8 @@ init_tetris(ModeInfo * mi) {
         ((tp->well) ? 3 * MINGRIDSIZE : MINGRIDSIZE)));
     tp->xs = tp->ys;
   }
-  if (tp->ys > MAXPIXELSIZE && !tp->well) {
-    if (tp->ys & 1) {
-      tp->ys--;
-      tp->xs = tp->ys;
-    }
+  if (tp->ys >= MAXPIXELSIZE && !tp->well) {
+    tp->xs = tp->ys = (tp->ys / 12) * 12;
     if (!create_images(mi, tp)) {
       free_images(tp);
       tp->pixelmode = True;
@@ -1849,7 +2078,7 @@ init_tetris(ModeInfo * mi) {
     for (i = 0; i < WELL_WIDTH; i++) {
       for (j = 0; j < WELL_WIDTH; j++) {
         tp->base[j][i].pmid = EMPTY;
-        tp->base[j][i].cid = 0;
+        tp->base[j][i].cid = BITMAPS - 1;
       }
     }
     for (i = 0; i < MAX_SIDES; i++) {
@@ -1995,9 +2224,9 @@ moveOne(ModeInfo *mi, move_t move)
             if (diff < 0)
               diff += WELL_WIDTH * MAX_SIDES;
             if (diff > WELL_WIDTH * (MAX_SIDES / 2))
-              move = LEFT;
+              move = MOVE_LEFT;
             else if (diff > 0)
-              move = RIGHT;
+              move = MOVE_RIGHT;
           }
         } else {
           cx = (cx - tp->xb) / tp->xs;
@@ -2005,10 +2234,10 @@ moveOne(ModeInfo *mi, move_t move)
           if (!(cx < 0 || cy < 1 || cx >= tp->ncols || cy > tp->nrows)) {
             if (cx > tp->curPolyomino.xpos +
   polytris[(int) tp->bonusNow].polyomino[tp->curPolyomino.polyomino_number].leadingEmptyWidth) {
-              move = RIGHT;
+              move = MOVE_RIGHT;
             } else if (cx < tp->curPolyomino.xpos +
   polytris[(int) tp->bonusNow].polyomino[tp->curPolyomino.polyomino_number].leadingEmptyWidth) {
-              move = LEFT;
+              move = MOVE_LEFT;
             }
           }
         }
@@ -2019,9 +2248,9 @@ moveOne(ModeInfo *mi, move_t move)
       if (LRAND() & 1)
         move = ROTATE;
       else if (LRAND() & 1)
-        move = LEFT;
+        move = MOVE_LEFT;
       else
-        move = RIGHT;
+        move = MOVE_RIGHT;
     }
   }
   if ((move == DROP) || ((move == FALL) && atBottom(mi))) {
