@@ -3,11 +3,11 @@
  *  Control-Line in a box
  *  Copyright (c) 2000 by Martin Berentsen <berentsen@sent5.uni-duisburg.de>
  *
- * speed is not in input mode
- * DEFAULT for count must be added in app-defaults
- *
- * change-log 0.4pl2 -> 0.5pl0 25/07/2000
+ * change-log 
+ * 0.4pl2 -> 0.5pl0 25/07/2000
  * -singl command line option becomes to -count num 
+ * 0.5pl0 -> 0.5pl1 02.02.2001
+ * bugs in -oldcolor and -randomstart option removed
  *
  * TODO as next:
  * - a user defined speed for every used plane 
@@ -15,10 +15,10 @@
  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)xcl.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)xcl.c    5.00 2000/11/01 xlockmore";
 #endif
 
-#define RAND(x)	(((my_random() % x )- x/2))  /* random number around 0 */
+#define RAND(x)    (((my_random() % x )- x/2))  /* random number around 0 */
 #define ROTATEDELAY 20000      /* delay for view-model rotating */
 #define STARTUPDELAY 5000      /* delay for the first calibration loop */
 #define REGULATE 25                    /* regulate delay every xx frames */
@@ -27,16 +27,16 @@ static const char sccsid[] = "@(#)xcl.c	5.00 2000/11/01 xlockmore";
 #define MAXCOUNT 30           /* define the max number of planes */
 
 #ifdef STANDALONE
-#define PROGCLASS "Life"
-#define HACK_INIT init_life
-#define HACK_DRAW draw_life
-#define life_opts xlockmore_opts
+#define PROGCLASS "Xcl"
+#define HACK_INIT init_xcl
+#define HACK_DRAW draw_xcl
+#define xcl_opts xlockmore_opts
 #define DEFAULTS "*delay: 20000 \n" \
- "*count: -3 \n"
+ "*count: 2 \n"
 #define UNIFORM_COLORS
-#include "xlockmore.h"		/* in xscreensaver distribution */
+#include "xlockmore.h"        /* in xscreensaver distribution */
 #else /* STANDALONE */
-#include "xlock.h"		/* in xlockmore distribution */
+#include "xlock.h"        /* in xlockmore distribution */
 #endif
 
 #include "xcl.h"             /* model line data file */
@@ -139,7 +139,7 @@ typedef struct
   float Ca,Cb,Cc,Sa,Sb,Sc; /* only for faster calculation */
   float Bx,By ;
   struct timeval tv1;
-  double time1, time2, time3; /* calibrate time */
+  double time1, time2, time3;        /* calibrate time */
   int drawtime;
   double alpha[MAXCOUNT];            /* direction */
   double omega_const[MAXCOUNT];      /*  constant omega speed */
@@ -170,17 +170,16 @@ static int countlines(void)
 {
   int array;
   int count = 0;
-  for(array = 0;array<LINEARRAYS;array++)
-    {
-      count += model_data2[array][0] - 1;
-    }
+  for(array = 0;array<LINEARRAYS;array++) {
+    count += model_data2[array][0] - 1;
+  }
   return(count);
 }
 
 static void um2(int X, int Y, int Z, xclstruct *dp)
 {
   float X2,X3,X4,Y1,Y3,Y4,Z1,Z2,Z4;
-
+  
   Y1 = dp->Ca * Y - dp->Sa * Z;
   Z1 = dp->Sa * Y + dp->Ca * Z;
   X2 = dp->Cb * X - dp->Sb * Z1;
@@ -209,23 +208,21 @@ static void view_3d(XSegment *xseg, xclstruct *dp)
   dp->Sb = sin(dp->Beta);
   dp->Sc = sin(dp->Gamma);
 
-  for(I = 0; I < ENDPOINTS; I++)
-    {
-      um2( model_data1 [ I * 3 + 0], model_data1 [ I * 3 + 1], model_data1 [ I * 3 + 2],dp);
-      BX [I] = dp->Bx;
-      BY [I] = dp->By;
-    }
-  for (I = 0; I < LINEARRAYS; I++)
-    {
-      for (J = 1; J < model_data2[I][0]; J++)
-	{
-	  xseg[count].x1 = (short) BX[(model_data2[I][J])-1];
-	  xseg[count].y1 = (short) BY[(model_data2[I][J])-1]+(dp->mid_y/2);
-	  xseg[count].x2 = (short) BX[(model_data2[I][J+1])-1];
-	  xseg[count].y2 = (short) BY[(model_data2[I][J+1])-1]+(dp->mid_y/2);
-	  count++;
-	}
+  for(I = 0; I < ENDPOINTS; I++) {
+    um2( model_data1 [ I * 3 + 0], model_data1 [ I * 3 + 1], 
+         model_data1 [ I * 3 + 2],dp);
+    BX [I] = dp->Bx;
+    BY [I] = dp->By;
+  }
+  for (I = 0; I < LINEARRAYS; I++) {
+    for (J = 1; J < model_data2[I][0]; J++) {
+      xseg[count].x1 = (short) BX[(model_data2[I][J])-1];
+      xseg[count].y1 = (short) BY[(model_data2[I][J])-1]+(dp->mid_y/2);
+      xseg[count].x2 = (short) BX[(model_data2[I][J+1])-1];
+      xseg[count].y2 = (short) BY[(model_data2[I][J+1])-1]+(dp->mid_y/2);
+      count++;
     }
+  }
 }
 
 static long get_color(Display *dpy,char *color,XColor *final_color)
@@ -233,24 +230,24 @@ static long get_color(Display *dpy,char *color,XColor *final_color)
   XColor cdef;
   Colormap cmap;
   cmap = DefaultColormap(dpy,DefaultScreen(dpy));
-
+  
   if (!XParseColor(dpy, cmap, color, &cdef) ||
       !XAllocColor(dpy, cmap, &cdef))
     {
       (void) fprintf(stderr, "Color \"%s\" wasn't found\n", color);
     }
   
-  if (final_color != NULL) *final_color = cdef;/* Copy the final color. */
+  if (final_color != NULL) *final_color = cdef;  /* copy the final color. */
   
   return(cdef.pixel);
 }
 
 static Bool get_GC(Display *dpy,Window win,GC *gc,long color)
 {
-  unsigned long valuemask = 0; /*ignore XGCvalues and use defaults */
+  unsigned long valuemask = 0;  /*ignore XGCvalues and use defaults */
   XGCValues values;
   unsigned int line_width = 1;
-  int line_style = LineSolid;  /* LineOnOffDash;*/
+  int line_style = LineSolid;   /* LineOnOffDash;*/
   int cap_style = CapRound;
   int join_style = JoinRound;
 
@@ -259,7 +256,7 @@ static Bool get_GC(Display *dpy,Window win,GC *gc,long color)
   XSetForeground(dpy, *gc, color);
 
   XSetLineAttributes(dpy, *gc, line_width, line_style,
-		     cap_style, join_style);
+             cap_style, join_style);
   return True;
 }
 
@@ -270,20 +267,20 @@ free_xcl(Display *display, xclstruct  *dp)
       
     for (plane = 0; plane < dp->planes; plane++) {
       if (dp->xseg[plane] != NULL) {
-	    (void) free((void *) dp->xseg[plane]);
+        (void) free((void *) dp->xseg[plane]);
         dp->xseg[plane] = NULL;
       }
-	  if (dp->xseg_old[plane] != NULL) {
-	    free(dp->xseg_old[plane]);
+      if (dp->xseg_old[plane] != NULL) {
+        free(dp->xseg_old[plane]);
         dp->xseg_old[plane] = NULL;
       }
-	  if (dp->gc[plane] != None) {
-	    XFreeGC(display, dp->gc[plane]);
+      if (dp->gc[plane] != None) {
+        XFreeGC(display, dp->gc[plane]);
         dp->gc[plane] = None;
       }
     }
     if (dp->erase_gc != None) {
-	  XFreeGC(display, dp->erase_gc);
+      XFreeGC(display, dp->erase_gc);
       dp->erase_gc = None;
     }
 }
@@ -307,7 +304,7 @@ void init_xcl(ModeInfo * mi)
   
   if (xcls == NULL) {
     if ((xcls = (xclstruct *) calloc(MI_NUM_SCREENS(mi),
-				     sizeof (xclstruct))) == NULL)
+                                     sizeof (xclstruct))) == NULL)
       return;
   }
   dp = &xcls[MI_SCREEN(mi)];
@@ -351,58 +348,58 @@ void init_xcl(ModeInfo * mi)
     
     random_pid = getpid(); /* goes here first for randomstart */
     
-    if(randomstart)
-      {
-	for(i=0;i< dp->planes; i++) { 
-	  switch(i)
-	    {
-	    case 0:
-	      dp->az[0] += (random_pid % 31) / 5.0;
-	    default:
-	      dp->az[i] = dp->az[0] + 2 * M_PI * i / (float)((dp->planes));
-	    }
-	}
+    if(randomstart) {
+      for(i=0;i< dp->planes; i++) { 
+        switch(i) {
+        case 0:
+          dp->az[0] += (random_pid % 31) / 5.0;
+          break;
+        default:
+          dp->az[i] = dp->az[0] + 2 * M_PI * i / (float)((dp->planes));
+        }
       }
+    }
     
     dp->bg = MI_BLACK_PIXEL(mi);
     
     if(MI_IS_MONO(mi))
       for(i=0;i< dp->planes; i++) { 
-	dp->planecolor[i] = MI_WHITE_PIXEL(mi);
+        dp->planecolor[i] = MI_WHITE_PIXEL(mi);
       }
     else {
       if(!oldcolor) {
-	for(i=0;i< dp->planes; i++) { 
-	  dp->planecolor[i] = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
-	}
+        for(i=0;i< dp->planes; i++) { 
+          dp->planecolor[i] = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+        }
       }
       else { /* with count >2 no so good */
-	for(i=0;i< dp->planes; i++) { 
-	  switch(i)
-	    {
-	    case 0:
-	      dp->planecolor[0] = get_color(display, (char *) "yellow",NULL);
-	    case 1:
-	      dp->planecolor[1] = get_color(display, (char *) "red",NULL);
-	    default:
-	      dp->planecolor[i] = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
-	    }
-	}
+        for(i=0;i< dp->planes; i++) { 
+          switch(i) {
+          case 0:
+            dp->planecolor[0] = get_color(display, (char *) "yellow",NULL);
+            break;
+          case 1:
+            dp->planecolor[1] = get_color(display, (char *) "red",NULL);
+            break;
+          default:
+            dp->planecolor[i] = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+          }
+        }
       }
     }
-
+    
     if(dp->erase_gc == NULL)
       if (!get_GC(display, MI_WINDOW(mi), &(dp->erase_gc),dp->bg)) {
         free_xcl(display, dp);
         return;
       }
-
+    
     dp->lines = countlines();
-
+    
     for(i=0;i< dp->planes; i++) { 
       if(dp->gc[i] == NULL)
         if (!get_GC(display, MI_WINDOW(mi), &(dp->gc[i]),
-	        dp->planecolor[i])) {
+                    dp->planecolor[i])) {
           free_xcl(display, dp);
           return;
         }
@@ -410,13 +407,13 @@ void init_xcl(ModeInfo * mi)
       
       if(dp->xseg[i] == NULL)
         if ((dp-> xseg[i] = (XSegment *) malloc(sizeof(XSegment) *
-		    dp->lines)) == NULL) {
+                                                dp->lines)) == NULL) {
           free_xcl(display, dp);
           return;
         }
       if(dp->xseg_old[i] == NULL)
         if ((dp->xseg_old[i] = (XSegment *) malloc(sizeof(XSegment) *
-            dp->lines)) == NULL) {
+                                                   dp->lines)) == NULL) {
           free_xcl(display, dp);
           return;
         }
@@ -432,19 +429,18 @@ void init_xcl(ModeInfo * mi)
       (void) printf("Lines per plane: %d\n",dp->lines);
       (void) printf("Spectator at %gm\n",spectator/1000.0);
       (void) printf("Try %g frames per Second (frametime: %dus)\n",
-	     1000000.0/frametime,frametime);
+                    1000000.0/frametime,frametime);
       (void) printf("Calibration at %d frames\n",REGULATE);
     }
   }
-
+  
   /* clear the screen */
   
   MI_CLEARWINDOW(mi);
-  /* XFillRectangle(display, MI_WINDOW(mi),dp->erase_gc,0,0,MI_WIDTH(mi),MI_HEIGHT(mi));    */
-  
-  
+    
   (void) gettimeofday(&(dp->tv1),0);
-  dp->time1 = (double)dp->tv1.tv_sec + (double)dp->tv1.tv_usec/(double)1000000;
+  dp->time1 = (double)dp->tv1.tv_sec + 
+    (double)dp->tv1.tv_usec/(double)1000000;
   
   dp->xcldelay = frametime;
 }
@@ -463,115 +459,112 @@ void draw_xcl(ModeInfo * mi)
   
   if(viewmodel == True)
     {
-      dp->Vx = 1;                 /* movement in X (width,negativ)*/
+      dp->Vx = 1;                     /* movement in X (width,negativ)*/
       if(dp->width < 900)
-	dp->Vy = 800*800/dp->width;   /*             Y (deep)*/
+        dp->Vy = 800*800/dp->width;   /*  Y (deep)*/
       else
-	dp->Vy = 800*1200/dp->width;  /*             Y (deep)*/
-      dp->Vz = -300;              /*             Z (height) */
+        dp->Vy = 800*1200/dp->width;  /*  Y (deep)*/
+      dp->Vz = -300;                  /*  Z (height) */
       dp->G = 350.0;          /* make it smaller when display is smaller */
-      
       dp->Alpha += 0.03;
       dp->Beta += 0.006;
       dp->Gamma += 0.009;
       
-      if (count != 0)
-	{
-	  XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->erase_gc,
-			dp->xseg_old[0],dp->lines);
-	  XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->gc[0],
-			dp->xseg[0],dp->lines);
-	  (void) usleep(ROTATEDELAY * 2);
-	}
+      if (count != 0) {
+        XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->erase_gc,
+                      dp->xseg_old[0],dp->lines);
+        XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->gc[0],
+                      dp->xseg[0],dp->lines);
+        (void) usleep(ROTATEDELAY * 2);
+      }
       (void) memcpy(dp->xseg_old[0],dp->xseg[0],sizeof(XSegment)*dp->lines);
       view_3d(dp->xseg[0],dp);
       count ++;
     }
-  else
-    {
-      if(automatic)
-	dp->G = dp->width / 2.1 ;
+  else {
+    if(automatic)
+      dp->G = dp->width / 2.1 ;
+    
+    for(i=0;i<dp->planes;i++) {
+      (void) memcpy(dp->xseg_old[i],dp->xseg[i],sizeof(XSegment)*dp->lines);
       
-      for(i=0;i<dp->planes;i++) {
-	
-	(void) memcpy(dp->xseg_old[i],dp->xseg[i],sizeof(XSegment)*dp->lines);
-	
-	dp->Alpha = - dp->alpha[i];
-	dp->Beta = - dp->el[i];
-	dp->Gamma = dp->az[i];
-	
-	dp->Vx = -(int)(cos(dp->az[i]) * cos(dp->el[i]) * line_length);
-	dp->Vy = spectator -(int)(sin(dp->az[i]) * cos(dp->el[i]) * line_length);
-	dp->Vz = (int)(sin(dp->el[i]) * line_length);
-
-	view_3d(dp->xseg[i],dp);
-	XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->erase_gc,dp->xseg_old[i],dp->lines);
-	XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->gc[i],dp->xseg[i],dp->lines);
-      }
+      dp->Alpha = - dp->alpha[i];
+      dp->Beta = - dp->el[i];
+      dp->Gamma = dp->az[i];
       
-      XFlush(MI_DISPLAY(mi));
-
-     /* now move all planes */
-      for(i=0;i<dp->planes;i++)
-	{
-	  dp->delta_az[i] = cos(dp->alpha[i]) * dp->omega_const[i] *
-	    frametime/1000000;
-	  dp->delta_el[i] = sin(dp->alpha[i]) * dp->omega_const[i] *
-	    frametime/1000000;
-
-	  dp->az[i] -= dp->delta_az[i];
-	  dp->el[i] -= dp->delta_el[i];
-
-	  if (dp->el[i] >= 0.0)
-	    switch (dp->turn[i])
-	      {
-	      case 0:
-		dp->turn_direction[i] *= -1;
-		dp->alpha[i] += 0.62831853 * dp->turn_direction[i];
-		dp->turn[i] ++;
-		break;
-	      case 1:
-	      case 2:
-	      case 3:
-	      case 4:
-	      case 5:
-		dp->alpha[i] += 0.62831853 * dp->turn_direction[i];
-		dp->turn[i] ++;
-		break;
-	      default:
-		dp->turn[i] ++;
-		break;
-	      }
-	  else
-	    dp->turn[i] = 0;
-	  if (dp->el[i] <= -(M_PI / 2.0))
-	    {
-	      dp->alpha[i] += M_PI;
-	      dp->az[i] += M_PI;
-	      /* el[i] = el[i] + (M_PI / 2.0) ; */
-	    }
-	  else
-	    if(dp->turn[i] == 0)
-	      dp->alpha[i] += (double)(RAND(600))  / 6283.0 * 2.0 * M_PI;
-	} /* for (i) */
-
-      count++;
-
-      (void) usleep(dp->xcldelay);
-      if((count % REGULATE) == 0)
-	{
-	  (void) gettimeofday(&(dp->tv1),0);
-	  dp->time2 = (double)dp->tv1.tv_sec + (double)dp->tv1.tv_usec/
-	    (double)1000000;
-	  dp->time3 = dp->time2 - dp->time1;
-	  dp->time1 = dp->time2;
-	  dp->drawtime = (int) (dp->time3 * (1000000/REGULATE))  
-	    - dp->xcldelay;
-	  if((dp->xcldelay = frametime - dp->drawtime) <= 0) {
-	    dp->xcldelay = 10;  /* 1 is possible xor xscreensaver mode */
-	  }
-	  if(debug == True)
-	    (void) printf("t_draw: %d, t_delay: %d\n",dp->drawtime,dp->xcldelay);
-	}
+      dp->Vx = -(int)(cos(dp->az[i]) * cos(dp->el[i]) * line_length);
+      dp->Vy = spectator - 
+        (int)(sin(dp->az[i]) * cos(dp->el[i]) * line_length);
+      dp->Vz = (int)(sin(dp->el[i]) * line_length);
+      
+      view_3d(dp->xseg[i],dp);
+      XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->erase_gc,
+                    dp->xseg_old[i],dp->lines);
+      XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->gc[i],
+                    dp->xseg[i],dp->lines);
     }
+    
+    XFlush(MI_DISPLAY(mi));
+    
+    /* now move all planes */
+    for(i=0;i<dp->planes;i++) {
+      dp->delta_az[i] = cos(dp->alpha[i]) * dp->omega_const[i] *
+        frametime/1000000;
+      dp->delta_el[i] = sin(dp->alpha[i]) * dp->omega_const[i] *
+        frametime/1000000;
+      
+      dp->az[i] -= dp->delta_az[i];
+      dp->el[i] -= dp->delta_el[i];
+      
+      if (dp->el[i] >= 0.0)
+        switch (dp->turn[i]) {
+        case 0:
+          dp->turn_direction[i] *= -1;
+          dp->alpha[i] += 0.62831853 * dp->turn_direction[i];
+          dp->turn[i] ++;
+          break;
+        case 1:
+        case 2:
+        case 3:
+        case 4:
+        case 5:
+          dp->alpha[i] += 0.62831853 * dp->turn_direction[i];
+          dp->turn[i] ++;
+          break;
+        default:
+          dp->turn[i] ++;
+          break;
+        }
+      else
+        dp->turn[i] = 0;
+      if (dp->el[i] <= -(M_PI / 2.0))
+        {
+          dp->alpha[i] += M_PI;
+          dp->az[i] += M_PI;
+          /* el[i] = el[i] + (M_PI / 2.0) ; */
+        }
+      else
+        if(dp->turn[i] == 0)
+          dp->alpha[i] += (double)(RAND(600))  / 6283.0 * 2.0 * M_PI;
+    } /* for (i) */
+    
+    count++;
+    
+    (void) usleep(dp->xcldelay);
+    if((count % REGULATE) == 0) {
+      (void) gettimeofday(&(dp->tv1),0);
+      dp->time2 = (double)dp->tv1.tv_sec + (double)dp->tv1.tv_usec/
+        (double)1000000;
+      dp->time3 = dp->time2 - dp->time1;
+      dp->time1 = dp->time2;
+      dp->drawtime = (int) (dp->time3 * (1000000/REGULATE))  
+        - dp->xcldelay;
+      if((dp->xcldelay = frametime - dp->drawtime) <= 0) {
+        dp->xcldelay = 10;  /* 1 is possible xor xscreensaver mode */
+      }
+      if(debug == True)
+        (void) printf("t_draw: %d, t_delay: %d\n",dp->drawtime,
+                      dp->xcldelay);
+    }
+  }
 }
