@@ -1103,9 +1103,21 @@ checkPasswd(char *buffer)
 #endif
 #endif
 	done = ((authenticate_user((struct passwd *) getpwnam(user),
-				   buffer, NULL) >= 0) || (allowroot &&
-			 (authenticate_user((struct passwd *) getpwnam(ROOT),
-					    buffer, NULL) >= 0)));
+				   buffer, NULL) >= 0);
+	if (!done && allowroot &&
+		 (authenticate_user((struct passwd *) getpwnam(ROOT),
+					    buffer, NULL) >= 0))) {
+		done = True;
+		if (!*buffer)
+			/*
+			 * root has no password, don't let him in...
+			 */
+			done = False;
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+		else
+			syslog(SYSLOG_NOTICE, "%s: %s unlocked screen", ProgramName, ROOT);
+#endif
+	}
 #ifdef HAVE_SETEUID
 	(void) setegid(rgid);
 #else
@@ -1156,8 +1168,18 @@ checkPasswd(char *buffer)
 		else if (rlc != NULL) {
 			style = login_getstyle(rlc, NULL, "auth-xlock");
 			if (auth_response(ROOT, rlc->lc_class, style,
-					  "response", NULL, "", buffer) > 0)
+					  "response", NULL, "", buffer) > 0) {
 				done = True;
+				if (!*buffer)
+	 				/*
+					 * root has no password, don't let him in...
+					 */
+					done = False;
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+				else
+					syslog(SYSLOG_NOTICE, "%s: %s unlocked screen", ProgramName, ROOT);
+#endif
+			}
 		}
 	}
 #if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
@@ -1229,16 +1251,16 @@ checkPasswd(char *buffer)
 	(void) printf("buffer=%s, encrypt=%s, userpass=%s\n", buffer,
 		      (char *) crypt(buffer, userpass), userpass);
 #endif
-	if (!done) {
-		done = (allowroot &&
-			!strcmp((char *) crypt(buffer, rootpass), rootpass));
-		if (allowroot && !*rootpass && !*buffer)
+	if (!done && allowroot && !strcmp((char *) crypt(buffer, rootpass),
+			 rootpass)) {
+		done = True;	
+		if (!rootpass || !*buffer)
 			/*
 			 * root has no password, don't let him in...
 			 */
 			done = False;
 #if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
-		if (done)
+		else
 			syslog(SYSLOG_NOTICE, "%s: %s unlocked screen", ProgramName, ROOT);
 #endif
 	}
@@ -1559,7 +1581,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 		return True;
 	return False;
 }
-#endif /* HAVE_KERB4 */
+#endif /* HAVE_KRB4 */
 
 #ifdef HAVE_KRB5
 /*-
@@ -1568,7 +1590,7 @@ krb_check_password(struct passwd *pwd, char *pass)
  * that if someone is locked for a long time, their credentials could expire,
  * so xlock must be able to get a new ticket.  -- dah <rodmur@ecst.csuchico.edu>
  */
-#define KRB5_DEFAULT_OPTIONS 0
+#define KRB5_DEFAULT_OPTIONS 		0
 #define KRB5_DEFAULT_LIFE 60*60*10	/* 10 hours */
 static int
 krb_check_password(struct passwd *pwd, char *pass)
@@ -1586,6 +1608,8 @@ krb_check_password(struct passwd *pwd, char *pass)
 	krb5_address **addrs = (krb5_address **) 0;
 	krb5_preauthtype *preauth = NULL;
 
+        krb5_deltat rlife = 0;
+
 	krb5_data   tgtname =
 	{
 		0,
@@ -1618,7 +1642,6 @@ krb_check_password(struct passwd *pwd, char *pass)
 	}
 	(void) memset((char *) &my_creds, 0, sizeof (my_creds));
 
-	my_creds.client = me;
 
 	if ((code = krb5_build_principal_ext(kcontext, &server,
 				      krb5_princ_realm(kcontext, me)->length,
@@ -1630,12 +1653,50 @@ krb_check_password(struct passwd *pwd, char *pass)
 		com_err(ProgramName, code, "while building server name");
 		return False;
 	}
+
+
+	my_creds.client = me;
 	my_creds.server = server;
 
 	my_creds.times.starttime = 0;
-	my_creds.times.endtime = now + lifetime;
+
 	my_creds.times.renew_till = 0;
 
+	/*
+        ** find our default ticket, and copy out renewable, forwardable info.
+        */
+	{
+	krb5_cc_cursor cur;
+	krb5_creds creds;
+
+	if ((code = krb5_cc_start_seq_get(kcontext, ccache, &cur))) {
+	    com_err(ProgramName, code, "while starting to retrieve tickets");
+	} else {
+
+	    while (!(code = krb5_cc_next_cred(kcontext, ccache, &cur, &creds))) {
+		if (creds.server->length == 2 &&
+		        strcmp(creds.server->realm.data, me->realm.data) == 0 &&
+		        strcmp((char *)creds.server->data[0].data, "krbtgt") == 0 &&
+		        strcmp((char *)creds.server->data[1].data, me->realm.data) == 0 &&
+		        creds.times.endtime > now) {
+		    /*
+                    ** this is the krbtgt token to steal options from...
+		    */
+	    
+		    if (creds.times.renew_till) {
+		       options |= KDC_OPT_RENEWABLE;
+		       my_creds.times.renew_till = creds.times.renew_till;
+		    }
+		    if (creds.ticket_flags & TKT_FLG_FORWARDABLE) {
+		       options |= KDC_OPT_FORWARDABLE;
+		    }
+		}
+		krb5_free_cred_contents(kcontext, &creds);
+	    }
+        }
+
+	}
+
 	if (strlen(pass) == 0)
 		(void) strcpy(pass, "*");
 	/* if pass is NULL, krb5_get_in_tkt_with_password will prompt with
