@@ -80,6 +80,10 @@ static const char sccsid[] = "@(#)resource.c	4.08 98/08/04 xlockmore";
 # endif
 #endif
 
+#ifdef __sgi
+#undef offsetof
+#endif
+
 #ifndef offsetof
 # define offsetof(s,m) ((char*)(&((s *)0)->m)-(char*)0)
 #endif
@@ -423,6 +427,14 @@ static XrmOptionDescRec genTable[] =
 	{(char *) "-wireframe", (char *) ".wireframe", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+wireframe", (char *) ".wireframe", XrmoptionNoArg, (caddr_t) "off"},
 
+#ifdef USE_GL
+	{(char *) "-showfps", (char *) ".showfps", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+showfps", (char *) ".showfps", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-fpstop", (char *) ".fpstop", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+fpstop", (char *) ".fpstop", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-fpsfont", (char *) ".fpsfont", XrmoptionSepArg, (caddr_t) NULL},
+#endif
+
 	{(char *) "-use3d", (char *) ".use3d", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+use3d", (char *) ".use3d", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-delta3d", (char *) ".delta3d", XrmoptionSepArg, (caddr_t) NULL},
@@ -610,6 +622,11 @@ static OptionStruct opDesc[] =
 	{(char *) "-timeout seconds", (char *) "number of seconds before password times out"},
 	{(char *) "-font fontname", (char *) "font to use for password prompt"},
 	{(char *) "-planfont fontname", (char *) "font to use for plan message"},
+#ifdef USE_GL
+	{(char *) "-/+showfps", (char *) "turn on/off display of FPS"},
+	{(char *) "-/+fpstop", (char *) "turn on/off FPS display on top of window"},
+	{(char *) "-fpsfont fontname", (char *) "font to use for FPS display"},
+#endif
 #ifdef USE_MB
 	{(char *) "-fontset fontsetname", (char *) "fontset to use for Xmb..."},
 #endif
@@ -630,7 +647,7 @@ static OptionStruct opDesc[] =
 	{(char *) "-/+wireframe", (char *) "turn on/off wireframe"},
 
 	{(char *) "-/+use3d", (char *) "turn on/off 3d view"},
-   {(char *) "-delta3d value", (char *) "space between the center of your 2 eyes for 3d mode"},
+        {(char *) "-delta3d value", (char *) "space between the center of your 2 eyes for 3d mode"},
 	{(char *) "-none3d color", (char *) "color to be used for null in 3d mode"},
 	{(char *) "-right3d color", (char *) "color to be used for the right eye in 3d mode"},
 	{(char *) "-left3d color", (char *) "color to be used for the left eye in 3d mode"},
@@ -734,6 +751,11 @@ Bool        usefirst;
 Bool        verbose;
 Bool        remote;
 
+#ifdef USE_GL
+Bool        fpsTop;
+Bool        showfps;
+char       *fpsfontname;
+#endif
 static char *visualname;
 int         VisualClassWanted;
 
@@ -856,6 +878,11 @@ static argtype genvars[] =
     {(caddr_t *) & mousemotion, (char *) "mousemotion", (char *) "MouseMotion", (char *) "off", t_Bool},
 	{(caddr_t *) & mono, (char *) "mono", (char *) "Mono", (char *) "off", t_Bool},
 	{(caddr_t *) & sound, (char *) "sound", (char *) "Sound", (char *) "off", t_Bool},
+#ifdef USE_GL
+	{(caddr_t *) & showfps, (char *) "showfps", (char *) "ShowFps", (char *) "off", t_Bool},
+	{(caddr_t *) & fpsTop, (char *) "fpstop", (char *) "FpsTop", (char *) "on", t_Bool},
+	{(caddr_t *) & fpsfontname, (char *) "fpsfont", (char *) "FpsFont", (char *) DEF_FONT, t_String},
+#endif
     {(caddr_t *) & timeelapsed, (char *) "timeelapsed", (char *) "TimeElapsed", (char *) "off", t_Bool},
 	{(caddr_t *) & usefirst, (char *) "usefirst", (char *) "UseFirst", (char *) "on", t_Bool},
 	{(caddr_t *) & verbose, (char *) "verbose", (char *) "Verbose", (char *) "off", t_Bool},
