@@ -1,4 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
+
 static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 
 #endif
@@ -22,6 +23,13 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
   3. Integer and floating point and string input.
  */
 
+/* 
+ * 01/2001
+ * I'm confused because the source is old and really not well programming 
+ * I put boolean option and option in the menubar
+ * I have had the option.h 
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/signal.h>
@@ -53,13 +61,17 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 #include <Xm/ToggleB.h>
 #include <Xm/List.h>
 #include <Xm/PushB.h>
+#include <Xm/CascadeB.h>
 #include <Xm/Form.h>
 #include <Xm/Label.h>
+#include <Xm/Separator.h>
 
 #if USE_XMU
 #include <X11/Xmu/Editres.h>
 #endif
 
+#include "option.h"
+
 #ifdef SunCplusplus
         extern int  kill(pid_t, int);
         extern pid_t  wait(int *);
@@ -75,24 +87,18 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 #include "bitmaps/m-xlock.xbm"	/* icon bitmap */
 
 /* like an enum */
-#define  LAUNCH	0
-#define  ROOT   1
-#define  WINDOW 2
-#define  EXIT   3
-
-/* number of buttons, toggles, and string options */
-#define  PUSHBUTTONS  4
-#define  TOGGLES  9
-#define  OPTIONS  8
-
-#define XLOCK "xlock"
-#define WINDOW_WIDTH 160
-#define WINDOW_HEIGHT 100
-#define WINDOW_GEOMETRY "160x100"
+enum ACTION {
+  LAUNCH=0,
+  ROOT,   
+  WINDOW 
+	};
 
 /* extern variable */
 extern char *c_Options[OPTIONS];
 extern Widget Menuoption;
+extern Widget Toggles[TOGGLES];
+
+extern char *r_Toggles[];
 
 static pid_t numberprocess = -1;	/* PID of xlock */
 
@@ -100,11 +106,13 @@ static pid_t numberprocess = -1;	/* PID of xlock */
 Widget      toplevel;
 static Widget Labelxlock;
 
-static Widget ScrolledListModes, PushButtons[PUSHBUTTONS], Toggles[TOGGLES];
+static Widget ScrolledListModes, PushButtons[PUSHBUTTONS];
 
  /*Resource string */
 #include "modes.h"
 
+static XmStringCharSet char_set = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
+
 /* some resources of buttons and toggles not really good programming :( */
 
 static char *r_PushButtons[PUSHBUTTONS] =
@@ -112,20 +120,6 @@ static char *r_PushButtons[PUSHBUTTONS] =
 	(char *) "Launch",
 	(char *) "In Root",
 	(char *) "In Window",
-	(char *) "Exit"
-};
-
-static char *r_Toggles[TOGGLES] =
-{
-	(char *) "mono",
-	(char *) "nolock",
-	(char *) "remote",
-	(char *) "allowroot",
-	(char *) "enablesaver",
-	(char *) "allowaccess",
-	(char *) "grabmouse",
-	(char *) "echokeys",
-	(char *) "usefirst"
 };
 
 char       *r_Options[OPTIONS] =
@@ -136,13 +130,22 @@ char       *r_Options[OPTIONS] =
 	(char *) "validate",
 	(char *) "invalid",
 	(char *) "geometry",
-	(char *) "font",
-	(char *) "program"
+	/*(char *) "font",
+	(char *) "program"*/
 };
 
 static int  numberinlist = 0;
 
 /* CallBack */
+
+void exitcallback(Widget w, XtPointer client_data, XtPointer call_data) {
+			if (numberprocess != -1) {
+				(void) kill(numberprocess, SIGKILL);
+				/*(void) wait(&n);*/
+			}
+			exit(0);
+	}
+
 static void
 f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 {
@@ -184,13 +187,6 @@ f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 		case ROOT:
 			(void) strcat(command, "-inroot ");
 			break;
-		case EXIT:
-			if (numberprocess != -1) {
-				(void) kill(numberprocess, SIGKILL);
-				(void) wait(&n);
-			}
-			exit(0);
-			break;
 	}
 	(void) strcat(command, "-mode ");
 	(void) strcat(command, LockProcs[numberinlist].cmdline_arg);
@@ -237,50 +233,56 @@ f_ScrolledListModes(Widget w, XtPointer client_data, XtPointer call_data)
 }
 
 /* Setup Widget */
+
+/**********************/
+
 static void
 Setup_Widget(Widget father)
 {
 	Arg         args[15];
 	int         i, ac;
-	Widget      Row, PushButtonRow, TogglesRow, ModeLabel, SwitchLabel;
+	Widget      Row, PushButtonRow, TogglesRow, ModeLabel, SwitchLabel,exitwidgetB;
 	char        string[160];
 	XmString    label_str;
 	XmString    TabXmStr[numprocs];
 
-	ac = 0;
-	XtSetArg(args[ac], XmNrightAttachment, XmATTACH_FORM);
-	ac++;
-	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_FORM);
-	ac++;
-	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
-	ac++;
-	SwitchLabel = XmCreateLabel(father, (char *) "Switches", args, ac);
-	XtManageChild(SwitchLabel);
-	ac = 0;
-	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
-	ac++;
-	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_FORM);
-	ac++;
-	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM);
-	ac++;
-	ModeLabel = XmCreateLabel(father, (char *) "Modes", args, ac);
-	XtManageChild(ModeLabel);
-	ac = 0;
-	XtSetArg(args[ac], XmNwidth, WINDOW_WIDTH);
-	ac++;
-	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
-	ac++;
-	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_FORM);
-	ac++;
-	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_WIDGET);
-	ac++;
-	XtSetArg(args[ac], XmNleftWidget, ModeLabel);
-	ac++;
-	Labelxlock = XmCreateLabel(father, (char *) "Window", args, ac);
-	XtManageChild(Labelxlock);
-
 /* buttons in the bottom */
 	ac = 0;
+	XtSetArg(args[ac], XmNorientation, XmHORIZONTAL);
+	ac++;
+	XtSetArg(args[ac], XmNrightAttachment, XmATTACH_FORM);
+	ac++;
+	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM);
+	ac++;
+	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_FORM);
+	ac++;
+
+	Menuoption = XmCreateMenuBar(father, (char *) "MenuBar", args,ac);
+	XtManageChild(Menuoption);
+
+	ac = 0;
+	XtSetArg(args[ac], XmNwidth, WINDOW_WIDTH);
+	ac++;
+	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
+	ac++;
+
+	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
+	XtSetArg(args[ac], XmNtopWidget, Menuoption); ac++;
+
+#if 0
+	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+
+	XtSetArg(args[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+
+	XtSetArg(args[ac], XmNleftWidget, ModeLabel); ac++;
+#endif
+
+	Labelxlock = XmCreateLabel(father, (char *) "Window", args, ac);
+	XtManageChild(Labelxlock);
+
+/* buttons in the bottom */
+ ac = 0;
+
 	XtSetArg(args[ac], XmNorientation, XmHORIZONTAL);
 	ac++;
 	XtSetArg(args[ac], XmNrightAttachment, XmATTACH_FORM);
@@ -291,9 +293,6 @@ Setup_Widget(Widget father)
 	ac++;
 	PushButtonRow = XmCreateRowColumn(father, (char *) "PushButtonRow", args, ac);
 
-	Menuoption = XmCreateMenuBar(PushButtonRow, (char *) "MenuBar", NULL, 0);
-	XtManageChild(Menuoption);
-
 	for (i = 0; i < PUSHBUTTONS; i++) {
 		ac = 0;
 #ifndef USE_MB
@@ -310,6 +309,24 @@ Setup_Widget(Widget father)
 			      (XtPointer) i);
 		XtManageChild(PushButtons[i]);
 	}
+/* 
+ *  Fro the exit button
+ */
+		ac = 0;
+#ifndef USE_MB
+		label_str = XmStringCreate("Exit", XmSTRING_DEFAULT_CHARSET);
+		XtSetArg(args[ac], XmNlabelString, label_str);
+		ac++;
+#endif
+		exitwidgetB = XmCreatePushButton(PushButtonRow, "Exit",
+						    args, ac);
+#ifndef USE_MB
+		XmStringFree(label_str);
+#endif
+		XtAddCallback(exitwidgetB, XmNactivateCallback, exitcallback, (XtPointer) NULL);
+		XtManageChild(exitwidgetB);
+
+	
 	XtManageChild(PushButtonRow);
 
 /* list and toggles in row like that (row(list)(TogglesRow(toggles...))) */
@@ -326,29 +343,26 @@ Setup_Widget(Widget father)
 	ac++;
 	XtSetArg(args[ac], XmNbottomWidget, PushButtonRow);
 	ac++;
-	XtSetArg(args[ac], XmNorientation, XmHORIZONTAL);
-	ac++;
-	Row = XmCreateRowColumn(father, (char *) "Row", args, ac);
-
-	for (i = 0; i < (int) numprocs; i++) {
-		(void) sprintf(string, "%-14s%s", LockProcs[i].cmdline_arg,
-			       LockProcs[i].desc);
-		TabXmStr[i] = XmStringCreate(string, XmSTRING_DEFAULT_CHARSET);
-	}
-	ac = 0;
 	XtSetArg(args[ac], XmNitems, TabXmStr);
 	ac++;
 	XtSetArg(args[ac], XmNitemCount, numprocs);
 	ac++;
 	XtSetArg(args[ac], XmNvisibleItemCount, 10);
 	ac++;
-	ScrolledListModes = XmCreateScrolledList(Row, (char *) "ScrolledListModes",
+
+
+	for (i = 0; i < (int) numprocs; i++) {
+		(void) sprintf(string, "%-14s%s", LockProcs[i].cmdline_arg,
+			       LockProcs[i].desc);
+		TabXmStr[i] = XmStringCreate(string, XmSTRING_DEFAULT_CHARSET);
+	}
+		ScrolledListModes = XmCreateScrolledList(father, (char *) "ScrolledListModes",
 						 args, ac);
 	XtAddCallback(ScrolledListModes, XmNbrowseSelectionCallback,
 		      f_ScrolledListModes, NULL);
 	XtManageChild(ScrolledListModes);
 
-	TogglesRow = XmCreateRowColumn(Row, (char *) "TogglesRow", NULL, 0);
+/*	TogglesRow = XmCreateRowColumn(Row, (char *) "TogglesRow", NULL, 0);
 	for (i = 0; i < TOGGLES; i++) {
 #ifndef USE_MB
 		ac = 0;
@@ -365,6 +379,8 @@ Setup_Widget(Widget father)
 	XtManageChild(TogglesRow);
 
 	XtManageChild(Row);
+*/
+
 	for (i = 0; i < (int) numprocs; i++) {
 		XmStringFree(TabXmStr[i]);
 	}
